// Source code for PSPedict.
// Copyright A. Moloney (PixelDragon) 2009.
// Copyright Batanen (from Qj.net forums) 2010.
// *********************************************************************
// * This program is free software: you can redistribute it and/or modify
// * it under the terms of the GNU General Public License as published by
// * the Free Software Foundation, either version 3 of the License, or
// * (at your option) any later version.
// *
// * This program is distributed in the hope that it will be useful,
// * but WITHOUT ANY WARRANTY; without even the implied warranty of
// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// * GNU General Public License for more details.
// *
// * You should have received a copy of the GNU General Public License
// * along with this program.  If not, see <http://www.gnu.org/licenses/>.
// *
// * Version History
// * ===============
// * V0.4 5/8/10 (by Batanen)
// * - Added Kanji Search by SKIP count and Stroke count or both. Works with English input mode. Start
// *   with #. Then write the SKIP code or stroke count. E.g. #1 12 11 (SKIP 1-12-11) or #3 
// *   (all kanjis that has a stroke count of 3). You may also use 0 as a meaning of anything. 
// *   E.g. #0 will list all kanjis and #1 0 0 5 all kanjis that has a SKIP pattern of 1 and 
// *   stroke count of 5.
// * - A new option to hide kanji / kana / English description.
// * - Added a custom dictionary for the user.
// * - Studylist randomization. Press select while in studylist. Not a perfect algorithm but good enough.
// * V0.3 6/11/09 
// * - Included support for Tanaka Corpus(TC) example sentences. The Edict files(s) are pre-prepared
// *   by another program to include file indexes for corresponding TC examples. LoadEdict() has been
// *   updated to read these file position number. There is a limit of three example sentences at
// *   present. If an example exists, it is in the format, #NNNNNNN,NNNNNNN,NNNNNNN#.
// *   Two new structs have been included to account for TC data, uTC and uTCWord.
// * - Corrected a dumbass error where I neglected to notice that I used a u8 to store the
// *   return value of strlen, causing incomplete strings. Probably not noticed as only affected
// *   maybe 6 different entries.
// *
// * V0.2A 2/11/09
// * - Bug Fix #01. When pressing X button on an empty Studylist or Foundset, PSP hangs. Fixed.
// * 
// * V0.2 26/10/09.
// * - Added ini file support via functions, INI_Check(), INI_SetDefaults(),
// *   INI_Load(), INI_Create() and INI_Save().
// * - Update functions find_SJIS_string() and find_kanji_string() to enable
// *   match_whole. This merely searches for the exact string. Changed case GS_OPTIONS
// *   in CheckButtons() and text message in ScreenUpdate().
// * - Fixed how screen flow is navigated. It is a bit more natural now.
// * - Now deletion of found set files has been implimented.
// * - Other EDICT files now supported. These are hard coded. Note that file names, study lists,
// *   found sets have changed (slightly) in most cases. As the kanji is much longer in some of these
// *   lists, have had to include extra checking in the list view, and implimented screen_kanji to
// *   handle more than 1 line of text.
// *
// * V0.1
// * - Initial release code.
// *********************************************************************

#include <pspkernel.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <pspgu.h>
#include <pspgum.h>
#include <pspdisplay.h>

#include <oslib/oslib.h>
#include "main.h"

PSP_MODULE_INFO("PSPedict", 0, 1, 0);
PSP_MAIN_THREAD_ATTR(PSP_THREAD_ATTR_USER | PSP_THREAD_ATTR_VFPU);
//PSP_HEAP_SIZE_KB(16*1024);

#define VER_NUM_STR "v0.4 by Batanen\0"

#define MEDEBUG 0

// Colors
enum colors {
    RED =		0xFF0000FF, //(0xFFBBGGRR)
    GREEN =		0xFF00FF00,
    BLUE =		0xFFFF0000,
    WHITE =		0xFFFFFFFF,
    LITEGRAY =	0xFFBFBFBF,
    GRAY =		0xFF7F7F7F,
    DARKGRAY =	0xFF3F3F3F,
	CYAN =		0xFFFFFF00,
	YELLOW =	0xFF00FFFF,
	MAGENTA =	0xFFFF00FF, // Be careful, this is the colour key too!
    BLACK =		0xFF020202
};

//Global variables
int game_status = GS_INIT, prev_game_status = GS_INIT;
int gs_level[MAX_LEVEL] = {GS_PLAYING,-1,-1,-1,-1}; // New 24/10/09
int cur_gs_level = GS_LEVEL_01; // New 24/10/09

char DataFilePath[] = "data/\0";
FILE *fKanji = NULL;
char *fKanjiName[MAX_EDICT] = {"edict.sjs\0", "computer terms.sjs\0", "legal terms.sjs\0", "life sciences.sjs\0", "custom.sjs\0"};
int edict_max[MAX_EDICT] = {MAX_LINES, MAX_LINES_COMP, MAX_LINES_LEGAL, MAX_LINES_LIFE, MAX_LINES_CUSTOM};
int cur_edict = EDICT_ORIG;
int cur_edict_index = EDICT_ORIG; // Used to move between files.
FILE *fKanjiDict = NULL;
char fKanjiDictName[] = "data/kanjidic_stroke.sjs\0"; // (SJIS file)
char fTCName[] = "data/examples.sjs\0"; // Added 7/11/09
int show_examples = TRUE;

/*Added by Batanen*/
enum KANJILISTING kanjiListing = NONE;
short int strokeCount = 0; // number of stroke counts searched
int userSkipCode[4] = {0,0,0,0}; // Searched SKIP
bool kanjiHiding = FALSE; // Don't hide kanji
bool kanaHiding = FALSE; // Don't hide kana
bool descHiding = FALSE; // Don't hide description

aacursor cursor = {AA_SCREEN_WIDTH / 2, AA_SCREEN_HEIGHT / 2, CURSOR_X , CURSOR_Y, CURSOR_WIDTH, CURSOR_HEIGHT, FALSE, FALSE, 0, 0};
aacursor mnu_icon[MAX_MENU_ICON] = {{MENU_ICON_EXIT_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_EXIT_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
                                    {MENU_ICON_SAVE_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_SAVE_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
                                    {MENU_ICON_SEARCH_X,			MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_SEARCH_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
                                    {MENU_ICON_LOAD_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_LOAD_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
                                    {MENU_ICON_OPT_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_OPT_SRC_Y,				MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
                                    {MENU_ICON_FIND_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_FIND_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
									{MENU_ICON_KDICT_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_KDICT_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
									{MENU_ICON_STUDY_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_STUDY_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE, FALSE, 0, 0},
									// Next two have enabled defaulted to FALSE
									{MENU_ICON_STUDY_ADD_X,			MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_STUDY_ADD_SRC_Y,		MENU_ICON_WIDTH, MENU_ICON_HEIGHT, FALSE, FALSE, 0, 0},
									{MENU_ICON_STUDY_MINUS_X,		MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_STUDY_MINUS_SRC_Y,		MENU_ICON_WIDTH, MENU_ICON_HEIGHT, FALSE, FALSE, 0, 0},
									{MENU_ICON_STUDY_MINUS_ALL_X,	MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_STUDY_MINUS_ALL_SRC_Y,	MENU_ICON_WIDTH, MENU_ICON_HEIGHT, FALSE, FALSE, 0, 0},
									{MENU_ICON_EDICT_X,				MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_EDICT_SRC_Y,			MENU_ICON_WIDTH, MENU_ICON_HEIGHT, FALSE, FALSE, 0, 0},
									{MENU_ICON_EDICT_LOAD_X,		MENU_ICON_Y, MENU_ICON_SRC_X , MENU_ICON_EDICT_LOAD_SRC_Y,		MENU_ICON_WIDTH, MENU_ICON_HEIGHT, TRUE,  FALSE, 0, 0}}; // New 24/10/09

aacursor scrollbar[2] = {{SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_SRC_X , SCROLLBAR_SRC_Y, SCROLLBAR_WIDTH, SCROLLBAR_HEIGHT, FALSE, FALSE, 0, 0}, {SCROLLBAR_DOT_X, SCROLLBAR_DOT_Y, SCROLLBAR_DOT_SRC_X , SCROLLBAR_DOT_SRC_Y, SCROLLBAR_DOT_WIDTH, SCROLLBAR_DOT_HEIGHT, FALSE, FALSE, 0, 0}};

float kanji_size = DEF_KANJI_SIZE; // Probably not going to be changed.

// Menu
int menu_status = M_CLOSED;
int menu_counter = 0;

// Options
int search_type = OPT_SEARCH_ANY; // Doesn't apply to english String - Eng string can search Whole Word.
int match_whole = FALSE;
int sort_type = OPT_SORT_ORIG;

// kanji (edict) array
ukanji kanji[MAX_LINES] = {{NULL, NULL, NULL, {-1,-1,-1}},};// Changed 7/11/09.
int kanji_index = 0;
//int show_mode = 0;
int lines_read = 0;
//unsigned short *ucode = NULL;
u8 *ucode = NULL;
char buffer[MAX_DESCRIP];

// Kanjidict array
ukanjidict kanjidict[MAX_KANJIDICT];
int kdict_index = 0;
int kdict_lines_read = 0;
int kdict_lines_read_backup = 0; // Added by Batanen

// Kdict Showing stuff (aka Kanji Pop-up)
unsigned int cursor_tick = 0;
int show_kdict = FALSE;
int kdict_shown = TRUE; // Won't show the pop-up upon startup
int show_kdict_tick = 0;
int show_kdict_x = 0, show_kdict_y = 0;
uscreenkanji screenkanji[MAX_SHOWN_F]; // Used to store kanji shown on the screen.
int kchar_width = 0;
u8 search_kanji[3] = "\0";
int kdict_search_index = 0;
// Store information re position on DrawBuffer
int kanji_popup_end_y = 0;
int kanji_popup_cur_y = 0;// This gets moved when scrolling Kanji Pop-up.

// Used for OSK
char osk_text[MAX_DESCRIP] = "\0";
char prev_osk_text[MAX_DESCRIP] = "\0";
char prev_osk_text_eng[MAX_DESCRIP] = "\0";
unsigned int osk_textUCS2[MAX_DESCRIP] = {0,};
unsigned short prev_osk_textUCS2[MAX_DESCRIP] = {0,};
int osk_language = OSK_ENGLISH; // Eng=1;Japanese=0
int prev_osk_language = OSK_ENGLISH; // Eng=1;Japanese=0

// Used for Found set
int found_list[MAX_FOUND] = {-1,};
int found_index = 0;// Used for printing list
int found_count = 0;
int found_more = FALSE;
int is_finding = FALSE;

// Study List
ustudylistel studylist[MAX_NUM_STUDY_LIST][MAX_STUDY_LIST] = {{{-1,0},},};
int studylist_index[MAX_NUM_STUDY_LIST] = {0,};// Used for printing list
int studylist_count[MAX_NUM_STUDY_LIST] = {0,};
int studylist_enabled = FALSE;// If something goes wrong, don't use it.
//char *studylist_fname[MAX_NUM_STUDY_LIST] = {"StudyList1.pel\0", "StudyList2.pel\0", "StudyList3.pel\0", "StudyList4.pel\0"};
char *studylist_fname[MAX_EDICT][MAX_NUM_STUDY_LIST] = {{"StudyList1.pel\0",		"StudyList2.pel\0",			"StudyList3.pel\0",			"StudyList4.pel\0"},
														{"CompStudyList1.pel\0",	"CompStudyList2.pel\0",		"CompStudyList3.pel\0",		"CompStudyList4.pel\0"},
														{"LegalStudyList1.pel\0",	"LegalStudyList2.pel\0",	"LegalStudyList3.pel\0",	"LegalStudyList4.pel\0"},
														{"LifeStudyList1.pel\0",	"LifeStudyList2.pel\0",		"LifeStudyList3.pel\0",		"LifeStudyList4.pel\0"}, 
														{ "CustomStudyList1.pel\0", "CustomStudyList2.pel\0", "CustomStudyList3.pel\0", "CustomStudyList4.pel\0"} };
char study_list_dirname[] = "study_lists/\0";
int cur_studylist = 0; // Workout which list we are on 0 - 3

// Screen Text Vert Scrolling - Only used for GS_SHOW_SEL
uscreentextel screen_text[MAX_SCREEN_TEXT] = {{0,0,LINE_SPACE_IF_Y,FONT_LATIN,NULL,0,0,0,0},};
uscreentextel tmp_screen_text = {0,0,LINE_SPACE_IF_Y,FONT_LATIN,NULL,0,0,0,0};
int st_max_height = MAX_ST_HEIGHT;
int st_index = 0;
int st_count = 0;
int st_can_scroll = TRUE;

// Saving/Loading files.
char *fname[MAX_SAVED_FILE_INDEX] = {NULL,};
int num_files = 0;
int fname_index = 0; // Must have at least one file name.
char conv_search_fname[256] = "\0"; // May not be required.
SceIoDirent mydir;
char saved_finds_file_path[] = "saved_finds/\0";
int create_file_list_errno = RTN_OK;

// Inifile
char fIniName[] = "data/PSPedict.ini\0";

// Debugging
OSL_MEMSTATUS ram;
uTC tmpTC = {NULL, NULL, 0, -1, 0, NULL};
//int max_kanji_count = 0, max_kana_count = 0, max_descrip_count = 0;
//int max_kanji_line_num = 0, max_kana_line_num = 0, max_descrip_line_num = 0;
//int cur_fr = 0;
//int sb_sml_x = SCROLLBAR_SML_X;

// Main drawing elements
OSL_FONT *ltn0 = NULL;
OSL_FONT *jpn0 = NULL;
OSL_IMAGE *bkg = NULL;
OSL_IMAGE *bkg2 = NULL;
OSL_IMAGE *bkg3 = NULL;
//	OSL_IMAGE *bkg4 = oslLoadImageFilePNG("bkg4.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_5650);
OSL_IMAGE *spr_cur = NULL;
OSL_IMAGE *drawBuf = NULL;


/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
    osl_quit = 1;
    return 0;
}

int initOSLib(){
    oslInit(0);
    oslInitGfx(OSL_PF_5551, 1);
    oslInitAudio();
    oslSetQuitOnLoadFailure(1);
    oslSetKeyAutorepeatInit(40);
    oslSetKeyAutorepeatInterval(6);
	//Sets the transparency color (pink)
	oslSetTransparentColor(RGB(255,0,255));
    return 0;
}

int endOSLib(){
    oslEndGfx();
    osl_quit = 1;
    return 0;
}

// ########
// # MAIN #
// ########
int main(){
	int skip = 0;
	int i = 0, rtn = 0;
	OSL_CONTROLLER *keys;
	//OSL_MEMSTATUS ram;

	// Initialise basic framework.
	rtn = doInit();
	if (rtn != RTN_OK){
		oslDebug( "Error. doInit returned %i", rtn );
		oslIntraFontShutdown();
		endOSLib();
		sceKernelExitGame();
		return 0;
	}

	// Initialise extra stuff.
	rtn = doInit2();
	if (rtn != RTN_OK){
		oslDebug( "Error. doInit2 returned %i", rtn );
		oslIntraFontShutdown();
		endOSLib();
		sceKernelExitGame();
		return 0;
	}

	game_status = GS_SETUP;

	skip = 0;

	oslSetFont(jpn0);
	oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	kchar_width = oslGetStringWidth(kanjidict[0].kanji);
	//oslDebug("kchar_width = %d", kchar_width );
	oslSetFont(ltn0);

	//// For Debugging
	//ram = oslGetRamStatus();
	//oslDebug("Mem Debugging\nbytes available=%dkb, %0.2fmb.\nmaxBlockSize = %dkb, %0.2fmb.", ram.maxAvailable/1024, ((float)ram.maxAvailable/1024)/1024, ram.maxBlockSize/1024, ((float)ram.maxBlockSize/1024)/1024);

	// Done here as CheckButtons() run after ScreenUpdate()
	keys = oslReadKeys();

	while(!osl_quit){
		if (!skip){
			oslStartDrawing();
			ScreenUpdate();
			oslEndDrawing();
		}// End if (!skip)

		CheckButtons();
		oslEndFrame();
		skip = oslSyncFrame();
	}// End while(!osl_quit)

	// Pre Shutdown Stuff goes Here
	// ============================
	// Free the kanji array
	for (i = 0; i < MAX_LINES; i++){
		if (kanji[i].k_kanji != NULL)
			free(kanji[i].k_kanji);
		if (kanji[i].k_kana != NULL)
			free(kanji[i].k_kana);
		if (kanji[i].e_descrip != NULL)
			free(kanji[i].e_descrip);
	}

	// Free the kanjidic array
	for (i = 0; i < MAX_KANJIDICT; i++){
		if (kanjidict[i].OnYomi != NULL)
			free(kanjidict[i].OnYomi);
		if (kanjidict[i].KunYomi != NULL)
			free(kanjidict[i].KunYomi);
		if (kanjidict[i].e_descrip != NULL)
			free(kanjidict[i].e_descrip);
	}

	// Free the screen_text array
	for (i = 0; i < MAX_SCREEN_TEXT; i++){
		if (screen_text[i].text != NULL){
			free(screen_text[i].text);
		}
	}

	INI_Save();

	oslDeleteFont(ltn0);
	oslDeleteImage(bkg);
	oslDeleteImage(spr_cur);
	oslIntraFontShutdown();
	endOSLib();
	sceKernelExitGame();
	return 0;
}// int main();

// Do basic program setup here.
int doInit(void){
	int rtn = 0;

	initOSLib();
	oslIntraFontInit(INTRAFONT_CACHE_ALL | INTRAFONT_STRING_UTF8); // All fonts loaded with oslLoadIntraFontFile will have UTF8 support

	// Load backgrounds
	bkg = oslLoadImageFilePNG("data/bkg.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_5551);
	bkg2 = oslLoadImageFilePNG("data/bkg2.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_5551);
	bkg3 = oslLoadImageFilePNG("data/bkg3.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_5551);
//	OSL_IMAGE *bkg4 = oslLoadImageFilePNG("bkg4.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_5650);
	spr_cur = oslLoadImageFilePNG("data/img2.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_5551);

	oslDisableTransparentColor();

	// Create a draw buffer - Will be used for scrolling kanji info.
	drawBuf = oslCreateImage(512, 512, OSL_IN_VRAM, OSL_PF_5551); // NEW

	if (spr_cur == NULL || bkg == NULL || bkg2 == NULL || bkg3 == NULL || drawBuf == NULL){
		oslDebug( "Error loading required images.\nPlease make sure image files are in the\ndata directory.");
		if (bkg != NULL)
			oslDeleteImage(bkg);
		if (bkg2 != NULL)
			oslDeleteImage(bkg2);
		if (bkg3 != NULL)
			oslDeleteImage(bkg3);
		if (spr_cur != NULL)
			oslDeleteImage(spr_cur);
		if (drawBuf != NULL)
			oslDeleteImage(drawBuf);
		return 1;
		//endOSLib();
	}

	oslDisableTransparentColor();

	// Load Font files
	ltn0 = oslLoadFontFile("flash0:/font/ltn0.pgf");
	if (ltn0 == NULL){
		oslDebug( "IntraFont: Error loading ltn0.");
		oslDeleteImage(bkg);
		oslDeleteImage(bkg2);
		oslDeleteImage(bkg3);
		oslDeleteImage(spr_cur);
		oslDeleteImage(drawBuf);
		//endOSLib();
		return 2;
	}

	// See if jpn0.pgf found in the EBOOT.PBP dir. This is for people who may have CFW that
	// has overwritten the file in flash0. If not found, then load from flash0.
	if (CheckFileExists("data/jpn0.pgf")){
		jpn0 = oslLoadIntraFontFile("data/jpn0.pgf", INTRAFONT_CACHE_ALL | INTRAFONT_STRING_SJIS); //japanese font with SJIS support
		//if (jpn0 != NULL)
		//	oslDebug("Successfully loaded jpn0.pgf from data folder!");
	}else{
		jpn0 = oslLoadIntraFontFile("flash0:/font/jpn0.pgf", INTRAFONT_CACHE_ALL | INTRAFONT_STRING_SJIS); //japanese font with SJIS support
	}
	if (jpn0 == NULL){
		oslDebug( "IntraFont: Error loading jpn0.\nIf you have Custom Firmware(CFW) installed\nthis file could have been overwritten.\nYou can still use this app if you place\nthe file jpn0.pgf in the data folder.");
		oslDeleteImage(bkg);
		oslDeleteImage(bkg2);
		oslDeleteImage(bkg3);
		oslDeleteImage(spr_cur);
		oslDeleteImage(drawBuf);
		//endOSLib();
		return 3;
	}

	// Checks for an ini file. If it exists, data is loaded, else
	// a file is created.
	INI_Check();

	oslStartDrawing();
	//oslDrawImageXY(bkg, 0, 0);
	oslClearScreen(RGB(250, 250, 250));
	DrawTitle(TITLE_SCREENPOS_X, TITLE_SCREENPOS_Y);
	// Draw Text
	oslSetFont(ltn0);
	oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	oslDrawString(20,80,"Checking Study Lists...");
	oslEndDrawing();
	oslSyncFrame();

	rtn = CheckStudyLists();

	oslStartDrawing();
	//oslDrawImageXY(bkg, 0, 0);
	oslClearScreen(RGB(250, 250, 250));
	DrawTitle(TITLE_SCREENPOS_X, TITLE_SCREENPOS_Y);
	oslSetFont(ltn0);
	oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	// Draw Text
	if (rtn == STUDY_LIST_CHK_CREATE_FILE_ERR){
			oslDrawString(20,100,"Couldn't create one of the Study Lists.");
			oslDrawString(20,120,"Make sure directory \"study_lists\" exists.");
			oslDrawString(20,140,"Study Lists are disabled.");
	}

	// Load the study lists, if no prev errors.
	if (studylist_enabled){
		rtn = LoadAllStudyLists(cur_edict);
		if (rtn == RTN_OK){
			oslDrawString(20,100,"Study Lists loaded OK.");
		}else{ // STUDY_LIST_CHK_LOAD_FILE_ERR
			oslDrawString(20,100,"Couldn't load one of the Study Lists.");
			oslDrawString(20,120,"Study Lists are disabled.");
		}
	}

	oslDrawString(20,160,"Loading Edict and Kanjidict files...");
	oslEndDrawing();
	oslSyncFrame();

	return RTN_OK;
}// int doInit(void);

// Basic setup has been done (e.g. graphics init, etc)...Now do the extras.
int doInit2(void){
	int i = 0;
	static int kdict_doonce = TRUE;
	//long flength = 0, nread = 0;


	// Clear found list
	for (i = 0; i < MAX_FOUND; i++)
		found_list[i] = -1;

	found_index = 0;
	found_count = 0;
	found_more = FALSE;
	is_finding = FALSE;

	// OSK Search Stuff.
	memset(osk_text, 0, sizeof(osk_text));
	memset(prev_osk_text, 0, sizeof(prev_osk_text));
	memset(osk_textUCS2, 0, sizeof(osk_textUCS2));
	osk_language = prev_osk_language = OSK_ENGLISH; // Eng=1;Japanese=0

	// Clear screen_text array
	for (i = 0; i < MAX_SCREEN_TEXT; i++){
		screen_text[i].x = 0;
		screen_text[i].y = 0;
		screen_text[i].offset_y = LINE_SPACE_IF_Y;
		screen_text[i].font_type = FONT_LATIN;
		if (screen_text[i].text != NULL){
			free(screen_text[i].text);
			screen_text[i].text = NULL;
		}
		screen_text[i].size = 1.0f;
		screen_text[i].color = BLACK;
		screen_text[i].shadowColor = WHITE;
		screen_text[i].options = INTRAFONT_ALIGN_LEFT;
	}
	// Clear tmp
	tmp_screen_text.x = 0;
	tmp_screen_text.y = 0;
	tmp_screen_text.offset_y = LINE_SPACE_IF_Y;
	tmp_screen_text.font_type = FONT_LATIN;
	if (tmp_screen_text.text != NULL){
		free(tmp_screen_text.text);
		tmp_screen_text.text = NULL;
	}
	tmp_screen_text.size = 1.0f;
	tmp_screen_text.color = BLACK;
	tmp_screen_text.shadowColor = WHITE;
	tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

	st_index = st_count = 0;
	st_can_scroll = TRUE;
	
	// ---------------------- EDICT STUFF HERE ----------------------------------
	//Now read in the contents.
	lines_read = LoadEdict();
	free(ucode);
	ucode = NULL;

	if (lines_read == 0){
		oslDebug( "\nLoadEdict: Error reading file \"%s\"\n Read error occured or no lines in file.\n", fKanjiName[cur_edict]);
		return 5;
	}
	kanji_index = 0;

	//// Debugging
	//ram = oslGetRamStatus();	
	//oslDebug("After LoadEdict(). Before LoadKanjidict().\nbytes available=%dkb, %0.2fmb.\nmaxBlockSize = %dkb, %0.2fmb.", ram.maxAvailable/1024, ((float)ram.maxAvailable/1024)/1024, ram.maxBlockSize/1024, ((float)ram.maxBlockSize/1024)/1024);
	// ---------------------- KDICT STUFF HERE ----------------------------------

	// Only do once.
	if (kdict_doonce){
		kdict_doonce = FALSE;
		kdict_lines_read = LoadKanjidict();
		kdict_lines_read_backup = kdict_lines_read;
		
		if (kdict_lines_read == 0){
			oslDebug( "\nLoadKD: Error reading file \"%s\"\n Read error occured or no lines in file.\n", fKanjiDictName );
			return 6;
		}

		// Now sort kdic by stroke count
		qsort( (void *)kanjidict, (size_t)kdict_lines_read + 1, sizeof( ukanjidict ), kanjidict_compare_stroke );
	}

	//// Debugging
	//ram = oslGetRamStatus();	
	//oslDebug("After LoadKanjidict().\nbytes available=%dkb, %0.2fmb.\nmaxBlockSize = %dkb, %0.2fmb.", ram.maxAvailable/1024, ((float)ram.maxAvailable/1024)/1024, ram.maxBlockSize/1024, ((float)ram.maxBlockSize/1024)/1024);
	

	return RTN_OK;

}// END FUNC int doInit2(void);

void CheckButtons(void){
	int i = 0, rtn = 0;//, kmatch_index = 0;
	static int tmp_game_status = 0;
	static int key_held_count = 0;
	static int tmp_sort_type = OPT_SORT_ORIG;
	static int tmp_search_type = OPT_SEARCH_ANY;
	static int tmp_match_whole = FALSE;
	static int tmp_show_examples = TRUE;	
	static int edict_load_wait = 0;
	OSL_CONTROLLER *keys;
	//OSL_MEMSTATUS ram;

	if (oslOskIsActive())
		return;

	keys = oslReadKeys();

	// Check for anologue movement.
	cursor.origx = cursor.x;
	cursor.origy = cursor.y;
	for (i=40;i <= 120;i += 20){// This is ramping the speed depending on where the stick is.
		if (keys->analogX > i) cursor.x++;
		if (keys->analogY > i) cursor.y++;
		if (keys->analogX < -i) cursor.x--;
		if (keys->analogY < -i) cursor.y--;
	}
	// Make sure cursor doesn't go out of bounds.
	if ( cursor.x > (AA_SCREEN_WIDTH - CURSOR_WIDTH))
		cursor.x = (AA_SCREEN_WIDTH - CURSOR_WIDTH);
	if ( cursor.x < 0)
		cursor.x = 0;
	if ( cursor.y > (AA_SCREEN_HEIGHT - CURSOR_HEIGHT))
		cursor.y = (AA_SCREEN_HEIGHT - CURSOR_HEIGHT);
	if ( cursor.y < 0)
		cursor.y = 0;

	if ((rtn = CheckMenu(&tmp_game_status, keys)) != RTN_OK){
		// Not an error, just means don't continue with other key checks.
		if (rtn == MENU_OPT){
			// Save the old search and sort vars to compare later.
			tmp_sort_type = sort_type;
			tmp_search_type = search_type;
			tmp_match_whole = match_whole;
			tmp_show_examples = show_examples;
		}
		return;
	}

	CheckKanjiPopup();

	if (keys->held.R || keys->held.L ){
		if (key_held_count <= MAX_KEY_HELD_COUNT)
				key_held_count++;
	}else{
		key_held_count = 0; // Used to increase page scroll from 10 to 100
	}
	
	switch( game_status ){
	case GS_INIT:
		break;
	case GS_INTRO:
		break;
	case GS_TEST: // Used for testing only
		//if (keys->pressed.circle){
		//	game_status = GS_SETUP;
		//	break;
		//}
		break;
	case GS_SETUP:
		if (keys->pressed.cross){
			// doInit2() should have set/re-set default values.
			cursor.enabled = TRUE;
			SetBaseGS(GS_PLAYING); // New 24/10/09
			break;
		}
		if (keys->pressed.circle){
			SetupScreenGSHELP();
			game_status = GS_HELP;
			break;
		}
		// For testing only - Remove for release
		//if (keys->pressed.select){
		//	game_status = GS_TEST;
		//	break;
		//}
		break;
	case GS_HELP:
		if (keys->pressed.circle){
			game_status = GS_SETUP;
			break;
		}
		// Used for scrolling if appropriate.
		if (keys->pressed.up){// Back.
			if (st_index - 1 >= 0)
				st_index--;
			break;
		}
		if (keys->pressed.down){// Forward.
			if (st_can_scroll){
				if (st_index + 1 < st_count)
					st_index++;
			}
			break;
		}
		break;
	case GS_SHOW_FILES:
		if (keys->pressed.cross){
			if (num_files > 0){
				rtn = LoadFoundSet();
				switch(rtn){
					case LOAD_ILLEGAL_INDEX:
						oslMessageBox("Illegal index to File List. Aborting Load.", "Load Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						ReturnToPrevGS(); // New 24/10/09
						return;
					case LOAD_FOPEN_ERR:
						oslMessageBox("Unable to Load the Found Set file. Aborting Load.", "File Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						ReturnToPrevGS(); // New 24/10/09
						return;
					case LOAD_FREAD_COUNT_ERR:
					case LOAD_FREAD_LIST_ERR:
						ResetFoundSet();
						oslMessageBox("Unable to read from the Found Set file. Found Set has been reset. Aborting Load.", "Read Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						ReturnToPrevGS(); // New 24/10/09
						return;
					case LOAD_STRRCHR_ERR:
						ResetFoundSet();
						oslMessageBox("Unable to set osk_text. Found Set has been reset. Aborting Load.", "Found Set Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						ReturnToPrevGS(); // New 24/10/09
						return;
				}
				
				// On Success, just go to found set. Don't worry about displaying message.
				// Should maybe go to GS_SEARCHING instead so current sort/find options will be run?
				found_index = 0;
				SetGS(GS_LEVEL_02, GS_SHOW_RESULTS, TRUE); // New 24/10/09
				break;
			}
		}
		if (keys->pressed.circle){ // Cancel
			ReturnToPrevGS(); // New 24/10/09
			break;
		}
		if (keys->pressed.square){ // Delete
			if (num_files > 0){
				rtn = DeleteFoundSet();

				switch(rtn){
					case DEL_ILLEGAL_INDEX:
						oslMessageBox("Illegal index to File List. Aborting Deletion.", "Delete Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						break;
					case DEL_FILE_NOT_FOUND:
						oslMessageBox("The selected file could not be found. Aborting Deletion.", "Delete Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						break;
					case DEL_DID_NOT_DEL:
						oslMessageBox("An error occured while attempting to delete the file. You may need to delete the file manually. Aborting Deletion.", "Delete Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						break;
					case DEL_CREATEFILELIST_ERR:
						oslMessageBox("Error recreating file list after deletion.", "File List Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						ReturnToPrevGS(); // New 24/10/09
						break;
					case RTN_OK:
						oslMessageBox("Successfully deleted the Found Set.", "Deleted!", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						break;
				}// End switch(rtn)
				break;
			}// End if (num_files > 0)
		}// End if (keys->pressed.square) // Delete

		// Move to next page
		if (keys->pressed.R){
			if ((fname_index + 10) < num_files)
				fname_index += 10;
			else
				fname_index = num_files - 1;
			break;
		}
		// Move to prev page
		if (keys->pressed.L){
			if ((fname_index - 10) >= 0)
				fname_index -= 10;
			else
				fname_index = 0;
			break;
		}
		if (keys->pressed.up){ // Move to prev file
			if ((fname_index - 1) >= 0 )
				fname_index--;
			break;
		}
		if (keys->pressed.down){ // Move to next file
			if ((fname_index + 1) < num_files )
				fname_index++;
			break;
		}
		break;
	case GS_SHOW_EDICT_FILES:
		if (keys->pressed.circle){ // Cancel
			ReturnToPrevGS(); // New 24/10/09
			break;
		}
		if (keys->pressed.cross){
			// If it's the same one, just return.
			if (cur_edict == cur_edict_index){
				memset(buffer, 0, sizeof(buffer));
				sprintf(buffer, "The file \"%s\" is already loaded.", fKanjiName[cur_edict]);
				oslMessageBox(buffer, "Already Loaded", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
				ReturnToPrevGS(); // New 24/10/09
				break;
			}
			cur_edict = cur_edict_index;
			SetGS(GS_LEVEL_04, GS_LOAD_EDICT, TRUE); // New 24/10/09
			break;
		}
		if (keys->pressed.up){ // Move to prev file
			if ((cur_edict_index - 1) >= 0 )
				cur_edict_index--;
			break;
		}
		if (keys->pressed.down){ // Move to next file
			if ((cur_edict_index + 1) < MAX_EDICT )
				cur_edict_index++;
			break;
		}
		
		break;
	case GS_LOAD_EDICT:
		if (edict_load_wait < 2){
			// Give the ScreenUpdate a chance to run.
			edict_load_wait++;
			break;
		}else
			edict_load_wait = 0;

		if (studylist_enabled){
			rtn = LoadAllStudyLists(cur_edict);

			oslStartDrawing();
			oslClearScreen(RGB(250, 250, 250));
			DrawTitle(TITLE_SCREENPOS_X, TITLE_SCREENPOS_Y);
			// Draw Text
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			
			if (rtn == RTN_OK){
				oslDrawString(20,100,"Study Lists loaded OK.");
			}else{ // STUDY_LIST_CHK_LOAD_FILE_ERR
				oslDrawString(20,100,"Couldn't load one of the Study Lists.");
				oslDrawString(20,120,"Study Lists are disabled.");
			}
			oslDrawString(20,160,"Loading new Edict file...");

			oslEndDrawing();
			oslSyncFrame();
		}

		// Ready to load the new edict file.
		rtn = doInit2();
		if (rtn != RTN_OK){
			oslDebug( "Error. doInit2 returned %i. Could not load file %s.", rtn,  fKanjiName[cur_edict]);
			endOSLib();
			break;
		}

		//// For Debugging
		//ram = oslGetRamStatus();
		//oslDebug("Mem Debugging\nbytes available=%dkb, %0.2fmb.\nmaxBlockSize = %dkb, %0.2fmb.", ram.maxAvailable/1024, ((float)ram.maxAvailable/1024)/1024, ram.maxBlockSize/1024, ((float)ram.maxBlockSize/1024)/1024);

		game_status = GS_SETUP;

		break;
	case GS_OPTIONS:
		if (keys->pressed.circle){
			if (show_examples != tmp_show_examples && GetPrevGS(cur_gs_level) == GS_SHOW_SEL){
				//oslDebug("GetPrevGS = %d\ncur_gs_level = %d\nGetPrevGSLevel(GS_LEVEL_03) = %d\ngame_status = %d\n prev_game_status = %d",GetPrevGS(cur_gs_level), cur_gs_level, GetPrevGSLevel(GS_LEVEL_03), game_status, prev_game_status);
				// Need to set up screen text again.
				switch(GetPrevGS(GS_LEVEL_03)){
					case GS_PLAYING:
						SetupScreenTextEdict(kanji_index);// NEW
						break;
					case GS_SHOW_RESULTS:
						if (found_count > 0){
						   SetupScreenTextEdict(found_list[found_index]);// NEW
						}
						break;
					case GS_SHOW_STUDYLIST:
						if (studylist_count[cur_studylist] > 0){
						   SetupScreenTextEdict(studylist[cur_studylist][studylist_index[cur_studylist]].index);// NEW
						}
						break;
				}
			}
			// Check if anything has changed, and if so, may need to do action.
			if (search_type != tmp_search_type && osk_language == OSK_JAPANESE){
				// Changed, re-run search.
				if (osk_text[0] != '\0'){
					is_finding = FALSE;
					SetGS(GS_LEVEL_04, GS_SEARCHING, TRUE); // New 24/10/09
					break;
				}
				ReturnToPrevGS(); // New 24/10/09
				break;
			}
			if (match_whole != tmp_match_whole){// && osk_language == OSK_ENGLISH){
				// Changed, re-run search.
				if (osk_text[0] != '\0'){
					is_finding = FALSE;
					SetGS(GS_LEVEL_04, GS_SEARCHING, TRUE); // New 24/10/09
					break;
				}
				ReturnToPrevGS(); // New 24/10/09
				break;
			}
			if (sort_type != tmp_sort_type){
				// Changed, re-sort found set
				if (found_count > 1){// A list of 1 is considered sorted.
					rtn = found_list[found_index];
					switch (sort_type){
						case OPT_SORT_ORIG:
							qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_orig );
							break;
						case OPT_SORT_LENGTH:
							qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_length );
							break;
						case OPT_SORT_READING:
							qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_reading );
							break;
					}
					// Reset the index
					for(i = 0; i < found_count;i++){
						if(found_list[i] == rtn){
							found_index = i;
							break;
						}
					}
					SetGS(GS_LEVEL_02, GS_SHOW_RESULTS, TRUE); // New 28/10/09
					break;
				}
				ReturnToPrevGS(); // New 24/10/09
				break;
			}// End if (sort_type != tmp_sort_type)

			ReturnToPrevGS(); // New 24/10/09
			break;
		}
		if (keys->pressed.cross){ // simulates mouse click.
			// Check Radio 1
			if ( cursorInRange(cursor.x, cursor.y, OPT_RADIO_01_TEXT_X - OPT_SCN_RADIO_WIDTH, OPT_RADIO_01_TEXT_Y, OPT_RADIO_01_TEXT_X + OPT_RADIO_01_TEXT_WIDTH, OPT_RADIO_01_TEXT_Y + LINE_SPACE_IF_Y) ){
				if (search_type != OPT_SEARCH_ANY)
					search_type = OPT_SEARCH_ANY;
			}else{
				// Check Radio 2
				if ( cursorInRange(cursor.x, cursor.y, OPT_RADIO_02_TEXT_X - OPT_SCN_RADIO_WIDTH, OPT_RADIO_02_TEXT_Y, OPT_RADIO_02_TEXT_X + OPT_RADIO_02_TEXT_WIDTH, OPT_RADIO_02_TEXT_Y + LINE_SPACE_IF_Y) ){
					if (search_type != OPT_SEARCH_START)
						search_type = OPT_SEARCH_START;
				}else{
					// Check Radio 3
					if ( cursorInRange(cursor.x, cursor.y, OPT_RADIO_03_TEXT_X - OPT_SCN_RADIO_WIDTH, OPT_RADIO_03_TEXT_Y, OPT_RADIO_03_TEXT_X + OPT_RADIO_03_TEXT_WIDTH, OPT_RADIO_03_TEXT_Y + LINE_SPACE_IF_Y) ){
						if (sort_type != OPT_SORT_ORIG)
							sort_type = OPT_SORT_ORIG;
					}else{
						// Check Radio 4
						if ( cursorInRange(cursor.x, cursor.y, OPT_RADIO_04_TEXT_X - OPT_SCN_RADIO_WIDTH, OPT_RADIO_04_TEXT_Y, OPT_RADIO_04_TEXT_X + OPT_RADIO_04_TEXT_WIDTH, OPT_RADIO_04_TEXT_Y + LINE_SPACE_IF_Y) ){
							if (sort_type != OPT_SORT_LENGTH)
								sort_type = OPT_SORT_LENGTH;
						}else{
							// Check Radio 5
							if ( cursorInRange(cursor.x, cursor.y, OPT_RADIO_05_TEXT_X - OPT_SCN_RADIO_WIDTH, OPT_RADIO_05_TEXT_Y, OPT_RADIO_05_TEXT_X + OPT_RADIO_05_TEXT_WIDTH, OPT_RADIO_05_TEXT_Y + LINE_SPACE_IF_Y) ){
								if (sort_type != OPT_SORT_READING)
									sort_type = OPT_SORT_READING;
							}else{
								// Check Check 1
								if ( cursorInRange(cursor.x, cursor.y, OPT_CHECK_01_TEXT_X - OPT_SCN_CHECK_WIDTH, OPT_CHECK_01_TEXT_Y, OPT_CHECK_01_TEXT_X + OPT_CHECK_01_TEXT_WIDTH, OPT_CHECK_01_TEXT_Y + (LINE_SPACE_IF_Y * /*4*/3)) ){
									if (match_whole)
										match_whole = FALSE;
									else
										match_whole = TRUE;
								}else{
									// Check Check 2 - show_examples
									if ( cursorInRange(cursor.x, cursor.y, OPT_CHECK_02_TEXT_X - OPT_SCN_CHECK_WIDTH, OPT_CHECK_02_TEXT_Y, OPT_CHECK_02_TEXT_X + OPT_CHECK_02_TEXT_WIDTH, OPT_CHECK_02_TEXT_Y + (LINE_SPACE_IF_Y * 2)) ){
										if (show_examples)
											show_examples = FALSE;
										else
											show_examples = TRUE;
									}
									else {
										// Don't show Check Kanji
										if (cursorInRange(cursor.x, cursor.y, OPT_CHECK_03_TEXT_X - OPT_SCN_CHECK_WIDTH, OPT_CHECK_03_TEXT_Y, OPT_CHECK_03_TEXT_X + OPT_CHECK_03_TEXT_WIDTH, OPT_CHECK_03_TEXT_Y + (LINE_SPACE_IF_Y * 2)))
										{
											if(!kanjiHiding)
												kanjiHiding = TRUE;
											else
												kanjiHiding = FALSE;
										}
										else {
											// Don't show Check Kana
										if (cursorInRange(cursor.x, cursor.y, OPT_CHECK_04_TEXT_X - OPT_SCN_CHECK_WIDTH, OPT_CHECK_04_TEXT_Y, OPT_CHECK_04_TEXT_X + OPT_CHECK_04_TEXT_WIDTH, OPT_CHECK_04_TEXT_Y + (LINE_SPACE_IF_Y * 2)))
										{
											if(!kanaHiding)
												kanaHiding = TRUE;
											else
												kanaHiding = FALSE;
										}
										else {
											// Don't show Check Desc
										if (cursorInRange(cursor.x, cursor.y, OPT_CHECK_05_TEXT_X - OPT_SCN_CHECK_WIDTH, OPT_CHECK_05_TEXT_Y, OPT_CHECK_05_TEXT_X + OPT_CHECK_05_TEXT_WIDTH, OPT_CHECK_05_TEXT_Y + (LINE_SPACE_IF_Y * 2)))
										{
											if(!descHiding)
												descHiding = TRUE;
											else
												descHiding = FALSE;
										}
										}
										}
									}
								}
							}
						}
					}
				}
			}			
			break;
		}
		break;
	case GS_RESET:
	case GS_PLAYING:
		if (show_kdict){
			CheckKanjiPopupInput(keys);
			break;
		}// End if (show_kdict)
		// English OSK
		if (keys->pressed.triangle){
			prev_osk_language = osk_language;
			osk_language = OSK_ENGLISH;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}
		if (keys->pressed.circle){
			break;
		}
		// Enter a value/selection
		if (keys->pressed.cross){ // simulates mouse click.
			// Only while debugging.
			//osl_quit = 1;
			// Show only the selected line
			//SetupScreenTextEdict(font_jpn, font_ltn, kanji_index);// NEW
			SetupScreenTextEdict(kanji_index);// NEW
			SetGS(GS_LEVEL_03, GS_SHOW_SEL, TRUE); // New 24/10/09
			break;
		}
		// Japanese OSK
		if (keys->pressed.square){ // search, japanese mode.
			prev_osk_language = osk_language;
			osk_language = OSK_JAPANESE;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}
		// Move to next page
		if (keys->pressed.R){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((kanji_index + 100) < lines_read){
					kanji_index += 100;
				}else{
					if ((kanji_index + 10) < lines_read)
						kanji_index += 10;
					else
						kanji_index = lines_read - 1;//2;
				}
			}else{
				if ((kanji_index + 10) < lines_read)
						kanji_index += 10;
				else
					kanji_index = lines_read - 1;//2;
			}

			cursor_tick = 0; // Added 30/09/09
			kdict_shown = FALSE;
			break;
		}
		// Move to prev page
		if (keys->pressed.L){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((kanji_index - 100) >= 0){
					kanji_index -= 100;
				}else{
					if ((kanji_index - 10) >= 0)
						kanji_index -= 10;
					else
						kanji_index = 0;
				}
			}else{
				if ((kanji_index - 10) >= 0)
					kanji_index -= 10;
				else
					kanji_index = 0;
			}

			//if ((kanji_index - ((key_held_count > MAX_KEY_HELD_COUNT)? 100 : 10)) >= 0 )
			//	kanji_index -= ((key_held_count > MAX_KEY_HELD_COUNT)? 100 : 10);//10;
			//else
			//	kanji_index = 0;

			cursor_tick = 0; // Added 30/09/09
			kdict_shown = FALSE;
			break;
		}
		
		if (keys->pressed.left){
			break;
		}
		if (keys->pressed.right){
			break;
		}
		
		if (keys->pressed.up){
			if ((kanji_index - 1) >= 0 )
				kanji_index--;
			break;
		}
		if (keys->pressed.down){
			if ((kanji_index + 1) < lines_read )
				kanji_index++;
			break;
		}
		// Open Menu
		if (keys->pressed.start){
			//kanji_index = 0;
			if (menu_status == M_CLOSED){
				menu_status = M_OPENING;
				menu_counter = 0;
			}
			break;
		}

		break;
	case GS_SHOW_KANJIDICT:
		if (keys->pressed.circle){
			// This is a base item, so user must select Edict from menu to change.
			break;
		}

		// English OSK
		if (keys->pressed.triangle){
			prev_osk_language = osk_language;
			osk_language = OSK_ENGLISH;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}
		// Japanese OSK
		if (keys->pressed.square){ // search, japanese mode.
			prev_osk_language = osk_language;
			osk_language = OSK_JAPANESE;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}

		// Enter a value/selection
		if (keys->pressed.cross){ // simulates mouse click.
			SetupScreenTextKdic();
			SetGS(GS_LEVEL_03, GS_SHOW_SEL, TRUE); // New 24/10/09
			break;
		}
		// Move to next page
		if (keys->pressed.R){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((kdict_index + 100) < kdict_lines_read){
					kdict_index += 100;
				}else{
					if ((kdict_index + 10) < kdict_lines_read)
						kdict_index += 10;
					else
						kdict_index = kdict_lines_read - 1;//2;
				}
			}else{
				if ((kdict_index + 10) < kdict_lines_read)
					kdict_index += 10;
				else
					kdict_index = kdict_lines_read - 1;//2;
			}

			//if ((kdict_index + ((key_held_count > MAX_KEY_HELD_COUNT)? 100 : 10)) < kdict_lines_read )
			//	kdict_index += ((key_held_count > MAX_KEY_HELD_COUNT)? 100 : 10);//10;
			break;
		}
		// Move to prev page
		if (keys->pressed.L){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((kdict_index - 100) >= 0){
					kdict_index -= 100;
				}else{
					if ((kdict_index - 10) >= 0)
						kdict_index -= 10;
					else
						kdict_index = 0;
				}
			}else{
				if ((kdict_index - 10) >= 0)
					kdict_index -= 10;
				else
					kdict_index = 0;
			}

			break;
		}
		if (keys->pressed.up){
			if ((kdict_index - 1) >= 0 )
				kdict_index--;
			break;
		}
		if (keys->pressed.down){
			if ((kdict_index + 1) < kdict_lines_read )
				kdict_index++;
			break;
		}
		// Open Menu
		if (keys->pressed.start){
			if (menu_status == M_CLOSED){
				menu_status = M_OPENING;
				menu_counter = 0;
			}
			break;
		}
		break;
	case GS_SHOW_OSK:
		break;
	case GS_SEARCHING:
		if (is_finding < 2){
			is_finding++;
			// Do nothing first time. This will allow ScreenUpdate to show the background + Text
			break;
		}else{
			is_finding = TRUE;
		}

		// Should only have come here if there is something to search
		if (osk_language == OSK_ENGLISH)
			found_count = find_eng_string(osk_text, match_whole);
		else{
			// Is it kana or kanji?
			if ((u8)osk_text[0] > 0x87)
				found_count = find_kanji_string(osk_text, search_type, match_whole);
			else
				found_count = find_SJIS_string(osk_text, search_type, match_whole);
		}

		if (found_count > 1){
			// Sort the data
			switch (sort_type){
				// Case OPT_SORT_ORIG is already sorted, so don't need to do anything.
				case OPT_SORT_LENGTH:
					qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_length );
					break;
				case OPT_SORT_READING:
					qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_reading );
					break;
			}
		}

		is_finding = FALSE;
		found_index = 0;
		
		if(kanjiListing == NONE)
		{
			SetGS(GS_LEVEL_02, GS_SHOW_RESULTS, TRUE); // New 24/10/09
		}
		
		else
		{
			SetGS(GS_LEVEL_02, GS_SHOW_KANJIDICT, TRUE);
		}
		
		break;
		
	case GS_SHOW_RESULTS:
		if (show_kdict){
			CheckKanjiPopupInput(keys);
			break;
		}// End if (show_kdict)

		// English OSK
		if (keys->pressed.triangle){
			prev_osk_language = osk_language;
			osk_language = OSK_ENGLISH;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}
		if (keys->pressed.circle){
			ReturnToPrevGS(); // New 24/10/09
			break;
		}
		// Enter a value/selection
		if (keys->pressed.cross){ // simulates mouse click.
			if (found_count > 0){ // Bug fix #01
				SetupScreenTextEdict(found_list[found_index]);// NEW
				SetGS(GS_LEVEL_03, GS_SHOW_SEL, TRUE); // New 24/10/09
			}
			break;
		}
		// Japanese OSK
		if (keys->pressed.square){ // search, japanese mode.
			prev_osk_language = osk_language;
			osk_language = OSK_JAPANESE;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}
		// Move to next page
		if (keys->pressed.R){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((found_index + 100) < found_count){
					found_index += 100;
				}else{
					if ((found_index + 10) < found_count)
						found_index += 10;
					else
						found_index = found_count - 1;
				}
			}else{
				if ((found_index + 10) < found_count)
						found_index += 10;
				else
					found_index = found_count - 1;
			}
			//if ((found_index + 10) < found_count)
			//	found_index += 10;

			cursor_tick = 0; // Added 30/09/09
			kdict_shown = FALSE;
			break;
		}
		// Move to prev page
		if (keys->pressed.L){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((found_index - 100) >= 0){
					found_index -= 100;
				}else{
					if ((found_index - 10) >= 0)
						found_index -= 10;
					else
						found_index = 0;
				}
			}else{
				if ((found_index - 10) >= 0)
					found_index -= 10;
				else
					found_index = 0;
			}

			cursor_tick = 0; // Added 30/09/09
			kdict_shown = FALSE;
			break;
		}
		if (keys->pressed.left){
			break;
		}
		if (keys->pressed.right){
			break;
		}
		
		if (keys->pressed.up){
			if ((found_index - 1) >= 0 )
				found_index--;
			break;
		}
		if (keys->pressed.down){
			if ((found_index + 1) < found_count )
				found_index++;
			break;
		}
		// Open Menu
		if (keys->pressed.start){
			//kanji_index = 0;
			if (menu_status == M_CLOSED){
				menu_status = M_OPENING;
				menu_counter = 0;
			}
			break;
		}
		break;
	case GS_SHOW_STUDYLIST:
		if (show_kdict){
			CheckKanjiPopupInput(keys);
			break;
		}// End if (show_kdict)

		// English OSK
		if (keys->pressed.triangle){
			prev_osk_language = osk_language;
			osk_language = OSK_ENGLISH;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}
		// Japanese OSK
		if (keys->pressed.square){ // search, japanese mode.
			prev_osk_language = osk_language;
			osk_language = OSK_JAPANESE;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}

		if (keys->pressed.circle){
			ReturnToPrevGS(); // New 24/10/09
			break;
		}
		// Enter a value/selection
		if (keys->pressed.cross){ // simulates mouse click.
			if (studylist_count[cur_studylist] > 0){ // Bug Fix #01
				SetupScreenTextEdict(studylist[cur_studylist][studylist_index[cur_studylist]].index);// NEW
				SetGS(GS_LEVEL_03, GS_SHOW_SEL, TRUE); // New 24/10/09
			}
			break;
		}

		// Move to next page
		if (keys->pressed.R){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((studylist_index[cur_studylist] + 100) < studylist_count[cur_studylist]){
					studylist_index[cur_studylist] += 100;
				}else{
					if ((studylist_index[cur_studylist] + 10) < studylist_count[cur_studylist])
						studylist_index[cur_studylist] += 10;
					else
						studylist_index[cur_studylist] = studylist_count[cur_studylist] - 1;
				}
			}else{
				if ((studylist_index[cur_studylist] + 10) < studylist_count[cur_studylist])
						studylist_index[cur_studylist] += 10;
				else
					studylist_index[cur_studylist] = studylist_count[cur_studylist] - 1;
			}

			cursor_tick = 0; // Added 30/09/09
			kdict_shown = FALSE;
			break;
		}
		// Move to prev page
		if (keys->pressed.L){
			if (key_held_count > MAX_KEY_HELD_COUNT){
				if ((studylist_index[cur_studylist] - 100) >= 0){
					studylist_index[cur_studylist] -= 100;
				}else{
					if ((studylist_index[cur_studylist] - 10) >= 0)
						studylist_index[cur_studylist] -= 10;
					else
						studylist_index[cur_studylist] = 0;
				}
			}else{
				if ((studylist_index[cur_studylist] - 10) >= 0)
					studylist_index[cur_studylist] -= 10;
				else
					studylist_index[cur_studylist] = 0;
			}

			cursor_tick = 0; // Added 30/09/09
			kdict_shown = FALSE;
			break;
		}
		if (keys->pressed.left){// Move to next list.
			if (cur_studylist == 0)
				cur_studylist = MAX_NUM_STUDY_LIST - 1;
			else
				cur_studylist--;
			break;
		}
		if (keys->pressed.right){// Move to prev list.
			if (cur_studylist == (MAX_NUM_STUDY_LIST - 1))
				cur_studylist = 0;
			else
				cur_studylist++;
			break;
		}
		if (keys->pressed.up){
			if ((studylist_index[cur_studylist] - 1) >= 0 )
				studylist_index[cur_studylist]--;
			break;
		}
		if (keys->pressed.down){
			if ((studylist_index[cur_studylist] + 1) < studylist_count[cur_studylist] )
				studylist_index[cur_studylist]++;
			break;
		}
		// Open Menu
		if (keys->pressed.start){
			//kanji_index = 0;
			if (menu_status == M_CLOSED){
				menu_status = M_OPENING;
				menu_counter = 0;
			}
			break;
		}

		// Added by Batanen
		if (keys->pressed.select)
		{
		
			// Check how many items are in studylist
			int items = 0;
			int i;
			for(i = 0; i < MAX_STUDY_LIST; ++i)
			{
				if ((studylist_index[cur_studylist] + i) < studylist_count[cur_studylist])
				{
					++items;
				}
				
				else
				{
					break;
				}
			}
			
		// Allow randomization only when studylist contains more than 1 items.
		if (items > 1)
		{
			// random number
			int iSecret = rand() % items;
			//int iSecret = 2;
		
			int i;
			ustudylistel tempStudyRandom[MAX_NUM_STUDY_LIST][MAX_STUDY_LIST] = {{{-1,0},},};;
			ustudylistel tempStudyOriginal[MAX_NUM_STUDY_LIST][MAX_STUDY_LIST] = {{{-1,0},},};;
			for(i=0; i<MAX_STUDY_LIST; ++i)
			{
				// Was not last randomize
				if ((studylist_index[cur_studylist] + i) < studylist_count[cur_studylist] && iSecret < items)
				{
					tempStudyRandom[cur_studylist][i] = studylist[cur_studylist][iSecret];
					tempStudyOriginal[cur_studylist][i] = studylist[cur_studylist][i];
					studylist[cur_studylist][i] = tempStudyRandom[cur_studylist][i];
					studylist[cur_studylist][iSecret] = tempStudyOriginal[cur_studylist][i];
					++iSecret;
				}
			
				else
				{
					break;
				}
			}
		}
		
		
		//studylist[cur_studylist][studylist_index[cur_studylist]] = studylist[cur_studylist][1];
		break;
		}
		
		break;
	case GS_SHOW_SEL:
		if (show_kdict){
			CheckKanjiPopupInput(keys);
			break;
		}// End if (show_kdict)

		if (keys->pressed.circle){
			ReturnToPrevGS(); // New 24/10/09
			break;
		}
		// Enter a value/selection
		if (keys->pressed.cross){ // simulates mouse click.
			if (prev_game_status == GS_SHOW_KANJIDICT){
				// Search for the Kanji
				found_count = find_kanji_string(kanjidict[kdict_index].kanji, search_type, match_whole);
				if (found_count > 1){// Added 18/10/09
					// Sort the data
					switch (sort_type){
						// Case OPT_SORT_ORIG is already sorted, so don't need to do anything.
						case OPT_SORT_LENGTH:
							qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_length );
							break;
						case OPT_SORT_READING:
							qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_reading );
							break;
					}
				}
				is_finding = FALSE;
				found_index = 0;
				strcpy(osk_text, kanjidict[kdict_index].kanji);
				osk_language = OSK_JAPANESE;
				SetGS(GS_LEVEL_02, GS_SHOW_RESULTS, TRUE); // New 24/10/09
				break;
			}
		}
		// Open Menu
		if (keys->pressed.start){
			//kanji_index = 0;
			if (menu_status == M_CLOSED){
				menu_status = M_OPENING;
				menu_counter = 0;
			}
			break;
		}

		// English OSK
		if (keys->pressed.triangle){
			prev_osk_language = osk_language;
			osk_language = OSK_ENGLISH;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}
		// Japanese OSK
		if (keys->pressed.square){ // search, japanese mode.
			prev_osk_language = osk_language;
			osk_language = OSK_JAPANESE;
			SetGS(GS_LEVEL_04, GS_SHOW_OSK, TRUE); // New 24/10/09
			break;
		}

		if (prev_game_status == GS_PLAYING || prev_game_status == GS_SHOW_RESULTS || prev_game_status == GS_SHOW_STUDYLIST){// || prev_game_status == GS_SHOW_KANJIDICT){
			// Used for scrolling if appropriate.
			if (keys->pressed.up){// Back.
				if (st_index - 1 >= 0)
					st_index--;
				break;
			}
			if (keys->pressed.down){// Forward.
				if (st_can_scroll){
					if (st_index + 1 < st_count)
						st_index++;
				}
				break;
			}
		}

		if (prev_game_status == GS_SHOW_KANJIDICT){
			if (keys->pressed.L){
				if ((kdict_index - 1) >= 0 ){
					kdict_index--;
					SetupScreenTextKdic();
				}
				break;
			}
			if (keys->pressed.R){
				if ((kdict_index + 1) < kdict_lines_read ){
					kdict_index++;
					SetupScreenTextKdic();
				}
				break;
			}
			// Used for scrolling if appropriate. Different from other prev_game_status.
			if (keys->pressed.up){// Back.
				//if (st_index - 1 >= 0)
				//	st_index--;
				//break;
				if (st_index == 0)
					break;

				// Assumes, only 1 'Heading' per line.
				if ((st_index - 2) < 0){
					st_index = 0;
					break;
				}

				if (screen_text[st_index-2].offset_y != 0){
					st_index--;
				}else{
					st_index -= 2;
				}
				break;
			}
			if (keys->pressed.down){// Forward.
				if (st_can_scroll){
					i=1;
					do{
						if (st_index + i < st_count){
							if (screen_text[st_index+i-1].offset_y != 0){
								st_index += i;
								break;
							}
						}
						i++;
					}while((st_index + i) < st_count);
				}
				break;
			}
			break;
		}// End if (prev_game_status == GS_SHOW_KANJIDICT)
		if (prev_game_status == GS_PLAYING){
			if (keys->pressed.L){
				if ((kanji_index - 1) >= 0 ){
					kanji_index--;
					SetupScreenTextEdict(kanji_index);// NEW
				}
				break;
			}
			if (keys->pressed.R){
				if ((kanji_index + 1) < lines_read ){
					kanji_index++;
					SetupScreenTextEdict(kanji_index);// NEW
				}
				break;
			}
			break;
		}// End if (prev_game_status == GS_PLAYING)
		if (prev_game_status == GS_SHOW_RESULTS){
			if (keys->pressed.L){
				if ((found_index - 1) >= 0 ){
					found_index--;
					SetupScreenTextEdict(found_list[found_index]);// NEW
				}
				break;
			}
			if (keys->pressed.R){
				if ((found_index + 1) < found_count ){
					found_index++;
					SetupScreenTextEdict(found_list[found_index]);// NEW
				}
				break;
			}
			break;
		}// End if (prev_game_status == GS_SHOW_RESULTS)
		if (prev_game_status == GS_SHOW_STUDYLIST){
			if (keys->pressed.L){
				if ((studylist_index[cur_studylist] - 1) >= 0 ){
					studylist_index[cur_studylist]--;
					SetupScreenTextEdict(studylist[cur_studylist][studylist_index[cur_studylist]].index);// NEW
				}
				break;
			}
			if (keys->pressed.R){
				if ((studylist_index[cur_studylist] + 1) < studylist_count[cur_studylist] ){
					studylist_index[cur_studylist]++;
					SetupScreenTextEdict(studylist[cur_studylist][studylist_index[cur_studylist]].index);// NEW
				}
				break;
			}
			break;
		}// End if (prev_game_status == GS_SHOW_STUDYLIST)
		break;
	}// End switch( game_status )
}// End Func void CheckButtons(void);

int cursorInRange(int cx, int cy, int x0, int y0, int x1, int y1){
	if ( (cx >= x0) && (cx <= x1) && (cy >= y0) && (cy <= y1) )
		return 1;
	else
		return 0;
}

// Return Value: If a menu item was clicked, returns the item number,
//               else returns RTN_OK.
int CheckMenu(int *tmp_game_status, OSL_CONTROLLER *keys){
	int menu_sel_index = -1;
	int i = 0, rtn = 0;
	char tmpbuff[512] = "\0";

	
	if (menu_status == M_OPEN){
		// Menu is shown as a band at the bottom of the screen.
		// Buttons can be selected from the menu - What is shown depends
		// Check to see if cursor hovering over an icon.
		menu_sel_index = -1;
		for (i = 0; i < MAX_MENU_ICON; i++){
			if ( mnu_icon[i].enabled && cursorInRange(cursor.x, cursor.y, mnu_icon[i].x - MENU_ICON_OFFSET, mnu_icon[i].y - MENU_ICON_OFFSET, mnu_icon[i].x + MENU_ICON_WIDTH + MENU_ICON_OFFSET, mnu_icon[i].y + MENU_ICON_HEIGHT + MENU_ICON_OFFSET) ){
				mnu_icon[i].mouse_over = TRUE;
				// Only assign once.
				if (menu_sel_index == -1)
					menu_sel_index = i;
			}else{
				mnu_icon[i].mouse_over = FALSE;
			}
		}

		if (keys->pressed.start || keys->pressed.circle){
			menu_status = M_CLOSING;
			return 0;
		}
		if (keys->pressed.cross){

			if (menu_sel_index != -1){
				if (!mnu_icon[menu_sel_index].enabled)
					return 0;
			}

			switch( menu_sel_index ){
					case MENU_EXIT:
						rtn = oslMessageBox("Exit from PSPEdict?", "Quit?", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_YES, OSL_KEY_CIRCLE, OSL_MB_NO, 0, 0));
						if (rtn == OSL_MB_YES){
							osl_quit = TRUE;
						}else{// Just close the menu
							menu_status = M_CLOSED;
						}
						rtn = MENU_EXIT;
						break;
					case MENU_SAVE:
							rtn = SaveFoundSet();// Returns RTN_OK success, > 0 error.
							switch(rtn){
								case RTN_OK:
									oslMessageBox("Successfully saved the Found Set.", "Saved!", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
								case SAV_NO_FOUNDSET:
									oslMessageBox("No Found Set exists. Nothing to save.", "No Found Set", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
								case SAV_CREATEFILELIST_ERR:
									oslMessageBox("Unable to check for existing Found Set files due to an error. Check sub directory \"saved_finds\" exists. Aborting save.", "Filelist Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
								case SAV_FOUNDSET_EXISTS:
									oslMessageBox("The current Found Set has already been saved. Aborting save.", "Already Exists", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
								case SAV_FOPEN_ERR:
									oslMessageBox("Unable to create the Found Set file. Aborting save.", "File Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
								case SAV_FWRITE_COUNT_ERR:
								case SAV_FWRITE_LIST_ERR:
									oslMessageBox("Unable to write the Found Set to file. Aborting save.", "Write Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
							}
							menu_status = M_CLOSED;
							rtn = MENU_SAVE;
						break;
					case MENU_LOAD: // Load found set. - This will now be specific to the corresponding EDICT file.
						// Set game_status to GS_SHOW_FILES.
						// When user actually selects a file, then LoadFoundSet() will be called.
						// Create file list again.
						switch(cur_edict){
							case EDICT_ORIG:
								rtn = CreateFileList(saved_finds_file_path, AAID_PEL);
								break;
							case EDICT_COMP:
								rtn = CreateFileList(saved_finds_file_path, AAID_PE2);
								break;
							case EDICT_LEGAL:
								rtn = CreateFileList(saved_finds_file_path, AAID_PE3);
								break;
							case EDICT_LIFE:
								rtn = CreateFileList(saved_finds_file_path, AAID_PE4);
								break;
							case EDICT_CUSTOM:
								rtn = CreateFileList(saved_finds_file_path, AAID_PE5);
								break;
						}
						if (rtn == 0){// Error occured.
							switch(create_file_list_errno){
								case CREATE_FL_MALLOC_ERR:
									oslMessageBox("Unable to allocate memory for Load File list. Aborting Load.", "Memory Allocation Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
								case CREATE_FL_DIR_ERR:
									oslMessageBox("Error opening sub directory, \"saved_finds\". Please ensure the directory exists in the same folder as the EBOOT.PBP. Aborting Load.", "Directory Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									break;
							}
							menu_status = M_CLOSED;
							rtn = MENU_LOAD;
							break;
						}

						SetGS(GS_LEVEL_04, GS_SHOW_FILES, TRUE); // New 24/10/09
						menu_status = M_CLOSED;
						rtn = MENU_LOAD;
						break;
					case MENU_EDICT_LOAD:
						SetGS(GS_LEVEL_04, GS_SHOW_EDICT_FILES, TRUE); // New 24/10/09
						menu_status = M_CLOSED;
						rtn = MENU_EDICT_LOAD;
						break;
					case MENU_SEARCH:
						// Go to Start of current list.
						switch( game_status ){
							case GS_SHOW_KANJIDICT:
								kdict_index = 0;
								break;
							case GS_PLAYING:
								kanji_index = 0;
								break;
							case GS_SHOW_RESULTS:
								if (found_count > 0)
									found_index = 0;// CHECK ME
								break;
							case GS_SHOW_STUDYLIST:
								studylist_index[cur_studylist] = 0;
								break;
							case GS_SHOW_SEL:
								// Need to know which list to reset.
								if (prev_game_status == GS_PLAYING){
									kanji_index = 0;
									SetupScreenTextEdict(kanji_index);
								}else{
									if (prev_game_status == GS_SHOW_KANJIDICT){
										kdict_index = 0;
										SetupScreenTextKdic();
									}else{
										if (prev_game_status == GS_SHOW_RESULTS){
											if (found_count > 0){
												found_index = 0;// CHECK ME
												SetupScreenTextEdict(found_list[found_index]);
											}
										}else{
											if (prev_game_status == GS_SHOW_STUDYLIST){
												studylist_index[cur_studylist] = 0;
												SetupScreenTextEdict(studylist[cur_studylist][studylist_index[cur_studylist]].index);// NEW
											}
										}
									}
								}
								break;
						}// End switch( game_status )
						menu_status = M_CLOSED;
						rtn = MENU_SEARCH;
						break;
					case MENU_OPT:
						// Options
						*tmp_game_status = game_status;
						SetGS(GS_LEVEL_05, GS_OPTIONS, TRUE); // New 24/10/09
						menu_status = M_CLOSED;
						rtn = MENU_OPT;
						break;
					case MENU_FIND:
						SetGS(GS_LEVEL_02, GS_SHOW_RESULTS, TRUE); // New 24/10/09
						menu_status = M_CLOSED;
						rtn = MENU_FIND;
						break;
					case MENU_KDICT:
						SetBaseGS(GS_SHOW_KANJIDICT); // New 24/10/09
						menu_status = M_CLOSED;
						rtn = MENU_KDICT;
						break;
					case MENU_EDICT: // New 24/10/09
						SetBaseGS(GS_PLAYING); // New 24/10/09
						menu_status = M_CLOSED;
						rtn = MENU_EDICT;
						break;
					case MENU_STUDY:
						SetGS(GS_LEVEL_02, GS_SHOW_STUDYLIST, TRUE); // New 24/10/09
						menu_status = M_CLOSED;
						rtn = MENU_STUDY;
						break;
					case MENU_STUDY_ADD:
						// Should only happen in strict circumstances.
						if ((game_status == GS_PLAYING) || (game_status == GS_SHOW_RESULTS) || ((game_status == GS_SHOW_SEL) && (prev_game_status == GS_PLAYING || prev_game_status == GS_SHOW_RESULTS))){
							if (game_status == GS_PLAYING){
								rtn = AddToStudyList(cur_studylist, kanji_index);
							}else{
								if (game_status == GS_SHOW_RESULTS){
									rtn = AddToStudyList(cur_studylist, found_list[found_index]);
								}else{
									rtn = AddToStudyList(cur_studylist, ((prev_game_status == GS_PLAYING) ? kanji_index : found_list[found_index]));
								}
							}

							if (rtn == RTN_OK){
								sprintf(tmpbuff, "The selected entry has been added to Study List %d.", cur_studylist + 1);
								oslMessageBox(tmpbuff, "Added to Study List", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_ADD;
								break;
							}
							if (rtn == STUDY_LIST_FULL_ERR){
								// Show list was full
								sprintf(tmpbuff, "Unable to add entry to Study List %d, the list is full.", cur_studylist + 1);
								oslMessageBox(tmpbuff, "Study List Full", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_ADD;
								break;
							}
							if (rtn == STUDY_LIST_ENTRY_EXISTS_ERR){
								sprintf(tmpbuff, "The entry has already been added to Study List %d.", cur_studylist + 1);
								oslMessageBox(tmpbuff, "Entry Already added", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_ADD;
								break;
							}

							// If we got here, must be bad. Disable the use of Study Lists.
							studylist_enabled = FALSE;
							sprintf(tmpbuff, "There was an error adding the entry to Study List %d. The study list may have become corrupt. You may need to delete Study List %d from your memory stick.", cur_studylist + 1, cur_studylist + 1);
							oslMessageBox(tmpbuff, "Study List Error - Add", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
							menu_status = M_CLOSED;
							rtn = MENU_STUDY_ADD;
							break;							
						}// End if able to add.
						menu_status = M_CLOSED;
						rtn = MENU_STUDY_ADD;
						break;
					case MENU_STUDY_MINUS:
						// Should only show up if game_status == GS_SHOW_STUDYLIST
						// If we're on GS_SHOW_SEL, may need to move the studylist index.
						if (game_status == GS_SHOW_STUDYLIST || (game_status == GS_SHOW_SEL && prev_game_status == GS_SHOW_STUDYLIST)){
							rtn = DelFromStudyList(cur_studylist, studylist_index[cur_studylist]); // Use a negative list_index to delete all.
							// If we were on GS_SHOW_SEL, and now don't have any entries, need to move to GS_SHOW_STUDYLIST
							if (rtn == RTN_OK){
								sprintf(tmpbuff, "The selected entry has been deleted from Study List %d.", cur_studylist + 1);
								oslMessageBox(tmpbuff, "Entry Deleted", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								if (studylist_count[cur_studylist] < 1){
									if (game_status == GS_SHOW_SEL)
										ReturnToPrevGS(); // New 24/10/09
								}
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_MINUS;
								break;
							}
							if (rtn == STUDY_LIST_EMPTY_ERR){
								// Show list was full
								sprintf(tmpbuff, "Study List %d is empty. No entries to delete.", cur_studylist + 1);
								oslMessageBox(tmpbuff, "Study List Empty", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_MINUS;
								break;
							}

							// If we got here, must be bad. Disable the use of Study Lists.
							studylist_enabled = FALSE;
							sprintf(tmpbuff, "There was an error deleting an entry from Study List %d. The study list may have become corrupt. You may need to delete Study List %d from your memory stick.", cur_studylist + 1, cur_studylist + 1);
							oslMessageBox(tmpbuff, "Study List Error - Delete", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
							// Also, as we're on GS_SHOW_STUDYLIST or GS_SHOW_SEL, if there is an error,
							// will need to move to another view. Probably go to GS_PLAYING.
							SetBaseGS(GS_PLAYING); // New 24/10/09
							menu_status = M_CLOSED;
							rtn = MENU_STUDY_MINUS;
							break;
						}
						menu_status = M_CLOSED;
						rtn = MENU_STUDY_MINUS;
						break;
					case MENU_STUDY_MINUS_ALL:
						if (game_status == GS_SHOW_STUDYLIST || (game_status == GS_SHOW_SEL && prev_game_status == GS_SHOW_STUDYLIST)){
							// Need to check with user if they wish to proceed.
							if (studylist_count[cur_studylist] < 1){
								sprintf(tmpbuff, "Study List %d is empty. No entries to delete.", cur_studylist + 1);
								oslMessageBox(tmpbuff, "Study List Empty", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_MINUS_ALL;
								break;
							}
							sprintf(tmpbuff, "Delete all entries in Study List %d?", cur_studylist + 1);
							rtn = oslMessageBox(tmpbuff, "Confirm Delete All", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, OSL_KEY_CIRCLE, OSL_MB_CANCEL, 0, 0));

							if (rtn == OSL_MB_OK){
								rtn = DelFromStudyList(cur_studylist, -1); // Use a negative list_index to delete all.

								// If we were on GS_SHOW_SEL, and now don't have any entries, need to move to GS_SHOW_STUDYLIST
								if (rtn == RTN_OK){
									sprintf(tmpbuff, "All entries have been deleted from Study List %d.", cur_studylist + 1);
									oslMessageBox(tmpbuff, "Entry Deleted", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									if (studylist_count[cur_studylist] < 1){
										if (game_status == GS_SHOW_SEL)
											ReturnToPrevGS(); // New 24/10/09
									}
									menu_status = M_CLOSED;
									rtn = MENU_STUDY_MINUS_ALL;
									break;
								}
								if (rtn == STUDY_LIST_EMPTY_ERR){
									// Show list was full
									sprintf(tmpbuff, "Study List %d is empty. No entries to delete.", cur_studylist + 1);
									oslMessageBox(tmpbuff, "Study List Empty", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
									menu_status = M_CLOSED;
									rtn = MENU_STUDY_MINUS_ALL;
									break;
								}

								// If we got here, must be bad. Disable the use of Study Lists.
								studylist_enabled = FALSE;
								sprintf(tmpbuff, "There was an error deleting all entries from Study List %d. The study list may have become corrupt. You may need to delete Study List %d from your memory stick.", cur_studylist + 1, cur_studylist + 1);
								oslMessageBox(tmpbuff, "Study List Error - Delete All", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								// Also, as we're on GS_SHOW_STUDYLIST or GS_SHOW_SEL, if there is an error,
								// will need to move to another view. Probably go to GS_PLAYING.
								SetBaseGS(GS_PLAYING); // New 24/10/09
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_MINUS_ALL;
								break;

							}else{
								oslMessageBox("Delete All has been aborted. No records deleted.", "Delete All - Aborted", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
								menu_status = M_CLOSED;
								rtn = MENU_STUDY_MINUS_ALL;
								break;
							}
						}
						menu_status = M_CLOSED;
						rtn = MENU_STUDY_MINUS_ALL;
						break;
			}// End switch( menu_sel_index )
		}// End if (keys->pressed.cross)
		return rtn;//0;
	}// End if (menu_status == M_OPEN)

	return RTN_OK;
}// int CheckMenu(int *tmp_game_status, OSL_CONTROLLER *keys)

// This function is run to check if we can enter the Kanji popup mode.
// The other function, CheckKanjiPopupInput is run to check for input events
// once the popup is shown.
void CheckKanjiPopup(void){
	int kmatch_index = 0;
	int rtn = 0;
	int i = 0;


	// Testing to see if this works.
	// Don't do anything if the menu is active/opening/closing
	if (menu_status != M_CLOSED){
		// Just return? Check below.
		cursor_tick = 0;
		kdict_shown = FALSE;
		return;
	}

	// For show Kanji thingy
	// Need to build in tollerance for "wandering" analogue sticks? Don't think so.
	if (game_status >= GS_PLAYING){
		if ((cursor.x == cursor.origx) && (cursor.y == cursor.origy) && !show_kdict){
			cursor_tick++;
			if (cursor_tick > KDICT_SHOW_MAXTICKS && !kdict_shown){// Don't show again if we haven't moved the cursor from last time.

				// Where we look depends on the game_status
				if ( game_status == GS_SHOW_SEL ){
					if (prev_game_status == GS_PLAYING || prev_game_status == GS_SHOW_RESULTS || prev_game_status == GS_SHOW_STUDYLIST){
						if (screenkanji[0].index != -1){// Has kanji
							// Difficulty is that kanji might be on more than one line.
							// -- This has now been implimented.

							kmatch_index = -1;
							for (i = 0; i < MAX_SHOWN_F; i++){
								if (screenkanji[i].index != -1){// Has kanji, possibly
									if ( cursorInRange(cursor.x, cursor.y, screenkanji[i].start_x, screenkanji[i].start_y, screenkanji[i].start_x + (screenkanji[i].num_chars * kchar_width), screenkanji[i].start_y + LINE_SPACE_Y) ){
										kmatch_index = i;
										break;
									}
								}else
									break;
							}


							//if ( cursorInRange(cursor.x, cursor.y, screenkanji[0].start_x, screenkanji[0].start_y, screenkanji[0].start_x + (screenkanji[0].num_chars * kchar_width), screenkanji[0].start_y + LINE_SPACE_IF_Y) ){
							if (kmatch_index > -1 && kmatch_index < MAX_SHOWN_F){ // Find the exact character.

								// Find the position of the exact character in the string.
								rtn = (int)((cursor.x - screenkanji[kmatch_index].start_x) / kchar_width) * 2;
								//rtn = (int)((cursor.x - screenkanji[0].start_x) / kchar_width) * 2;
								// need to account for more than one line!
								// TODO - I think this is causing a problem for the new example sentences...
								// How do we tell the ukanji lines from the TC lines. TC lines don't need to
								// account for prev lines.
								if (screenkanji[kmatch_index].kanji_type != TC_KANJI){
									for (i=kmatch_index-1; i >=0; i--){
										rtn += screenkanji[i].num_chars * 2;
									}
									// Once found, can then search for it.
									strncpy((char *)search_kanji, (char *)kanji[screenkanji[/*0*/kmatch_index].index].k_kanji + rtn, 2);
								}else{
									strncpy((char *)search_kanji, screen_text[screenkanji[kmatch_index].index].text + rtn, 2);
								}
								search_kanji[2] = 0;
								// Is it a kanji?
								if (search_kanji[0] >= 0x87){
									// Get the index
									kdict_search_index = search_kanjidict(search_kanji);
									if (kdict_search_index > -1){
										show_kdict = TRUE; // Can be exited by pressing "Circle" button.
										SetupScreenTextPopup();// NEW 12/10/09
										kanji_popup_cur_y = 0;// NEW 15/10/09
										kdict_index = kdict_search_index; // NEW 14/10/09
										cursor_tick = 0;
										show_kdict_tick = 0;
										show_kdict_x = cursor.x;
										show_kdict_y = cursor.y;
									}
								}// End if (search_kanji[0] >= 0x87)
							}// End if ( cursorInRange(cursor.x, cursor.y, screenkanji[0].start_x, screenkanji[0].start_y, screenkanji[0].start_x + (screenkanji[0].num_chars * kchar_width), screenkanji[0].start_y + LINE_SPACE_IF_Y) )
						}// End if (screenkanji[0].index != -1)
					}// End if (prev_game_status == GS_PLAYING || prev_game_status == GS_SHOW_RESULTS || prev_game_status == GS_SHOW_STUDYLIST)

				}else{// GS_PLAYING, GS_SHOW_RESULTS or GS_SHOW_STUDYLIST
					// Will only be on one line.
					kmatch_index = -1;
					for (i = 0; i < MAX_SHOWN_F; i++){
						if (screenkanji[i].index != -1){// Has kanji, possibly
							if ( cursorInRange(cursor.x, cursor.y, screenkanji[i].start_x, screenkanji[i].start_y, screenkanji[i].start_x + (screenkanji[i].num_chars * kchar_width), screenkanji[i].start_y + LINE_SPACE_Y) ){
								kmatch_index = i;
								break;
							}
						}
					}
					if (kmatch_index > -1 && kmatch_index < MAX_SHOWN_F){ // Find the exact character.
						// Find the position of the exact character in the string.
						rtn = (int)((cursor.x - screenkanji[kmatch_index].start_x) / kchar_width) * 2;
						// Once found, can then search for it.
						strncpy((char *)search_kanji, (char *)kanji[screenkanji[kmatch_index].index].k_kanji + rtn, 2);
						search_kanji[2] = 0;
						// Is it a kanji?
						if (search_kanji[0] >= 0x87){
							// Get the index
							kdict_search_index = search_kanjidict(search_kanji);
							if (kdict_search_index > -1){
								show_kdict = TRUE; // Can be exited by pressing "Circle" button.
								SetupScreenTextPopup();// NEW 12/10/09
								kanji_popup_cur_y = 0;// NEW 15/10/09
								kdict_index = kdict_search_index; // NEW 14/10/09
								cursor_tick = 0;
								show_kdict_tick = 0;
								show_kdict_x = cursor.x;
								show_kdict_y = cursor.y;
							}
						}// End if (search_kanji[0] >= 0x87)
					}// End if (kmatch_index > -1 && kmatch_index < MAX_SHOWN_F)
				}// End if ( game_status == GS_SHOW_SEL )
			}// End if (cursor_tick > KDICT_SHOW_MAXTICKS)
		}else{
			cursor_tick = 0;
			kdict_shown = FALSE;
		}// End if ((cursor.x == cursor.origx) && (cursor.y == cursor.origy) && !show_kdict)
		//}// End if (menu_active == TRUE)
	}else{
		cursor_tick = 0;
		kdict_shown = FALSE;
	}// End if (game_status >= GS_PLAYING)

}// void CheckKanjiPopup(void);

int search_kanjidict(u8 *search_str){
	int found = -1;
	int i = 0;

	// May be a faster way of searching if
	// we check the second character (second byte)?
	// NEED TO CHECK
	for (i = 0; i < MAX_KANJIDICT; i++){
		if (strcmp((char *)kanjidict[i].kanji, (char *)search_str) == 0){
			found = i;
			break;
		}
	}

	return found;
}// int search_kanjidict(u8 *search_str)

// It is a condition that buttons have been checked beforehand if the
// mouse is hovering over them.
void ScreenUpdate(void){
	int i = 0, j = 0, tmpint = 0;
	//int rtn = 0;
	int show_fmax = 0;
	int y = 0, x = 0;
	int osk_skip = 0; // Used for OSK
	int osk_done = 0; // Used for OSK
	char tmpbuff[512] = "\0";
	unsigned int ucs2buff[128] = {0,};
	int str_width = 0, str_len = 0;

	int go_forward = TRUE;
	int x_space_left = 0;
	int num_printable_chars = 0;
	char tmp_ch = '\0';
	//char tmp_SJIS[3] = "\0";
	char is_ascii = TRUE;

	// For Showing Found Set files
	char *pFound_Char = NULL;
	char *pos = NULL;
	char placeholder_file_chars_ASCII[] = "[]#+~^{}!\0";
	char placeholder_file_chars_SJIS[] = "#!\0";

	//int doonce = TRUE;


	switch( game_status ){
		case GS_INIT: // Not used for this project.
		case GS_INTRO:// Same as above.
			break;
		case GS_TEST: // Used for testing purposes only
			//oslClearScreen(RGB(250, 250, 250));
			//if (tmpTC.file_pos == -1)
			//	i = LoadTCLine(fTCName, kanji[26].TCfilepos[0], &tmpTC);

			//oslSetFont(jpn0);
			//oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			//y = HELP_Y;
			//i = IFPrintTextBoxSJIS(HELP_X, y, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), MAX_PRINT_LINES * LINE_SPACE_IF_Y, (char *)tmpTC.j_scentence);
			////rtn = IFPrintTextBoxSJIS(HELP_X, y, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), MAX_PRINT_LINES * LINE_SPACE_IF_Y, (char *)kanji[index].k_kanji);
			//y += (i+1)*LINE_SPACE_IF_Y;
			//i = IFPrintTextBox(HELP_X, y, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), MAX_PRINT_LINES * LINE_SPACE_IF_Y, tmpTC.e_scentence);

			break;
		case GS_SETUP:
			//oslDrawImageXY(bkg, 0, 0);
			oslClearScreen(RGB(250, 250, 250));
			DrawTitle(TITLE_SCREENPOS_X, TITLE_SCREENPOS_Y);
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(20,200,"Files loaded: OK.");
			oslIntraFontSetStyle(ltn0, 0.7f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(20,240,"X - Continue...");
			i = oslGetStringWidth("( ) - Help");
			oslDrawString(AA_SCREEN_WIDTH - i - 20,240,"( ) - Help");
			///*if (doonce){
			//	doonce = FALSE;
			//	oslSetFont(ltn0);
			//	oslIntraFontSetStyle(ltn0, 0.7f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			//	oslDebug("Length of\n\"%s\"\nis %d.", "Any position", oslGetStringWidth("Any position"));
			//	oslDebug("Length of\n\"%s\"\nis %d.", "From start", oslGetStringWidth("From start"));

			//	oslDebug("Length of\n\"%s\"\nis %d.", "Match whole string?", oslGetStringWidth("Match whole string?"));
			//	oslDebug("Length of\n\"%s\"\nis %d.", "(english only)", oslGetStringWidth("(english only)"));
			//	oslDebug("Length of\n\"%s\"\nis %d.", "wildcard \'*\' at end", oslGetStringWidth("wildcard \'*\' at end"));
			//	oslDebug("Length of\n\"%s\"\nis %d.", "still allowed.", oslGetStringWidth("still allowed."));
			//}*/
			break;
		case GS_HELP:
			DrawScreenTextHelp();
			break;
		case GS_SHOW_FILES: 
			// For loading saved found sets
			oslClearScreen(RGB(218, 242, 242));

			// Draw which edict file we have.
			oslSetFont(ltn0);
			memset(tmpbuff, 0, sizeof(tmpbuff));
			switch(cur_edict){
				case EDICT_ORIG:
					strcpy(tmpbuff, "General");
					break;
				case EDICT_COMP:
					strcpy(tmpbuff, "Computer Terms");
					break;
				case EDICT_LEGAL:
					strcpy(tmpbuff, "Legal Terms");
					break;
				case EDICT_LIFE:
					strcpy(tmpbuff, "Life Sciences");
					break;
				case EDICT_CUSTOM:
					strcpy(tmpbuff, "Custom");
					break;
			}
			oslIntraFontSetStyle(ltn0, 0.7f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
			i = oslGetStringWidth(tmpbuff);
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_IF_Y + 3, tmpbuff);

			// Draw Text
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

			if (num_files < 1){
				i = oslGetStringWidth("No Found Set(s) to Load\0");
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,"No Found Set(s) to Load\0");
				//oslDrawString(HELP_X, HELP_Y,"No Found Set(s) to Load\0");
				oslIntraFontSetStyle(ltn0, 0.7f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				oslDrawString(20,260,"Press CIRCLE to Cancel.");
				oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				break;
			}

			oslIntraFontSetStyle(ltn0, 0.6f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
			i = oslGetStringWidth("Select file. Press CROSS to Load, CIRCLE to Cancel, SQUARE to Delete.");
			oslDrawString((AA_SCREEN_WIDTH - i)/2, 240,"Select file. Press CROSS to Load, CIRCLE to Cancel, SQUARE to Delete.");

			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			i = oslGetStringWidth("Load Found Set");
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,"Load Found Set");
			oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

			y = (fname_index / MAX_SHOWN_F) * MAX_SHOWN_F;

			if (num_files > y + (MAX_SHOWN_F-1)){
			   show_fmax = y + (MAX_SHOWN_F-1);
			}
			else{ // End is in this lot
			   show_fmax = num_files - 1;
			}

			if (num_files > MAX_SHOWN_F)
				DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, fname_index, num_files);

			i = 0;
			// Show scrolling of files if more than show_fmax
			for (; y < show_fmax + 1; y++,i++){
				if (fname[y] == NULL || (HELP_Y + (LINE_SPACE_Y * (2 + i))) >= (AA_SCREEN_HEIGHT - 10))
					break;

				// Need to print the filename correctly. It could be an SJIS string or ASCII, also it
				// may have illegal filename characters stripped, so need to restore the string.
				strcpy(tmpbuff, fname[y]);
				str_len = strlen(tmpbuff);

				// Remove the file extention
				pos = strrchr(tmpbuff, '.');
				if (pos == NULL){ // Not found.
					//printf("ShowFiles()<<< Error in strrchr(). Aborting. >>>\n");
					oslMessageBox("GS_SHOW_FILES - Error in strrchr().", "Display Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
					break;
				}

				pos--;
				strncpy(tmpbuff, tmpbuff, pos - tmpbuff);
				tmpbuff[str_len-4] = '\0';
				str_len = strlen(tmpbuff);
				
				if (tmpbuff[0] == '0'){
					tmpint = 0; // Used to decide what type of string to print later.
					// Japanese SJIS string
					// Take out the '0' char from index 0.
					strcpy(tmpbuff, tmpbuff+1);
					str_len--;
					// Convert placeholder characters back to illegal characters
					for (j = 0; j < str_len; j++){
						pFound_Char = strchr(placeholder_file_chars_SJIS, tmpbuff[j]);
						if (pFound_Char != NULL){
							// Need to replace the illegal character
							switch(tmpbuff[j]){
								// Found in SJIS
								/*bad*/case '#': tmpbuff[j] = '\\';// OK - doesn't exist in SJIS
									break;
								/*bad*/case '!': tmpbuff[j] = '|';// OK - doesn't exist in SJIS
									break;
							}// End switch(tmpbuff[i]);
						}
					}
				}else{
					// English ASCII string
					tmpint = 1; // Used to decide what type of string to print later.
					// Convert placeholder characters back to illegal characters
					for (j = 0; j < str_len; j++){
						pFound_Char = strchr(placeholder_file_chars_ASCII, tmpbuff[j]);
						if (pFound_Char != NULL){
							// Need to replace the illegal character
							switch(tmpbuff[j]){
								// Found in SJIS
								/*bad*/case '[': tmpbuff[j] = '\\';// Cant use - exists in SJIS string.
									break;
								/*OK*/case ']': tmpbuff[j] = '/';// Cant use
									break;
								/*OK*/case '#': tmpbuff[j] = ':';// OK
									break;
								/*OK*/case '+': tmpbuff[j] = '*';// OK
									break;
								/*OK*/case '~': tmpbuff[j] = '\?';// Cant use
									break;
								/*OK*/case '^': tmpbuff[j] = '\"';// Cant use
									break;
								/*OK*/case '{': tmpbuff[j] = '<';// Cant use
									break;
								/*OK*/case '}': tmpbuff[j] = '>';// Cant use
									break;
								/*bad*/case '!': tmpbuff[j] = '|';// OK
									break;
							}// End switch(tmpbuff[i]);
						}
					}
				}// End if (fname[i][0] == '0')
				
				if (tmpint){ // English ASCII
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				}else{ // Japanese SJIS
					oslSetFont(jpn0);
					oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				}
				oslDrawStringf(HELP_X, HELP_Y + LINE_SPACE_Y + (LINE_SPACE_Y * (2 + i)),"%s", tmpbuff);


				if (y == fname_index){
					// Show selected file
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					oslDrawString(HELP_X - 20, ((HELP_Y - 2) + LINE_SPACE_Y + (LINE_SPACE_Y * (2 + i))),"->");
				}
			}
			break;
		case GS_SHOW_EDICT_FILES:
				oslClearScreen(RGB(218, 242, 242));

				// Draw Text
				oslSetFont(ltn0);
				oslIntraFontSetStyle(ltn0, 0.6f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
				i = oslGetStringWidth("Select file. Press CROSS to Load, CIRCLE to Cancel.");
				oslDrawString((AA_SCREEN_WIDTH - i)/2, 240,"Select file. Press CROSS to Load, CIRCLE to Cancel.");

				oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				i = oslGetStringWidth("Load Edict File");
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,"Load Edict File");
				oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

				for (i = 0; i < MAX_EDICT; i++){
					if (cur_edict == i){
						oslIntraFontSetStyle(ltn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
					}else{
						oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					}

					// Show filename without extention.
					strcpy(tmpbuff, fKanjiName[i]);
					str_len = strlen(tmpbuff);

					// Remove the file extention
					pos = strrchr(tmpbuff, '.');
					if (pos == NULL){ // Not found.
						oslMessageBox("GS_SHOW_EDICT_FILES - Error in strrchr().", "Display Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
						break;
					}

					pos--;
					strncpy(tmpbuff, tmpbuff, pos - tmpbuff);
					tmpbuff[str_len-4] = '\0';
					//str_len = strlen(tmpbuff);

					oslDrawStringf(HELP_X, HELP_Y + LINE_SPACE_Y + (LINE_SPACE_Y * (2 + i)),"%s", /*fKanjiName[i]*/tmpbuff);

					if (i == cur_edict_index){
						oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
						oslDrawString(HELP_X - 20, ((HELP_Y - 2) + LINE_SPACE_Y + (LINE_SPACE_Y * (2 + i))),"->");
					}

				}

				break;
		case GS_LOAD_EDICT:
				oslClearScreen(RGB(250, 250, 250));
				DrawTitle(TITLE_SCREENPOS_X, TITLE_SCREENPOS_Y);
				// Draw Text
				oslSetFont(ltn0);
				oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				if (studylist_enabled)
					oslDrawString(20,80,"Loading new Study Lists...");
				else
					oslDrawString(20,160,"Loading new Edict file...");

				break;
		case GS_OPTIONS:
			oslClearScreen(RGB(250, 230, 250));
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

			i = oslGetStringWidth("Options");
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,"Options");
			oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);
			oslIntraFontSetStyle(ltn0, 0.5f,RED,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(HELP_X, AA_SCREEN_HEIGHT - LINE_SPACE_IF_Y,"Note: Any changes will be immediately actioned when circle is pressed.");
			oslIntraFontSetStyle(ltn0, 0.7f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 2),"Kana/Kanji Searches");
			oslDrawString((AA_SCREEN_WIDTH / 2) + HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 2),"Sort Find");

			// Draw Kana/Kanji search Radio Buttons
			if (search_type == OPT_SEARCH_ANY){
				// Line 1
				spr_cur->x = HELP_X;
				spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 3);
				oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_ON_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
				oslDrawImage(spr_cur);
				// Line 2
				spr_cur->x = HELP_X;
				spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 4);
				oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
				oslDrawImage(spr_cur);
			}else{
				// Line 1
				spr_cur->x = HELP_X;
				spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 3);
				oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
				oslDrawImage(spr_cur);
				// Line 2
				spr_cur->x = HELP_X;
				spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 4);
				oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_ON_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
				oslDrawImage(spr_cur);
			}

			//oslIntraFontSetStyle(ltn0, 0.7f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			
			oslDrawString(OPT_RADIO_01_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 3) + LINE_IF_OFFSET_Y,"Any position");
			oslDrawString(OPT_RADIO_02_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 4) + LINE_IF_OFFSET_Y,"From start");

			// Draw foundset and study list sort Radio Buttons
			switch (sort_type){
				case OPT_SORT_ORIG:
					// Line 1
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 3);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_ON_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					// Line 2
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 4);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					// Line 3
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 5);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					break;
				case OPT_SORT_LENGTH:
					// Line 1
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 3);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					// Line 2
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 4);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_ON_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					// Line 3
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 5);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					break;
				case OPT_SORT_READING:
					// Line 1
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 3);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					// Line 2
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 4);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_OFF_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					// Line 3
					spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
					spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 5);
					oslSetImageTileSize(spr_cur, OPT_SCN_RADIO_SRC_X, OPT_SCN_RADIO_ON_SRC_Y, OPT_SCN_RADIO_WIDTH, OPT_SCN_RADIO_HEIGHT);
					oslDrawImage(spr_cur);
					break;
			}

			oslDrawString(OPT_RADIO_03_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 3) + LINE_IF_OFFSET_Y,"Original");
			oslDrawString(OPT_RADIO_04_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 4) + LINE_IF_OFFSET_Y,"Length");
			oslDrawString(OPT_RADIO_05_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 5) + LINE_IF_OFFSET_Y,"Reading");

			// Draw Check Box 01
			//oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 6),"English Searches");
			spr_cur->x = HELP_X;
			spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 7);//7);
			oslSetImageTileSize(spr_cur, OPT_SCN_CHECK_SRC_X, match_whole ? OPT_SCN_CHECK_ON_SRC_Y : OPT_SCN_CHECK_OFF_SRC_Y, OPT_SCN_CHECK_WIDTH, OPT_SCN_CHECK_HEIGHT);
			oslDrawImage(spr_cur);
			oslDrawString(OPT_CHECK_01_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * /*7*/7) + LINE_IF_OFFSET_Y,"Match whole string?");
			oslDrawString(OPT_CHECK_01_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * /*9*/8) + LINE_IF_OFFSET_Y,"wildcard \'*\' at end");
			oslDrawString(OPT_CHECK_01_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * /*10*/9) + LINE_IF_OFFSET_Y,"still allowed(English only).");

			// Draw Check Box 02
			spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
			spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 7);
			oslSetImageTileSize(spr_cur, OPT_SCN_CHECK_SRC_X, show_examples ? OPT_SCN_CHECK_ON_SRC_Y : OPT_SCN_CHECK_OFF_SRC_Y, OPT_SCN_CHECK_WIDTH, OPT_SCN_CHECK_HEIGHT);
			oslDrawImage(spr_cur);
			oslDrawString(OPT_CHECK_02_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * /*7*/7) + LINE_IF_OFFSET_Y,"Show Example");
			oslDrawString(OPT_CHECK_02_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * /*9*/8) + LINE_IF_OFFSET_Y,"Sentences?");
			//oslDrawString(OPT_CHECK_02_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * /*10*/8) + LINE_IF_OFFSET_Y,"still allowed(English only).");

			// Added by Batanen: Draw Check Box 03 for hiding things
			oslDrawString((AA_SCREEN_WIDTH / 2) + HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 9),"Don't Show:");
			
			spr_cur->x = (AA_SCREEN_WIDTH / 2) + HELP_X;
			spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 10);
			oslSetImageTileSize(spr_cur, OPT_SCN_CHECK_SRC_X, kanjiHiding ? OPT_SCN_CHECK_ON_SRC_Y : OPT_SCN_CHECK_OFF_SRC_Y, OPT_SCN_CHECK_WIDTH, OPT_SCN_CHECK_HEIGHT);
			oslDrawImage(spr_cur);
			oslDrawString(OPT_CHECK_03_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 10) + LINE_IF_OFFSET_Y,"Kanji");
			
			spr_cur->x = (AA_SCREEN_WIDTH / 2) + 50 + HELP_X;
			spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 10);
			oslSetImageTileSize(spr_cur, OPT_SCN_CHECK_SRC_X, kanaHiding ? OPT_SCN_CHECK_ON_SRC_Y : OPT_SCN_CHECK_OFF_SRC_Y, OPT_SCN_CHECK_WIDTH, OPT_SCN_CHECK_HEIGHT);
			oslDrawImage(spr_cur);
			oslDrawString(OPT_CHECK_04_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 10) + LINE_IF_OFFSET_Y,"Kana");
			
			spr_cur->x = (AA_SCREEN_WIDTH / 2) + 100 + HELP_X;
			spr_cur->y = HELP_Y + (LINE_SPACE_IF_Y * 10);
			oslSetImageTileSize(spr_cur, OPT_SCN_CHECK_SRC_X, descHiding ? OPT_SCN_CHECK_ON_SRC_Y : OPT_SCN_CHECK_OFF_SRC_Y, OPT_SCN_CHECK_WIDTH, OPT_SCN_CHECK_HEIGHT);
			oslDrawImage(spr_cur);
			oslDrawString(OPT_CHECK_05_TEXT_X, HELP_Y + (LINE_SPACE_IF_Y * 10) + LINE_IF_OFFSET_Y,"Desc");

			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			break;
		case GS_SHOW_OSK:
			// Need to set this to be used for Save Found Set to ensure
			// we use the correct string.
			memset(prev_osk_text, 0, sizeof(prev_osk_text));
			strcpy(prev_osk_text, osk_text);

			// Clear the receiving strings
			memset(osk_text, 0, sizeof(osk_text));
			memset(osk_textUCS2, 0, sizeof(osk_textUCS2));

			osk_skip = osk_done = 0;
			if (osk_language == OSK_ENGLISH)
				oslInitOsk(osk_text, prev_osk_text_eng, MAX_DESCRIP, 1, osk_language);
			else
				oslInitOsk(osk_textUCS2, "\0" /*prev_osk_textUCS2*/, MAX_DESCRIP, 1, osk_language);
			while(!osl_quit && !osk_done){
				if (!osk_skip){
					oslStartDrawing();
					// Draw background image.
					oslDrawImageXY(bkg2, 0, 0);
					if (oslOskIsActive())
						oslDrawOsk();
					if (oslGetOskStatus() == PSP_UTILITY_DIALOG_NONE){
						if (oslOskGetResult() == OSL_OSK_CANCEL){
							if (osk_language != OSK_ENGLISH){
								osk_textUCS2[0] = 0;
							}
							//strcpy(osk_text, "\0");
							memset(osk_text, 0, sizeof(osk_text));
							osk_done = 1;
						}else{
							if (osk_language == OSK_ENGLISH){
								oslOskGetText(osk_text);
								memset(prev_osk_text_eng, 0, sizeof(prev_osk_text_eng));
								strcpy(prev_osk_text_eng, osk_text);
							}else{
								oslOskGetTextUCS2(ucs2buff/*osk_textUCS2*/);
								basic_UCS2toSJIS(ucs2buff, osk_text); // (Instr, Outstr)
								//memset(prev_osk_textUCS2, 0, sizeof(prev_osk_textUCS2));
								//strcpy((char *)prev_osk_textUCS2, (char *)ucs2buff);
								//oslOskGetTextUCS2(prev_osk_textUCS2);
							}
							osk_done = 1;
						}
						oslEndOsk();
					}
					oslEndDrawing();
				}
				oslEndFrame();
				osk_skip = oslSyncFrame();
			}// End while(!osl_quit && !done)

			if (osk_text[0] != '\0'){
				is_finding = FALSE;
				SetGS(GS_LEVEL_04, GS_SEARCHING, TRUE); // New 24/10/09
			}else{// Return from whence we came
				// User cancelled, restore prev search string and language selection
				// as is needed later if user decides to save last search.
				strcpy(osk_text, prev_osk_text);
				osk_language = prev_osk_language;// Prev_osk_language set when OSK language selected.
				ReturnToPrevGS(); // New 24/10/09
			}
			break;
		case GS_SEARCHING:
			//oslClearScreen(RGB(235, 235, 235));
			oslDrawImageXY(bkg2, 0, 0);
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

			i = oslGetStringWidth("Searching");
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,"Searching");
			oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

			oslDrawString(20,80 + (LINE_SPACE_Y * 3),"Searched String:");

			if (osk_language == OSK_ENGLISH){
				oslIntraFontSetStyle(ltn0, 1.0f,WHITE,BLACK,INTRAFONT_ALIGN_LEFT);
				oslDrawString(30,80 + (LINE_SPACE_Y * 4), osk_text);
			}else{
				oslIntraFontSetStyle(jpn0, 1.0f,WHITE,BLACK,INTRAFONT_ALIGN_LEFT);
				oslSetFont(jpn0);
				oslDrawString(30,80 + (LINE_SPACE_Y * 4), osk_text);
			}

			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			oslSetFont(ltn0);
			oslDrawString(20,80 + (LINE_SPACE_Y * 6),"Please wait...");

			break;
		case GS_SHOW_STUDYLIST:
			oslClearScreen(RGB(235, 216, 235));

			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			oslSetFont(ltn0);

			memset(tmpbuff, 0, sizeof(tmpbuff));
			if (studylist_count[cur_studylist] < 1){
				sprintf(tmpbuff, "Study List %d: No Entries", cur_studylist + 1);
				i = oslGetStringWidth(tmpbuff);
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,tmpbuff);
				oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

				// Draw which edict file we have.
				memset(tmpbuff, 0, sizeof(tmpbuff));
				switch(cur_edict){
					case EDICT_ORIG:
						strcpy(tmpbuff, "General");
						break;
					case EDICT_COMP:
						strcpy(tmpbuff, "Computer Terms");
						break;
					case EDICT_LEGAL:
						strcpy(tmpbuff, "Legal Terms");
						break;
					case EDICT_LIFE:
						strcpy(tmpbuff, "Life Sciences");
						break;
					case EDICT_CUSTOM:
						strcpy(tmpbuff, "Custom");
						break;
				}
				oslIntraFontSetStyle(ltn0, 0.7f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
				i = oslGetStringWidth(tmpbuff);
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_IF_Y + 3, tmpbuff);

				oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 5),"Press TRIANGE to search English.\0");
				oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 6),"Press SQUARE to search Japanese (Kana).\0");
				oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 7),"Press CIRCLE to return to Main List.\0");
				break;
			}

			// Show basic list.
			y = (studylist_index[cur_studylist] / MAX_SHOWN_F) * MAX_SHOWN_F;

			if (studylist_count[cur_studylist] > y + (MAX_SHOWN_F-1)){
				show_fmax = y + (MAX_SHOWN_F-1);
			}
			else{ // End is in this lot
				show_fmax = studylist_count[cur_studylist] - 1;
			}

			if (studylist_count[cur_studylist] > MAX_SHOWN_F)
				DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, studylist_index[cur_studylist], studylist_count[cur_studylist]);

			// Found something. Display the results.
			sprintf(tmpbuff, "Study List %d: Showing (%d to %d)", cur_studylist + 1, y+1, (y+10)>studylist_count[cur_studylist]? studylist_count[cur_studylist] : y+10);
			i = oslGetStringWidth(tmpbuff);
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,tmpbuff);
			oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

			// Draw which edict file we have.
			memset(tmpbuff, 0, sizeof(tmpbuff));
			switch(cur_edict){
				case EDICT_ORIG:
					strcpy(tmpbuff, "General");
					break;
				case EDICT_COMP:
					strcpy(tmpbuff, "Computer Terms");
					break;
				case EDICT_LEGAL:
					strcpy(tmpbuff, "Legal Terms");
					break;
				case EDICT_LIFE:
					strcpy(tmpbuff, "Life Sciences");
					break;
				case EDICT_CUSTOM:
					strcpy(tmpbuff, "Custom");
					break;
			}
			oslIntraFontSetStyle(ltn0, 0.7f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
			i = oslGetStringWidth(tmpbuff);
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_IF_Y + 3, tmpbuff);

			ClearScreenKanjiList();
			i = 0;
			for (; y < show_fmax + 1; y++,i++){
				if (/*fname[y] == NULL ||*/ (HELP_LIST_Y + (LINE_SPACE_IF_Y * (2 + i))) >= (AA_SCREEN_HEIGHT - 10))
					break;

				// Show Kanji (if it exists), Kana and lastly Descrip if any space left over.
				x = HELP_X;

				// Show kanji
				if (kanjiHiding == FALSE)
				{
				if (kanji[studylist[cur_studylist][y].index].k_kanji != NULL){
					oslSetFont(jpn0);
					//oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					// If example sentences exist, show text in Blue. // Added 7/11/09.
					if (kanji[studylist[cur_studylist][y].index].TCfilepos[0] != -1 && show_examples)
						oslIntraFontSetStyle(jpn0, 1.0f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
					else
						oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

					if ((str_width = oslGetStringWidth((char *)kanji[studylist[cur_studylist][y].index].k_kanji)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[studylist[cur_studylist][y].index].k_kanji);
						// For showing individual kanji from kanjidict
						screenkanji[i].index = studylist[cur_studylist][y].index;
						screenkanji[i].start_x = x;
						screenkanji[i].start_y = HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)) - SK_OFFSET_Y;
						screenkanji[i].num_chars = str_width / kchar_width;
						x += (str_width + 3);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[studylist[cur_studylist][y].index].k_kanji);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < kchar_width)
							goto check_studylist_index;
						num_printable_chars = x_space_left / kchar_width;
						if ((str_len/2) > (num_printable_chars * 2))// Quick hack assuming always 2bytes
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							if ((u8)tmpbuff[0] < 0x7F){
								is_ascii = TRUE;
								j = 1;
							}else{
								is_ascii = FALSE;
								j = 2;
							}
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j - (is_ascii == TRUE? 1 : 2)] = '\0';
									//if (is_ascii)
									//	tmpbuff[j-1] = '\0';
									//else
									//	tmpbuff[j-2] = '\0';
									break;
								}

								if ((j + (is_ascii == TRUE? 1 : 2) >= str_len)){
									break;
								}
								tmpbuff[j] = tmp_ch;

								if ((u8)tmpbuff[j] < 0x7F){
									is_ascii = TRUE;
									j++;
								}else{
									is_ascii = FALSE;
									j += 2;
								}
							}
						}// End if (go_forward == FALSE)
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
							// For showing individual kanji from kanjidict
							screenkanji[i].index = studylist[cur_studylist][y].index;
							screenkanji[i].start_x = x;
							screenkanji[i].start_y = HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)) - SK_OFFSET_Y;
							screenkanji[i].num_chars = str_width / kchar_width;
						}
						goto check_studylist_index;
					}// Else, won't all fit.
				}// End if (kanji[studylist[cur_studylist][y].index].k_kanji != NULL)
				}
				
				if (kanaHiding == FALSE)
				{
				// Show kana
				if (kanji[studylist[cur_studylist][y].index].k_kana == NULL){
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
					oslDrawString(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"<< NULL >>");
					str_width = oslGetStringWidth("<< NULL >>\0");
					x += (str_width + 3);
				}else{
					oslSetFont(jpn0);
					oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);

					if ((str_width = oslGetStringWidth((char *)kanji[studylist[cur_studylist][y].index].k_kana)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[studylist[cur_studylist][y].index].k_kana);
						x += (str_width + 3);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[studylist[cur_studylist][y].index].k_kana);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < kchar_width)
							goto check_studylist_index;
						num_printable_chars = x_space_left / kchar_width;
						if ((str_len/2) > (num_printable_chars * 2))// Quick hack assuming always 2bytes
							go_forward = TRUE;
						else
							go_forward = FALSE;
						//while ((str_width  >  (AA_SCREEN_WIDTH - x - 15)) && (str_len > 0)){
						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							if ((u8)tmpbuff[0] < 0x7F){
								is_ascii = TRUE;
								j = 1;
							}else{
								is_ascii = FALSE;
								j = 2;
							}
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j - (is_ascii == TRUE? 1 : 2)] = '\0';
									break;
								}

								if ((j + (is_ascii == TRUE? 1 : 2) >= str_len)){
									break;
								}
								tmpbuff[j] = tmp_ch;

								if ((u8)tmpbuff[j] < 0x7F){
									is_ascii = TRUE;
									j++;
								}else{
									is_ascii = FALSE;
									j += 2;
								}
							}
						}// End if (go_forward == FALSE)
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
						}
						goto check_studylist_index;
					}// Else, won't all fit.
				}
				}
				
				if (descHiding == FALSE)
				{
				// Draw what we can of the english description.
				oslSetFont(ltn0);
				//oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				if (kanji[studylist[cur_studylist][y].index].TCfilepos[0] != -1 && show_examples)
				{
					oslIntraFontSetStyle(ltn0, 1.0f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
				}
				else
				{
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				}

				if (kanji[studylist[cur_studylist][y].index].e_descrip != NULL){
					if ((str_width = oslGetStringWidth((char *)kanji[studylist[cur_studylist][y].index].e_descrip)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[studylist[cur_studylist][y].index].e_descrip);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[studylist[cur_studylist][y].index].e_descrip);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < PRINT_CHAR_WIDTH)
							goto check_studylist_index;
						num_printable_chars = x_space_left / PRINT_CHAR_WIDTH;
						if (str_len > (num_printable_chars * 2))
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							j = 1;
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j-1] = '\0';
									break;
								}
								if ((j + 1 >= str_len)){
									break;
								}
								tmpbuff[j++] = tmp_ch;
							}
						}
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
						}
					}
				}
				}
check_studylist_index:
				if (y == studylist_index[cur_studylist]){
					// Show selected file
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					oslDrawString(HELP_X - 20, ((HELP_LIST_Y - 2) + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i))),"->");
				}
			}// End for (; y < show_fmax + 1; y++,i++)
			break;
		case GS_SHOW_RESULTS:
			//oslClearScreen(RGB(235, 235, 235));
			oslDrawImageXY(bkg2, 0, 0);

			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

			if (found_count < 1){
				i = oslGetStringWidth("Found: NONE\0");
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,"Found: NONE\0");
				oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

				// Draw which edict file we have.
				memset(tmpbuff, 0, sizeof(tmpbuff));
				switch(cur_edict){
					case EDICT_ORIG:
						strcpy(tmpbuff, "General");
						break;
					case EDICT_COMP:
						strcpy(tmpbuff, "Computer Terms");
						break;
					case EDICT_LEGAL:
						strcpy(tmpbuff, "Legal Terms");
						break;
					case EDICT_LIFE:
						strcpy(tmpbuff, "Life Sciences");
						break;
					case EDICT_CUSTOM:
					strcpy(tmpbuff, "Custom");
					break;
				}
				oslIntraFontSetStyle(ltn0, 0.7f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
				i = oslGetStringWidth(tmpbuff);
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_IF_Y + 3, tmpbuff);

				oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 5),"Press TRIANGE to search English.\0");
				oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 6),"Press SQUARE to search Japanese (Kana).\0");
				oslDrawString(HELP_X, HELP_Y + (LINE_SPACE_IF_Y * 7),"Press CIRCLE to return to Main List.\0");
				break;
			}

			// Show basic list.
			y = (found_index / MAX_SHOWN_F) * MAX_SHOWN_F;

			if (found_count > y + (MAX_SHOWN_F-1)){
				show_fmax = y + (MAX_SHOWN_F-1);
			}else{ // End is in this lot
				show_fmax = found_count - 1;
			}

			if (found_count > MAX_SHOWN_F)
				DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, found_index, found_count);

			// Found something. Display the results.
			memset(tmpbuff, 0, sizeof(tmpbuff));
			sprintf(tmpbuff, "Found: %d. Showing (%d to %d)", found_count, y+1, (y+10)>found_count? found_count : y+10);
			i = oslGetStringWidth(tmpbuff);
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,tmpbuff);

			if (found_more){
				memset(tmpbuff, 0, sizeof(tmpbuff));
				oslIntraFontSetStyle(ltn0, 0.7f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				sprintf(tmpbuff, "- found more than limit (%d) -", MAX_FOUND - 2);
				i = oslGetStringWidth(tmpbuff);
				oslDrawStringf((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y - 2,"- found more than limit (%d) -", MAX_FOUND - 2);
				oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			}else{
				oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);
			}

			// Draw which edict file we have.
			memset(tmpbuff, 0, sizeof(tmpbuff));
			switch(cur_edict){
				case EDICT_ORIG:
					strcpy(tmpbuff, "General");
					break;
				case EDICT_COMP:
					strcpy(tmpbuff, "Computer Terms");
					break;
				case EDICT_LEGAL:
					strcpy(tmpbuff, "Legal Terms");
					break;
				case EDICT_LIFE:
					strcpy(tmpbuff, "Life Sciences");
					break;
				case EDICT_CUSTOM:
					strcpy(tmpbuff, "Custom");
					break;
			}
			oslIntraFontSetStyle(ltn0, 0.7f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
			i = oslGetStringWidth(tmpbuff);
			if (found_more)
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_IF_Y + 5, tmpbuff);
			else
				oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_IF_Y + 3, tmpbuff);

			ClearScreenKanjiList();
			i = 0;
			for (; y < show_fmax + 1; y++,i++){
				if (/*fname[y] == NULL ||*/ (HELP_LIST_Y + (LINE_SPACE_IF_Y * (2 + i))) >= (AA_SCREEN_HEIGHT - 10))
					break;

				// Show Kanji (if it exists), Kana and lastly Descrip if any space left over.
				x = HELP_X;

				if (kanjiHiding == FALSE)
				{
				// Show kanji
				if (kanji[found_list[y]].k_kanji != NULL){
					oslSetFont(jpn0);
					//oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					// If example sentences exist, show text in Blue. // Added 7/11/09.
					if (kanji[found_list[y]].TCfilepos[0] != -1 && show_examples)
						oslIntraFontSetStyle(jpn0, 1.0f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
					else
						oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

					if ((str_width = oslGetStringWidth((char *)kanji[found_list[y]].k_kanji)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[found_list[y]].k_kanji);
						// For showing individual kanji from kanjidict
						screenkanji[i].index = found_list[y];
						screenkanji[i].start_x = x;
						screenkanji[i].start_y = HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)) - SK_OFFSET_Y;
						screenkanji[i].num_chars = str_width / kchar_width;
						x += (str_width + 3);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[found_list[y]].k_kanji);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < kchar_width)
							goto check_found_index;
						num_printable_chars = x_space_left / kchar_width;
						if ((str_len/2) > (num_printable_chars * 2))// Quick hack assuming always 2bytes
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							if ((u8)tmpbuff[0] < 0x7F){
								is_ascii = TRUE;
								j = 1;
							}else{
								is_ascii = FALSE;
								j = 2;
							}
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j - (is_ascii == TRUE? 1 : 2)] = '\0';
									//if (is_ascii)
									//	tmpbuff[j-1] = '\0';
									//else
									//	tmpbuff[j-2] = '\0';
									break;
								}

								if ((j + (is_ascii == TRUE? 1 : 2) >= str_len)){
									break;
								}
								tmpbuff[j] = tmp_ch;

								if ((u8)tmpbuff[j] < 0x7F){
									is_ascii = TRUE;
									j++;
								}else{
									is_ascii = FALSE;
									j += 2;
								}
							}
						}// End if (go_forward == FALSE)
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
							// For showing individual kanji from kanjidict
							screenkanji[i].index = found_list[y];
							screenkanji[i].start_x = x;
							screenkanji[i].start_y = HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)) - SK_OFFSET_Y;
							screenkanji[i].num_chars = str_width / kchar_width;
						}
						goto check_found_index;
					}// Else, won't all fit.
				}// End if (kanji[found_list[y]].k_kanji != NULL)
			}
			
			if (kanaHiding == FALSE)
			{
				// Show kana
				if (kanji[found_list[y]].k_kana == NULL){
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
					oslDrawString(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"<< NULL >>");
					str_width = oslGetStringWidth("<< NULL >>\0");
					x += (str_width + 3);
				}else{
					oslSetFont(jpn0);
					oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);

					if ((str_width = oslGetStringWidth((char *)kanji[found_list[y]].k_kana)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[found_list[y]].k_kana);
						x += (str_width + 3);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[found_list[y]].k_kana);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < kchar_width)
							goto check_found_index;
						num_printable_chars = x_space_left / kchar_width;
						if ((str_len/2) > (num_printable_chars * 2))// Quick hack assuming always 2bytes
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							if ((u8)tmpbuff[0] < 0x7F){
								is_ascii = TRUE;
								j = 1;
							}else{
								is_ascii = FALSE;
								j = 2;
							}
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j - (is_ascii == TRUE? 1 : 2)] = '\0';
									break;
								}

								if ((j + (is_ascii == TRUE? 1 : 2) >= str_len)){
									break;
								}
								tmpbuff[j] = tmp_ch;

								if ((u8)tmpbuff[j] < 0x7F){
									is_ascii = TRUE;
									j++;
								}else{
									is_ascii = FALSE;
									j += 2;
								}
							}
						}// End if (go_forward == FALSE)
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
						}
						goto check_found_index;
					}// Else, won't all fit.

				}
			}
			if (descHiding == FALSE)
			{
				// Draw what we can of the english description.
				oslSetFont(ltn0);
				//oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				if (kanji[found_list[y]].e_descrip != NULL){
					// If example sentences exist, show text in Blue. // Added 7/11/09.
					if (kanji[found_list[y]].TCfilepos[0] != -1 && show_examples)
						oslIntraFontSetStyle(ltn0, 1.0f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
					else
						oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

					if ((str_width = oslGetStringWidth((char *)kanji[found_list[y]].e_descrip)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[found_list[y]].e_descrip);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[found_list[y]].e_descrip);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < PRINT_CHAR_WIDTH)
							goto check_found_index;
						num_printable_chars = x_space_left / PRINT_CHAR_WIDTH;
						if (str_len > (num_printable_chars * 2))
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							j = 1;
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j-1] = '\0';
									break;
								}
								if ((j + 1 >= str_len)){
									break;
								}
								tmpbuff[j++] = tmp_ch;
							}
						}
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
						}
					}
				}
			}
check_found_index:
				if (y == found_index){
					// Show selected file
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					oslDrawString(HELP_X - 20, ((HELP_LIST_Y - 2) + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i))),"->");
				}
			}// End for (; y < show_fmax + 1; y++,i++)

			break;
		case GS_RESET:
		case GS_PLAYING:
			oslDrawImageXY(bkg, 0, 0);
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			
			// Get ready to draw the list.
			y = (kanji_index / MAX_SHOWN_F) * MAX_SHOWN_F;

			if (lines_read > y + (MAX_SHOWN_F-1)){
				show_fmax = y + (MAX_SHOWN_F-1);
			}else{ // End is in this lot
				show_fmax = lines_read - 1;
			}

			if (lines_read > MAX_SHOWN_F)
				DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, kanji_index, lines_read+1);

			// Found something. Display the results.
			memset(tmpbuff, 0, sizeof(tmpbuff));
			sprintf(tmpbuff, "PSPedict %d to %d (of %d)", y+1, (y+10)>lines_read? lines_read : y+10, lines_read);
			i = oslGetStringWidth(tmpbuff);
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,tmpbuff);
			oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

			// Draw which edict file we have.
			memset(tmpbuff, 0, sizeof(tmpbuff));
			switch(cur_edict){
				case EDICT_ORIG:
					strcpy(tmpbuff, "General");
					break;
				case EDICT_COMP:
					strcpy(tmpbuff, "Computer Terms");
					break;
				case EDICT_LEGAL:
					strcpy(tmpbuff, "Legal Terms");
					break;
				case EDICT_LIFE:
					strcpy(tmpbuff, "Life Sciences");
					break;
				case EDICT_CUSTOM:
					strcpy(tmpbuff, "Custom");
					break;
			}
			oslIntraFontSetStyle(ltn0, 0.7f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
			i = oslGetStringWidth(tmpbuff);
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_IF_Y + 3, tmpbuff);


			ClearScreenKanjiList();
			i = 0;
			for (; y < show_fmax + 1; y++,i++){
				if ( (HELP_LIST_Y + (LINE_SPACE_IF_Y * (2 + i))) >= (AA_SCREEN_HEIGHT - 10))
					break;

				// Show Kanji (if it exists), Kana and lastly Descrip if any space left over.
				x = HELP_X;
				if (kanjiHiding == FALSE)
				{
				// Show kanji
				if (kanji[y].k_kanji != NULL){
					oslSetFont(jpn0);

					// If example sentences exist, show text in Blue. // Added 7/11/09.
					if (kanji[y].TCfilepos[0] != -1 && show_examples)
						oslIntraFontSetStyle(jpn0, 1.0f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
					else
						oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

					if ((str_width = oslGetStringWidth((char *)kanji[y].k_kanji)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[y].k_kanji);
						// For showing individual kanji from kanjidict
						screenkanji[i].index = y;
						screenkanji[i].start_x = x;
						screenkanji[i].start_y = HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)) - SK_OFFSET_Y;
						screenkanji[i].num_chars = str_width / kchar_width;
						x += (str_width + 3);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[y].k_kanji);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < kchar_width)
							goto check_kanji_index;
						num_printable_chars = x_space_left / kchar_width;
						if ((str_len/2) > (num_printable_chars * 2))// Quick hack assuming always 2bytes
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							if ((u8)tmpbuff[0] < 0x7F){
								is_ascii = TRUE;
								j = 1;
							}else{
								is_ascii = FALSE;
								j = 2;
							}
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j - (is_ascii == TRUE? 1 : 2)] = '\0';
									//if (is_ascii)
									//	tmpbuff[j-1] = '\0';
									//else
									//	tmpbuff[j-2] = '\0';
									break;
								}

								if ((j + (is_ascii == TRUE? 1 : 2) >= str_len)){
									break;
								}
								tmpbuff[j] = tmp_ch;

								if ((u8)tmpbuff[j] < 0x7F){
									is_ascii = TRUE;
									j++;
								}else{
									is_ascii = FALSE;
									j += 2;
								}
							}
						}// End if (go_forward == FALSE)
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
							// For showing individual kanji from kanjidict
							screenkanji[i].index = y;
							screenkanji[i].start_x = x;
							screenkanji[i].start_y = HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)) - SK_OFFSET_Y;
							screenkanji[i].num_chars = str_width / kchar_width;
						}
						goto check_kanji_index;
					}// Else, won't all fit.
				}// End if (kanji[y].k_kanji != NULL)
				}

				if (kanaHiding == FALSE)
				{
				// Show kana
				if (kanji[y].k_kana == NULL){
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
					oslDrawString(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"<< NULL >>");
					str_width = oslGetStringWidth("<< NULL >>\0");
					x += (str_width + 3);
				}else{
					oslSetFont(jpn0);
					oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);

					if ((str_width = oslGetStringWidth((char *)kanji[y].k_kana)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[y].k_kana);
						x += (str_width + 3);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[y].k_kana);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);

						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < kchar_width)
							goto check_kanji_index;
						num_printable_chars = x_space_left / kchar_width;
						if ((str_len/2) > (num_printable_chars * 2))// Quick hack assuming always 2bytes
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							if ((u8)tmpbuff[0] < 0x7F){
								is_ascii = TRUE;
								j = 1;
							}else{
								is_ascii = FALSE;
								j = 2;
							}
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j - (is_ascii == TRUE? 1 : 2)] = '\0';
									//if (is_ascii)
									//	tmpbuff[j-1] = '\0';
									//else
									//	tmpbuff[j-2] = '\0';
									break;
								}

								if ((j + (is_ascii == TRUE? 1 : 2) >= str_len)){
									break;
								}
								tmpbuff[j] = tmp_ch;

								if ((u8)tmpbuff[j] < 0x7F){
									is_ascii = TRUE;
									j++;
								}else{
									is_ascii = FALSE;
									j += 2;
								}
							}
						}// End if (go_forward == FALSE)
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
						}
						goto check_kanji_index;
					}// Else, won't all fit.
				}// End if( kanji[y].k_kana == NULL )
				}
				
				// Draw what we can of the english description.
				oslSetFont(ltn0);
				// If example sentences exist, show text in Blue. // Added 7/11/09.
				if (kanji[y].TCfilepos[0] != -1 && show_examples)
					oslIntraFontSetStyle(ltn0, 1.0f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
				else
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

				if(descHiding == FALSE)
				{
				if (kanji[y].e_descrip != NULL){
					if ((str_width = oslGetStringWidth((char *)kanji[y].e_descrip)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
						oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanji[y].e_descrip);
					}else{
						// Won't all fit. Just print enough to fit.
						memset(tmpbuff,0,sizeof(tmpbuff));
						strcpy(tmpbuff, kanji[y].e_descrip);
						str_len = strlen(tmpbuff);
						tmpbuff[str_len - 1] = '\0';
						str_len--;
						str_width = oslGetStringWidth(tmpbuff);
						//Check if we should move from the start or end of the string.
						// This might help improve some performance. Also, looks nicer
						// than just using clipping, more complex though.
						x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
						if (x_space_left < PRINT_CHAR_WIDTH)
							goto check_kanji_index;
						num_printable_chars = x_space_left / PRINT_CHAR_WIDTH;
						if (str_len > (num_printable_chars * 2))
							go_forward = TRUE;
						else
							go_forward = FALSE;

						if (go_forward == FALSE){
							// Check from the end of the string
							while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);
							}
						}else{
							// Check from the start of the string.
							j = 1;
							while (1){
								tmp_ch = tmpbuff[j];
								tmpbuff[j] = '\0';
								str_width = oslGetStringWidth(tmpbuff);
								if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
									tmpbuff[j-1] = '\0';
									break;
								}
								if ((j + 1 >= str_len)){
									break;
								}
								tmpbuff[j++] = tmp_ch;
							}
						}
						if (str_len > 0){
							strcat(tmpbuff, "...");
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
						}
					}
				}
				}
check_kanji_index:
				if (y == kanji_index){
					// Show selected file
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					oslDrawString(HELP_X - 20, ((HELP_LIST_Y - 2) + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i))),"->");
				}
			}// End for (; y < show_fmax + 1; y++,i++)

			break;
		
		case GS_SHOW_KANJIDICT:
			//oslDrawImageXY(bkg, 0, 0);
			oslDrawImageXY(bkg3, 0, 0);

			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			oslSetFont(ltn0);

			// Show basic list.
			y = (kdict_index / MAX_SHOWN_F) * MAX_SHOWN_F;

			if (kdict_lines_read > y + (MAX_SHOWN_F-1)){
				show_fmax = y + (MAX_SHOWN_F-1);
			}
			else{ // End is in this lot
				show_fmax = kdict_lines_read - 1;
			}

			if (kdict_lines_read > MAX_SHOWN_F)
				DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, kdict_index, kdict_lines_read+1);

			// Found something. Display the results.
			memset(tmpbuff, 0, sizeof(tmpbuff));
			sprintf(tmpbuff, "Kanjidic %d to %d (of %d)", y+1, (y+10)>kdict_lines_read? kdict_lines_read : y+10, kdict_lines_read);
			i = oslGetStringWidth(tmpbuff);
			oslDrawString((AA_SCREEN_WIDTH - i)/2, HELP_Y,tmpbuff);
			oslDrawFillRect((AA_SCREEN_WIDTH - i)/2, HELP_Y + LINE_SPACE_Y, ((AA_SCREEN_WIDTH - i)/2) + i, HELP_Y + LINE_SPACE_Y + 2, BLACK);

			i = 0;
			
			for (; y < show_fmax + 1; y++,i++){
				if ( (HELP_LIST_Y + (LINE_SPACE_IF_Y * (2 + i))) >= (AA_SCREEN_HEIGHT - 10))
					break;

					// Show Kanji
					x = HELP_X;
					oslSetFont(jpn0);
					oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				
					if (kanjiHiding == FALSE)
					{
					oslDrawString(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y  * (2 + i)), (char *)kanjidict[y].kanji);
					str_width = oslGetStringWidth(kanjidict[y].kanji);
	
					x += (str_width + 3);
					}
					
					if(kanaHiding == FALSE)
					{
						// Show On Yomi (if it exists)
						if (kanjidict[y].OnYomi != NULL){
							oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
							oslDrawString(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y  * (2 + i)), (char *)kanjidict[y].OnYomi);
							str_width = oslGetStringWidth(kanjidict[y].OnYomi);
							x += (str_width + 3);
						}

						// Show Kun Yomi (if it exists)
						if (kanjidict[y].KunYomi != NULL){
							oslSetFont(jpn0);
							oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

							if ((str_width = oslGetStringWidth((char *)kanjidict[y].KunYomi)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
								oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanjidict[y].KunYomi);
								x += (str_width + 3);
							}else{
								// Won't all fit. Just print enough to fit.
								memset(tmpbuff,0,sizeof(tmpbuff));
								strcpy(tmpbuff, (char *)kanjidict[y].KunYomi);
								str_len = strlen(tmpbuff);
								tmpbuff[str_len - 1] = '\0';
								str_len--;
								str_width = oslGetStringWidth(tmpbuff);

								//Check if we should move from the start or end of the string.
								// This might help improve some performance. Also, looks nicer
								// than just using clipping, more complex though.
								x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
								if (x_space_left < kchar_width)
									goto check_kanjidic_index;
								num_printable_chars = x_space_left / kchar_width;
								if ((str_len/2) > (num_printable_chars * 2))// Quick hack assuming always 2bytes
									go_forward = TRUE;
								else
									go_forward = FALSE;

								if (go_forward == FALSE){
									// Check from the end of the string
									while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
										tmpbuff[str_len - 1] = '\0';
										str_len--;
										str_width = oslGetStringWidth(tmpbuff);
									}
								}else{
									// Check from the start of the string.
									if ((u8)tmpbuff[0] < 0x7F){
										is_ascii = TRUE;
										j = 1;
									}
							
									else{
										is_ascii = FALSE;
										j = 2;
									}
									while (1){
										tmp_ch = tmpbuff[j];
										tmpbuff[j] = '\0';
										str_width = oslGetStringWidth(tmpbuff);
										if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
											tmpbuff[j - (is_ascii == TRUE? 1 : 2)] = '\0';
											break;
										}

										if ((j + (is_ascii == TRUE? 1 : 2) >= str_len)){
											break;
										}
										tmpbuff[j] = tmp_ch;

										if ((u8)tmpbuff[j] < 0x7F){
											is_ascii = TRUE;
											j++;
										}else{
											is_ascii = FALSE;
											j += 2;
										}
									}
								}// End if (go_forward == FALSE)
								if (str_len > 0){
									strcat(tmpbuff, "...");
									oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
								}
								goto check_kanjidic_index;
							}// Else, won't all fit.
						}
					} // end kanaHiding
					// Draw what we can of the english description.
					// Note, there is not always an english description.
					// Learned that the hard way!
					oslSetFont(ltn0);
					oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
					
					if(descHiding == FALSE)
					{
					if (kanjidict[y].e_descrip != NULL){
						if ((str_width = oslGetStringWidth((char *)kanjidict[y].e_descrip)) < (AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) ){
							oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", kanjidict[y].e_descrip);
						}else{
							// Won't all fit. Just print enough to fit.
							memset(tmpbuff,0,sizeof(tmpbuff));
							strcpy(tmpbuff, kanjidict[y].e_descrip);
							str_len = strlen(tmpbuff);
							tmpbuff[str_len - 1] = '\0';
							str_len--;
							str_width = oslGetStringWidth(tmpbuff);

							//Check if we should move from the start or end of the string.
							// This might help improve some performance. Also, looks nicer
							// than just using clipping, more complex though.
							x_space_left = ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x)-15);
							if (x_space_left < PRINT_CHAR_WIDTH)
								goto check_kanjidic_index;
							num_printable_chars = x_space_left / PRINT_CHAR_WIDTH;
							if (str_len > (num_printable_chars * 2))
								go_forward = TRUE;
							else
								go_forward = FALSE;

							if (go_forward == FALSE){
								// Check from the end of the string
								while ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15)) && (str_len > 0)){
									tmpbuff[str_len - 1] = '\0';
									str_len--;
									str_width = oslGetStringWidth(tmpbuff);
								}
							}else{
								// Check from the start of the string.
								j = 1;
								while (1){
									tmp_ch = tmpbuff[j];
									tmpbuff[j] = '\0';
									str_width = oslGetStringWidth(tmpbuff);
									if ((str_width  >  ((AA_SCREEN_WIDTH - (AA_SCREEN_WIDTH - (SCROLLBAR_X + 3)) - x) - 15))){
										tmpbuff[j-1] = '\0';
										break;
									}
									if ((j + 1 >= str_len)){
										break;
									}
									tmpbuff[j++] = tmp_ch;
								}
							}
							if (str_len > 0){
								strcat(tmpbuff, "...");
								oslDrawStringf(x, HELP_LIST_Y + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i)),"%s", tmpbuff);
							}
						}
					}
					}
check_kanjidic_index:
				if (y == kdict_index){
					// Show selected file
					oslSetFont(ltn0);
					oslDrawString(HELP_X - 20, ((HELP_LIST_Y - 2) + LINE_SPACE_IF_Y + (LINE_SPACE_IF_Y * (2 + i))),"->");
				}
			}// End for (; y < show_fmax + 1; y++,i++)
			break;
			
		case GS_SHOW_SEL:
			// CROSS was pressed on GS_PLAYING, GS_SHOW_RESULTS, GS_SHOW_KANJIDICT or GS_SHOW_STUDYLIST screen.

			// Draw Background
			switch(prev_game_status){
				case GS_SHOW_KANJIDICT:
					oslDrawImageXY(bkg3, 0, 0);
					break;
				case GS_PLAYING:
					oslDrawImageXY(bkg, 0, 0);
					break;
				case GS_SHOW_RESULTS:
					oslDrawImageXY(bkg2, 0, 0);
					break;
				case GS_SHOW_STUDYLIST:
					oslClearScreen(RGB(235, 216, 235));
					break;
			}

			// Draw Kanji if it exists.
			if (prev_game_status == GS_SHOW_KANJIDICT){// Only used for debugging at the mo.
				DrawScreenTextKdic();
				break;
			}
			if (prev_game_status == GS_PLAYING){
				DrawScreenTextEdict(kanji_index);
				break;
			}
			if (prev_game_status == GS_SHOW_RESULTS){
				DrawScreenTextEdict(found_list[found_index]);
				break;
			}
			if (prev_game_status == GS_SHOW_STUDYLIST){
				DrawScreenTextEdict(studylist[cur_studylist][studylist_index[cur_studylist]].index);
				break;
			}
			// Reset jpn0 font just in case.
			oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

			break;
	}// End switch( game_status )

	DrawKanjiPopup();

	DrawMenu();// Only draws a menu if appropriate

	DrawCursor();

} // End Func void ScreenUpdate(void);

// #################################################################################################
// Loads a string with the char ';' as a separator. Only three fields.
// 2/10/09 - Changed to use dynamic array.
// 7/11/09 - Added support for Tanka Corpus example sentences using stored file position.
// #################################################################################################
int LoadEdict(void){
	int i = 0, lines_count = 0, j = 0;
	int calloc_err = 0;
	long flength = 0, nread = 0;
	char fnamebuffer[150] = "\0";
	long tmpnum = 0;
	char numbuff[64] = "\0";

	// Clear the kanji array
	for (i = 0; i < MAX_LINES; i++){
		if (kanji[i].k_kanji != NULL){
			free(kanji[i].k_kanji);
			kanji[i].k_kanji = NULL;
		}
		if (kanji[i].k_kana != NULL){
			free(kanji[i].k_kana);
			kanji[i].k_kana = NULL;
		}
		if (kanji[i].e_descrip != NULL){
			free(kanji[i].e_descrip);
			kanji[i].e_descrip = NULL;
		}
		for (j=0; j < MAX_TC_FILEPOS_COUNT; j++)
			kanji[i].TCfilepos[j] = -1;
	}
	
	// Debugging
	//ram = oslGetRamStatus();	
	//oslDebug("Before LoadEdict()\nbytes available=%dkb, %0.2fmb.\nmaxBlockSize = %dkb, %0.2fmb.", ram.maxAvailable/1024, ((float)ram.maxAvailable/1024)/1024, ram.maxBlockSize/1024, ((float)ram.maxBlockSize/1024)/1024);

	memset(fnamebuffer, 0, sizeof(fnamebuffer));
	strcpy(fnamebuffer, DataFilePath);
	strcat(fnamebuffer, fKanjiName[cur_edict]);

	// SJIS file. Open in text mode
	if( (fKanji = fopen( fnamebuffer, "r" )) == NULL ){
		oslDebug( "\n\nThe file \"%s\"\n was not opened. Check the file exists in the \"data\" folder.\n", fKanjiName[cur_edict] );
		return 0;
	}
	//oslDebug( "Opened file \"%s\".", fKanjiName[cur_edict] );

	fseek( fKanji, 0, SEEK_END );
	flength = ftell( fKanji );
	fseek( fKanji, 0, SEEK_SET );
	// Because we're dealing with unsigned short (2 bytes),
	// need to divide flength by 2 to prevent incorrect access of array.
//	flength = flength / 2;

	if (MEDEBUG) oslDebug("Before allocating memory for ucode...");

	ucode = (u8 *) calloc((flength + 1), sizeof(u8));

	//oslDebug( "File length = %d.\nsize of ucode = %d.", flength, (sizeof(u8) * flength));
	if (ucode == NULL){
		oslDebug( "ucode: Couldn't allocate memory." );
		fclose(fKanji);
		return 0;
	}

	if (MEDEBUG) oslDebug("After allocating memory for ucode.");

	if (MEDEBUG) oslDebug("Before reading from file...");
	nread = fread( ucode, 1, flength, fKanji ); // (void *buffer, size_t size, size_t count, FILE *stream)
	if( ferror( fKanji ) || nread == 0 ){
		//printf( "Read error from file." );
		oslDebug( "Read error from file %s.", fKanjiName[cur_edict]);
		fclose(fKanji);
		if (ucode != NULL)
			free (ucode);
		return 0;
	}

	if (MEDEBUG) oslDebug("After reading from file.");
	//flength = flength / 2;
	//oslDebug( "nread = %d. flength now = %d.", nread, flength);

	// <<<<<<<<<<<<<<####################  START OF STUFF PULLED FROM DSLearnJ #############>>>>>>>>>>>>>>>
	// search for JIS escape sequences and remove them
	// Removes the stuff at the start and end of a line.
	//	int i;
	
	for (i = 0; i < flength; i++ ) {
		if ( ucode[i] == 0x1b )
			if ( ucode[i+1] == 0x28 || 0x24 )
				if (ucode[i+2] == 0x40 || 0x42 || 0x4a )
				{
					int j = i;
					while ( j < flength )  // remove the 3 characters that were just found
					{
						ucode[j] = ucode[j+3];
						j++;
					}  	
					flength = flength - 3;
				}
	}

	fclose(fKanji);
	fKanji = NULL;
	ucode[flength] = '\0'; // null-terminate the string

	// ------------- BEGIN PROCESSING EDICT FILE ------------------

	if (MEDEBUG) oslDebug("Begin Processing file...");

	char *line = NULL;

	// Cycle until end of file reached:
	lines_count = 0;
	i = 1;
	calloc_err = FALSE;

	line = (char*)calloc(1024, sizeof(char));

	if (line == NULL){
		oslDebug("\nLoadEdict Error: Memory Allocation Failed for line. Aborting");
		return 0;
	}

	char breaker[] = {0x20, 0x00};// Use ' ' (space in SJIS encoding. Japanese words do not have spaces)
	//char breaker2[] = {0x2f, 0x00};// Use '/'(forward slash in SJIS encoding)
	//char breaker3[] = {0x23, 0x00};// Use '#'(Hash in SJIS encoding)
	line = strtok(ucode, "\r\n"); // read in the text for the first card

	lines_count = 0;
	while(line!=NULL) {
		char *part1 = (char*)calloc(256, sizeof(char));
		char *part2 = (char*)calloc(256, sizeof(char));
		char *part3 = (char*)calloc(512, sizeof(char));
		if (part1 == NULL || part2 == NULL || part3 == NULL){
			oslDebug("\nLoadEdict Error: Memory Allocation Failed for line %d. Aborting", lines_count);
			return 0;
		}
		int length1 = 0;// Changed 6/11/09
		int length2 = 0;// Changed 6/11/09
		int length3 = 0;// Changed 6/11/09
		int numlen = 0;// Added 7/11/09
		char *split1 = NULL;
		split1 = strstr(line,breaker); // Is there any Kanji, or all Kana.

		if (split1 != NULL){// Shouldn't be NULL
			// Is this Kanji or Kana part. Check next character
			length1 = split1-line; // the length of the first part

			if (line[length1+1] == 0x5b){
				// This was a Kanji
				strncpy(part1, line, length1); // store the first part of the card
				length1 = strlen(part1);
			}else{
				// This was a Kana
				length2 = length1;
				length1 = 0;
				strncpy(part2, line, length2); // store the first part of the card
				length2 = strlen(part2);
			}
		}else{
			length1 = 0;
		}

		// Now get Kana part (if first string was Kanji) - There is always a Kana part and English discription.
		char *split2 = NULL;//determine where the string changes from the second to third parts
		char *split3 = NULL;
		char *split4 = NULL;
		if (length1 > 0){
			//tmpst = split1+2;
			//split2 = strstr(split1+2,breaker2);
			split2 = strstr(split1+2,breaker);
			length2 = split2-(split1+2);
			strncpy(part2, split1+2, length2 - 1);
			length2 = strlen(part2);
		}

		// Check for TC filepos numbers - This is new 7/11/09
		if (((length1 > 0)? (*(split2+1)) : (*(split1+1))) == '#'){
			// Get the TC data
			split3 = (length1 > 0)? (split2+2) : (split1+2);
			split4 = split3;
			j = 0;
			do{
				// Should be at the start of the first number...
				while(*split4 != ',' && *split4 != '#')
					split4++;

				if (split4 == split3){
					tmpnum = -1;
				}else{
					numlen = split4-split3;
					memset(numbuff, 0, sizeof(numbuff));
					strncpy(numbuff, split3, numlen);
					// Convert the number.
					tmpnum = atoi(numbuff);
				}
				// Save the number
				kanji[lines_count].TCfilepos[j] = tmpnum;
				if (++j >= MAX_TC_FILEPOS_COUNT){
					break;
				}
				// split4 should now be on either a ',' or '#' character.
				if (*split4 == '#'){// Last number read
					break;
				}
				split4++;
				split3 = split4;

			}while(1);
			// Make sure we get on the last correct char for split3
			while(*split4 != '/')
					split4++;

			split3 = split4+1;

		}else{
			split3 = (length1 > 0)? (split2+2) : (split1+2);
		}

		//if (length1 > 0)
		//	strcpy(part3, split3);//split2+1);
		//else
		//	strcpy(part3, split1+2);
		strcpy(part3, split3);

		length3 = strlen(part3);
		part3[length3-1] = '\0';
		length3--;

		// Do Saving Stuff
		// Kanji (if any exists)
		if (length1 > 0){
			//strcpy((char *)kanji[lines_count].k_kanji, part1);
			// Allocate memory for k_kanji;
			kanji[lines_count].k_kanji = (u8 *) calloc((strlen(part1) + 1), sizeof(u8));
			if (kanji[lines_count].k_kanji == NULL){
				oslDebug("\n%d)LoadEdict Error: Memory Allocation Failed for k_kanji. Aborting", lines_count);
				return 0;
			}
			memset(kanji[lines_count].k_kanji, 0, (strlen(part1) + 1) * sizeof(u8)); // clear any existing data
			strcpy((char *)kanji[lines_count].k_kanji, part1);
		}else{
			kanji[lines_count].k_kanji = NULL;//kanji[lines_count].k_kanji[0] = '\0';
		}

		// Kana.
		if (length2 > 0){
			// Allocate memory for k_kana;
			//strcpy((char *)kanji[lines_count].k_kana, part2);
			kanji[lines_count].k_kana = (u8 *) calloc((strlen(part2) + 1), sizeof(u8));
			if (kanji[lines_count].k_kana == NULL){
				oslDebug("\n%d)LoadEdict Error: Memory Allocation Failed for k_kana. Aborting", lines_count);
				return 0;
			}
			memset(kanji[lines_count].k_kana, 0, (strlen(part2) + 1) * sizeof(u8)); // clear any existing data
			strcpy((char *)kanji[lines_count].k_kana, part2);
		}else{
			kanji[lines_count].k_kana = NULL;
		}

		// English Description (If Any)
		if (length3 > 0){
			// Allocate memory for k_kana;
			//strcpy(kanji[lines_count].e_descrip, part3);
			kanji[lines_count].e_descrip = (char *) calloc((strlen(part3) + 1), sizeof(char));
			if (kanji[lines_count].e_descrip == NULL){
				oslDebug("\n%d)LoadEdict Error: Memory Allocation Failed for e_descrip. Aborting", lines_count);
				return 0;
			}
			memset(kanji[lines_count].e_descrip, 0, (strlen(part3) + 1) * sizeof(char)); // clear any existing data
			strcpy(kanji[lines_count].e_descrip, part3);
		}else{
			kanji[lines_count].e_descrip = NULL;
		}

		free(part1); // free the memory
		free(part2);
		free(part3);
		if ((lines_count + 1) >= /*MAX_LINES*/edict_max[cur_edict]){
			break;
		}
		lines_count++;
		line = strtok(NULL, "\r\n"); // read in the text for the next card
	}// End while(line!=NULL)
	
	// ------------- LASTLY CLEANUP ---------------
	// ucode freed in calling function (doInit2())

	return lines_count;
}//int LoadEdict();

// Used to read 1 example sentence
int LoadTCLine(char *filepath, long filepos, uTC *tmpTC){// Tanaka Corpus Example stuff.
	int char_count = 0;
	int length1 = 0, length2 = 0, length3 = 0;
	FILE *fTC = NULL;
	u8 buff1[1024];
	//u8 buff2[1024];

	char breaker[] = {0x09, 0x00};// Use '\t' (TAB in SJIS encoding) Separate Jap & Eng on line 'A'
	char breaker2[] = {0x23, 0x00};// Use '#'(forward slash in SJIS encoding) Start of ID number on line 'A'
	//char breaker3[] = {0x20, 0x00};// Use ' ' (space in SJIS encoding) Separates words on line 'B'

	char part1[512] = "\0";
	char part2[512] = "\0";
	char part3[512] = "\0";
	int tmpid = 0;
	char word[256] = "\0";
	// Use below if reading 'B' line data from TC.
	//char *kana_pos = NULL;
	//char *sense_pos = NULL;
	//char *form_pos = NULL;
	//char *match_pos = NULL;


	// Now go through and create more managable file chunks.
	if( (fTC = fopen( filepath, "r" )) == NULL ){
		oslDebug( "\n\nThe file \"%s\"\n was not opened\n", filepath );
		return 0;
	}

	fseek( fTC, 0L, SEEK_SET );
	fseek( fTC, filepos, SEEK_CUR );
	memset(buff1, 0, sizeof(buff1));
	//memset(buff2, 0, sizeof(buff2));
	// 'A' line
	if (fgets((char *)buff1, 1024, fTC) == NULL){
		printf("\nError reading exact location.\n");
		return 1;// Error reading from file.
	}

	length1 = strlen((char *)buff1);
	buff1[length1-1] = 0;

	//// 'B' line
	//if (fgets(buff2, 1024, fTC) == NULL){
	//	printf("\nError reading exact location.\n");
	//	return 2;// Error reading from file.
	//}

	fclose(fTC);

	// Read in the stuff we need. Now process it.

	// Clear TC
	if ((*tmpTC).j_scentence != NULL){
		free((*tmpTC).j_scentence);
		(*tmpTC).j_scentence = NULL;
	}
	if ((*tmpTC).e_scentence != NULL){
		free((*tmpTC).e_scentence);
		(*tmpTC).e_scentence = NULL;
	}
	(*tmpTC).file_num = 0;
	(*tmpTC).file_pos = -1;
	(*tmpTC).ID = -1;
	(*tmpTC).word = NULL;

	memset(part1, 0, sizeof(part1));
	memset(part2, 0, sizeof(part2));
	memset(part3, 0, sizeof(part3));
	memset(word, 0, sizeof(word));
	tmpid = 0;

	
	// Process line 'A' first. Should actually check we have a line 'A'.
	// Get Japanese Scentence
	char *split1 = NULL;
	char_count = 3;// Offset for string "A: " or "B: "
	split1 = strstr((char *)(buff1+char_count),breaker);
	length1 = split1-(char *)buff1;
	length1 = length1 - char_count;
	strncpy(part1, (char *)buff1+char_count, length1);
	length1 = strlen(part1);

	char_count += length1+1;

	// Get English Scentence
	char *split2 = NULL;
	split2 = strstr((char *)buff1+char_count,breaker2);
	length2 = split2 - ((char *)buff1+char_count);
	strncpy(part2, (char *)buff1+char_count, length2);

	char_count += length2;

	// Get the ID number.
	char_count += 4;
	memset(part3, 0, sizeof(part3));
	strcpy(part3, (char *)buff1+char_count);
	length3 = strlen(part3);
	tmpid = atoi(part3);

	// Save the Japanese and English sentences, and index.
	//Japanese sentence
	(*tmpTC).j_scentence = (u8 *) calloc(length1 + 1, sizeof(u8));
	if ((*tmpTC).j_scentence == NULL){
		oslDebug("\nLoadTCLine Error: Memory Allocation Failed for j_scentence. Aborting");
		return 3;
	}
	memset((*tmpTC).j_scentence, 0, (length1 + 1) * sizeof(u8)); // clear any existing data
	strcpy((char *)(*tmpTC).j_scentence, part1);

	// English scentence
	(*tmpTC).e_scentence = (char *) calloc(length2 + 1, sizeof(char));
	if ((*tmpTC).e_scentence == NULL){
		oslDebug("\nLoadTC Error: Memory Allocation Failed for e_scentence. Aborting");
		return 0;
	}
	memset((*tmpTC).e_scentence, 0, (length2 + 1) * sizeof(char)); // clear any existing data
	strcpy((*tmpTC).e_scentence, part2);

	(*tmpTC).ID = tmpid;
	(*tmpTC).file_pos = filepos;

	// These would be used for line 'B' processing, but not required to show example sentences.
	//char *kana_pos = NULL;
	//char *sense_pos = NULL;
	//char *form_pos = NULL;
	//char *match_pos = NULL;

	return RTN_OK;
}// int LoadTCLine(char *filepath, long filepos, uTC *tmpTC);

// #################################################################################################
// Load the kanjidict file.
// The kanjidict file being used has had much information stripped out.
// Only the kanji, unicode, stroke count, On Yomi, Kun Yomi and english descriptions
// exist in the file, mainly for space considerations.
// #################################################################################################
int LoadKanjidict(void){

#define TMP_BUFF_SIZE 128
#define KDBUFF_SIZE 512

	char KDbuffer[KDBUFF_SIZE];
	char tmpbuff[TMP_BUFF_SIZE];
	char tmpbuff_on[MAX_ONYOMI];
	char tmpbuff_kun[MAX_KUNYOMI];
	char *line = NULL;

	char *KDucode = NULL; // - One byte - Shift JIS
	int KDflength = 0, KDnread = 0, i = 0, now_proc = 0;
	int lines_count = 0;
	int keep_going = 0, line_len = 0, doing_eng = 0;

	// Clear the array
	for (i = 0; i < MAX_KANJIDICT; i++){
		kanjidict[i].kanji[0] = '\0';
		kanjidict[i].num_strokes = 0;
		kanjidict[i].OnYomi = NULL;
		kanjidict[i].KunYomi = NULL;
		kanjidict[i].e_descrip = NULL;
	}


	// Open the Kanji file in text mode (SJIS)
	if( (fKanjiDict = fopen( fKanjiDictName, "r" )) == NULL ){ // NDS program is using text mode.
		oslDebug( "\n\nLoadKD Error: The file \"%s\"\n was not opened.\n", fKanjiDictName );
		fclose(fKanjiDict);
		return 0;
	}
	//oslDebug( "\nOpened file \"%s\".\n", fname );

	fseek( fKanjiDict, 0, SEEK_END );
	KDflength = ftell( fKanjiDict );
	fseek( fKanjiDict, 0, SEEK_SET );
	//oslDebug( "\nFile length = %d.\n", KDflength );

	KDucode = (char *) calloc((KDflength + 1), sizeof(char));
	if (KDucode == NULL){
		oslDebug( "LoadKD Error: Couldn't allocate memory for file \"%s\".", fKanjiDictName);
		fclose(fKanjiDict);
		return 0;
	}

	KDnread = fread( KDucode, 1, KDflength, fKanjiDict ); // (void *KDbuffer, size_t size, size_t count, FILE *stream)
	if( ferror( fKanjiDict ) || KDnread == 0 ){
		oslDebug( "LoadKD Error: Read error from file \"%s\".", fKanjiDictName);
		fclose(fKanjiDict);
		return 0;
	}


	//oslDebug( "\nFile loading complete.\nPress a key to continue...\n");


	// Search for JIS escape sequences and remove them
	// Removes the stuff at the start and end of a line.
	
	for (i = 0; i < KDflength; i++ ) {
		if ( KDucode[i] == 0x1b )
			if ( KDucode[i+1] == 0x28 || 0x24 )
				if (KDucode[i+2] == 0x40 || 0x42 || 0x4a )
				{
					int j = i;
					while ( j < KDflength )  // remove the 3 characters that were just found
					{
						KDucode[j] = KDucode[j+3];
						j++;
					}  	
					KDflength = KDflength - 3;
				}
	}

	fclose(fKanjiDict); // close the file
	KDucode[KDflength] = '\0'; // null-terminate the string

	//oslDebug("Finished checking for escape sequence. Writing to debug file.");


	// ---------------- Start going through file ------------------- //
	int char_count = 0;
	line = (char*)calloc(512, sizeof(char));
	
		char breaker[] = {0x20, 0x00};// Use ' ' (space in SJIS encoding. Japanese words do not have spaces)
		char breaker2[] = {0x7d, 0x00};// Use '}'(curly brace in SJIS encoding)
		line = strtok(KDucode, "\r\n"); // read in the text for the first card

		lines_count = 0;
		while(line!=NULL) {
			int length1 = 0;// Changed 6/11/09
			int tmp_length = 0;// Changed 6/11/09

			// Firstly, should be the actual Kanji, followed by a space,
			// then the four byte ucode followed by a space.
			// Check the length of the line. If it less than 10, skip it.
			line_len = strlen(line);
			if (line_len < 10)
				continue;
			// First two chars are the actual kanji in SJIS
			memset(kanjidict[lines_count].kanji, 0, sizeof(kanjidict[lines_count].kanji)); // clear any existing data
			kanjidict[lines_count].kanji[0] = line[0];
			kanjidict[lines_count].kanji[1] = line[1];
			kanjidict[lines_count].kanji[2] = 0;

			// Now to read the stroke count. 358 Kanji have disputed stroke count. Only read the first.
			kanjidict[lines_count].num_strokes = 0;
			char *stroke_split = NULL;//strstr(line,breaker); //determine where the string changes from the first to second parts
			char_count = 9;
			stroke_split = strstr(line+char_count,breaker); // Need to check if there is any Kanji, or all Kana.
			length1 = stroke_split - (line+char_count);
			memset(tmpbuff, 0, sizeof(tmpbuff));
			strncpy(tmpbuff, line+char_count, length1);
			kanjidict[lines_count].num_strokes = atoi(tmpbuff);
			//oslDebug("\nstroke_split = \"%s\", length = %d. num_strokes = %d.\n", tmpbuff, length1, kanjidict[lines_count].num_strokes);

			// Now need to see if there is another Sx information
			char_count += length1 + 1;
			//oslDebug("line[%d] = '%c'\n", char_count, line[char_count]);
			if (line[char_count] == 'S'){
				// Move to the next block.
				do{
					char_count++;
				}while(line[char_count] != 0x20);
				char_count++;
				//oslDebug("\nNext block is another stroke count. Bummer...\n");
			}
			
			// Added by Batanen. SKIP CODE IS NEXT
			
			++char_count; // P to number
			char firstSkip[3] = {'\0'};
			char secondSkip[3] = {'\0'};
			char thirdSkip[3] = {'\0'};
			
			int k=0;
			while(line[char_count] != '-')
			{
				firstSkip[k] = line[char_count];
				++k;
				++char_count;
			}
			++char_count; // - to number
			k=0;
			while(line[char_count] != '-')
			{
				secondSkip[k] = line[char_count];
				++k;
				++char_count;
			}
			++char_count; // - to number
			
			k=0;
			while(line[char_count] != 0x20)
			{
				thirdSkip[k] = line[char_count];
				++k;
				++char_count;
			}
			
			kanjidict[lines_count].skipCode[0] = atoi(firstSkip);
			kanjidict[lines_count].skipCode[1] = atoi(secondSkip);
			kanjidict[lines_count].skipCode[2] = atoi(thirdSkip);
			
			++char_count; // space to kana.
			
			// Now we can read the On Yomi (Katakana)/Kun Yomi(Hiragana) and English Description
			// It appears that if there is On Yomi, it will always appear first.

			// Test if this is On Yomi or Kun Yomi reading.
			now_proc = PROC_ONYOMI;
			if ((u8)line[char_count] < 0x83){
				// May need to check for special characters.
				// E.g. -/ bou, etc. These will show up here.
				if ((u8)line[char_count] < 0x82){
					// Need to check if the char following is Hiragana or Katakana
					if ((u8)line[char_count + 2] < 0x83){
						now_proc = PROC_KUNYOMI;
					}else{
						now_proc = PROC_ONYOMI;
					}
				}else{
					now_proc = PROC_KUNYOMI;
				}
			}else{
				now_proc = PROC_ONYOMI;
			}
			
			// Keep reading till we encounter the english description.
			// All english descriptions start with the character '{'
			// and end with '}'. If more than one description, they are
			// separated by a space.

			keep_going = TRUE;
			doing_eng = FALSE;
			memset(tmpbuff_on, 0, sizeof(tmpbuff_on));
			memset(tmpbuff_kun, 0, sizeof(tmpbuff_kun));
			memset(KDbuffer, 0, sizeof(KDbuffer)); // For Eng Descrip
			char *onkun_split = NULL;
			do{
				memset(tmpbuff, 0, sizeof(tmpbuff));
				// Read next string
				if (doing_eng)
					onkun_split = strstr(line+char_count,breaker2); // Could be either Katakana, Hiragana, or English
				else
					onkun_split = strstr(line+char_count,breaker); // Could be either Katakana, Hiragana, or English

				if (onkun_split == NULL){
					// Found the end, save all details here...
					// Save On Yomi;
					// Save Kun Yomi;
					// Save Eng Descrip;
					keep_going = FALSE;
					break;
				}
				tmp_length = onkun_split - (line+char_count);
				strncpy(tmpbuff, line+char_count, tmp_length); // store the first part of the card
				tmp_length = strlen(tmpbuff);
				// Check if we've moved onto the English Descriptions
				// Or changed from On Yomi to Kun Yomi.

				switch ((u8)tmpbuff[0]){
					case '\n': // Would this do anything? Do we need to do anything?
						keep_going = FALSE;
						break;
					case '\r': // Would this do anything? Do we need to do anything?
						keep_going = FALSE;
						break;
					case 0x83: // On Yomi (Katakana). On Yomi is always first.
						// No need to check prev state, as On Yomi is always first (if it exists).
						// First, check if we have space to add this reading.
						if ((strlen(tmpbuff_on) + tmp_length + 2) >= MAX_ONYOMI){
							// Move to next token.
							char_count += tmp_length + 1;
							// Check if we are now onto Eng Descrip
							if ((u8)line[char_count] == 0x7b)
								doing_eng = TRUE;
							break;
						}
						// Have space to add the string
						if (tmpbuff_on[0] == 0){
							strcpy(tmpbuff_on, tmpbuff);
							//strcat(tmpbuff_on, ", \0");
						}else{
							strcat(tmpbuff_on, ", \0");
							strcat(tmpbuff_on, tmpbuff);
						}
						char_count += tmp_length + 1;
						// Check if we are now onto Eng Descrip
						if ((u8)line[char_count] == 0x7b)
							doing_eng = TRUE;
							break;
						break;
					case 0x82: // Kun Yomi (Hiragana)
						// First, check if we have space to add this reading.
						if ((strlen(tmpbuff_kun) + tmp_length + 2) >= MAX_KUNYOMI){
							// Move to next token. Don't add.
							char_count += tmp_length + 1;
							// Check if we are now onto Eng Descrip
							if ((u8)line[char_count] == 0x7b)
								doing_eng = TRUE;
							break;
						}
						// Have space to add the string
						if (tmpbuff_kun[0] == 0){
							strcpy(tmpbuff_kun, tmpbuff);
							//strcat(tmpbuff_kun, ", \0");
						}else{
							strcat(tmpbuff_kun, ", \0");
							strcat(tmpbuff_kun, tmpbuff);
						}
						char_count += tmp_length + 1;
						// Check if we are now onto Eng Descrip
						if ((u8)line[char_count] == 0x7b)
							doing_eng = TRUE;
						break;
					case '{': // English Description
						doing_eng = TRUE;
						// First, check if we have space to add this reading.
						if ((strlen(KDbuffer) + tmp_length) >= MAX_KD_DESCRIP){
							// Move to next token. Don't add.
							char_count += tmp_length + 2; // Check me.
							// Checking this as this could be the last string to check.
							if (char_count > (line_len - 3))
								keep_going = FALSE;
							break;
						}
						// Save the string. The first & last char will be '{' and '}'.
						if (KDbuffer[0] == 0){
							strncpy(KDbuffer, tmpbuff + 1, tmp_length - 1);
						}else{
							strcat(KDbuffer, ", \0");
							strncat(KDbuffer, tmpbuff + 1, tmp_length - 1);
						}
						char_count += tmp_length + 2;
						// Checking this as this could be the last string to check.
						if (char_count > (line_len - 3))
							keep_going = FALSE;

						break;
					default: // Special Character?
						// Need to check for On Yomi or Kun Yomi?
						//oslDebug( "\n%d)Found Special Char(%c), value(%d), hex(%x)\nPress a key to continue...\n", lines_count, tmpbuff[0], (u8)tmpbuff[0], (u8)tmpbuff[0]);
						if (tmpbuff[0] == 'S'){
							char_count += tmp_length + 1; // Check me.
						}
						break;
				}// End switch ((u8)tmpbuff[0])
				/*if(lines_count > 230) // For checking which kanjidict lines crashes
				{
				oslDebug("BEFORE line[%d] = '%c' LINES COUNT: %d\n", char_count, line[char_count], lines_count);
				}*/
			}while(keep_going);

			// Found the end, save all details here...

			// Allocate memory
			if (tmpbuff_on[0] != 0){
				kanjidict[lines_count].OnYomi = (u8 *) calloc((strlen(tmpbuff_on) + 1), sizeof(u8));
				if (kanjidict[lines_count].OnYomi == NULL){
					oslDebug("\n%d)LoadKD Error: Memory Allocation Failed for OnYomi. Aborting", lines_count);
					return 0;
				}
				memset(kanjidict[lines_count].OnYomi, 0, (strlen(tmpbuff_on) + 1) * sizeof(u8)); // clear any existing data
				strcpy((char *)kanjidict[lines_count].OnYomi, tmpbuff_on);
			}
			if (tmpbuff_kun[0] != 0){
				kanjidict[lines_count].KunYomi = (u8 *) calloc((strlen(tmpbuff_kun) + 1), sizeof(u8));
				if (kanjidict[lines_count].KunYomi == NULL){
					oslDebug("\n%d)LoadKD Error: Memory Allocation Failed for KunYomi. Aborting", lines_count);
					return 0;
				}
				memset(kanjidict[lines_count].KunYomi, 0, (strlen(tmpbuff_kun) + 1) * sizeof(u8)); // clear any existing data
				strcpy((char *)kanjidict[lines_count].KunYomi, tmpbuff_kun);
			}
			if (KDbuffer[0] != 0){
				kanjidict[lines_count].e_descrip = (char *) calloc((strlen(KDbuffer) + 1), sizeof(char));
				if (kanjidict[lines_count].e_descrip == NULL){
					oslDebug("\n%d)LoadKD Error: Memory Allocation Failed for Descrip. Aborting", lines_count);
					return 0;
				}
				memset(kanjidict[lines_count].e_descrip, 0, (strlen(KDbuffer) + 1) * sizeof(char)); // clear any existing data
				strcpy(kanjidict[lines_count].e_descrip, KDbuffer);
			}

			// Check to see if we can continue to the next line
			if ((lines_count + 1) >= MAX_KANJIDICT){
				break;
			}
			lines_count++;
			line = strtok(NULL, "\r\n"); // read in the text for the next line
		}// End while(line!=NULL)

	//free(line); - Dont think this is required. Was worried about memory leakage.
	free(KDucode);

	// Print out contents of cards when finished.
	//oslDebug("\nFinished in read_file\n");

	return lines_count;

}// int LoadKanjidict(void)

int find_number_string(char *search_str)
{
	int i = 0;
	int num_found = 0; // Number of results found
	
	// Search by stroke count
	if (strlen(search_str) < 4)
	{
		kanjiListing = STROKES;
		char *strokes = strtok(search_str,"#"); // remove #
		strokeCount = atoi(strokes);
		
		for (i=0; i < MAX_KANJIDICT; ++i)
		{
			if(kanjidict[i].num_strokes == strokeCount)
			{
				// Found kanji that matches given stroke count
				found_list[num_found] = i;
				++num_found;
			}
		}
	}
	
	// Search SKIP codes
	else
	{
		kanjiListing = SKIPCODE;
		
		char delims[] = " #"; // remove spaces and #
		char *skipCode = strtok(search_str,delims); // remove spaces and #
		int j=0;
		short int first=0;
		short int second=0;
		short int third=0;
		short int fourth=0; // for strokes
		
		while(skipCode != NULL)
		{
			if(j==0)
			{
				first = atoi(skipCode);
			}
			
			else if(j==1)
			{
				second = atoi(skipCode);
			}
			
			else if(j==2)
			{
				third = atoi(skipCode);
			}
			
			else if(j==3)
			{
				fourth = atoi(skipCode);
				userSkipCode[3] = fourth;
			}
			
			else
			{
				break;
			}
			skipCode = strtok(NULL,delims);
			++j;
		}
		
		if (j < 2) // Not enough parameters
		{
			first = 0;
			second = 0;
			third = 0;
		}
		
		userSkipCode[0] = first;
		userSkipCode[1] = second;
		userSkipCode[2] = third;
		
		for (i=0; i < MAX_KANJIDICT; ++i)
		{
			// Search with SKIP and stroke
			if(j==4)
			{
			// Type and second parameter can be anything
			if(userSkipCode[0] == 0 && userSkipCode[1] == 0)
			{
				int n=1; // for first
				int m=0; // for second
				for(; n < 4; ++n)
				{
					m=0;
					for(; m < 21; ++m)
					{
						if( (kanjidict[i].skipCode[0] == n) && (kanjidict[i].skipCode[1] == m) && (kanjidict[i].skipCode[2] == userSkipCode[2]) && (kanjidict[i].num_strokes == userSkipCode[3]))
						{
							// Found kanji that matches SKIP
							found_list[num_found] = i;
							++num_found;
						}
					}
				}
			}
			
			// Type and third parameter can be anything
			else if(userSkipCode[0] == 0 && userSkipCode[2] == 0)
			{
				int n=1; // for first
				int m=0; // for third
				for(; n < 4; ++n)
				{
					m=0;
					for(; m < 23; ++m)
					{
						if( (kanjidict[i].skipCode[0] == n) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == m) && (kanjidict[i].num_strokes == userSkipCode[3]))
						{
							// Found kanji that matches SKIP
							found_list[num_found] = i;
							++num_found;
						}
					}
				}
			}
			
			// Type is specified, but second and third parameter can be anything
			else if(userSkipCode[1] == 0 && userSkipCode[2] == 0)
			{
				int n=0; // for second
				int m=0; // for third
				for(; n < 21; ++n)
				{
					m=0;
					for(; m < 23; ++m)
					{
						if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == n) && (kanjidict[i].skipCode[2] == m) && (kanjidict[i].num_strokes == userSkipCode[3]))
						{
							// Found kanji that matches SKIP
							found_list[num_found] = i;
							++num_found;
						}
					}
				}
			}
			
			// Search with all types: 1, 2, 3, 4
			else if(userSkipCode[0] == 0)
			{
				int n=1;
				for(;n<4;++n)
				{
					if( ((kanjidict[i].skipCode[0] == n) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == userSkipCode[2])) && (kanjidict[i].num_strokes == userSkipCode[3]))
					{
						// Found kanji that matches SKIP
						found_list[num_found] = i;
						++num_found;
					}
				}
			}
			
			// Middle code can be anything
			else if(userSkipCode[1] == 0)
			{
				int n; // Checked that middle skip maximum is 20.
				for(n=0; n < 21; ++n)
				{
					if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == n) && (kanjidict[i].skipCode[2] == userSkipCode[2]) && (kanjidict[i].num_strokes == userSkipCode[3]))
					{
						// Found kanji that matches SKIP
						found_list[num_found] = i;
						++num_found;
					}
				}
			}
			// Last code can be anything
			else if(userSkipCode[2] == 0)
			{
				int n;
				for(n=0; n < 23; ++n) // maximum of 3rd skip is 22
				{
					if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == n) && (kanjidict[i].num_strokes == userSkipCode[3]))
					{
						// Found kanji that matches SKIP
						found_list[num_found] = i;
						++num_found;
					}
				}
			}
			
			// Search exact match given by user
			else if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == userSkipCode[2]) && (kanjidict[i].num_strokes == userSkipCode[3]) )
			{
				// Found kanji that matches given stroke count
				found_list[num_found] = i;
				++num_found;
			}
			}
		
			// Search only SKIP
			else
			{
			// Type and second parameter can be anything
			if(userSkipCode[0] == 0 && userSkipCode[1] == 0)
			{
				int n=1; // for first
				int m=0; // for second
				for(; n < 4; ++n)
				{
					m=0;
					for(; m < 21; ++m)
					{
						if( (kanjidict[i].skipCode[0] == n) && (kanjidict[i].skipCode[1] == m) && (kanjidict[i].skipCode[2] == userSkipCode[2]) )
						{
							// Found kanji that matches SKIP
							found_list[num_found] = i;
							++num_found;
						}
					}
				}
			}
			
			// Type and third parameter can be anything
			else if(userSkipCode[0] == 0 && userSkipCode[2] == 0)
			{
				int n=1; // for first
				int m=0; // for third
				for(; n < 4; ++n)
				{
					m=0;
					for(; m < 23; ++m)
					{
						if( (kanjidict[i].skipCode[0] == n) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == m) )
						{
							// Found kanji that matches SKIP
							found_list[num_found] = i;
							++num_found;
						}
					}
				}
			}
			
			// Type is specified, but second and third parameter can be anything
			else if(userSkipCode[1] == 0 && userSkipCode[2] == 0)
			{
				int n=0; // for second
				int m=0; // for third
				for(; n < 21; ++n)
				{
					m=0;
					for(; m < 23; ++m)
					{
						if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == n) && (kanjidict[i].skipCode[2] == m) )
						{
							// Found kanji that matches SKIP
							found_list[num_found] = i;
							++num_found;
						}
					}
				}
			}
			
			// Search with all types: 1, 2, 3, 4
			else if(userSkipCode[0] == 0)
			{
				if( ((kanjidict[i].skipCode[0] == 1) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == userSkipCode[2])) ||
					((kanjidict[i].skipCode[0] == 2) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == userSkipCode[2])) ||
					((kanjidict[i].skipCode[0] == 3) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == userSkipCode[2])) ||
					((kanjidict[i].skipCode[0] == 4) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == userSkipCode[2])) )
					{
					
					// Found kanji that matches SKIP
					found_list[num_found] = i;
					++num_found;
					}
			}
			
			// Middle code can be anything
			else if(userSkipCode[1] == 0)
			{
				int n; // Checked that middle skip maximum is 20.
				for(n=0; n < 21; ++n)
				{
					if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == n) && (kanjidict[i].skipCode[2] == userSkipCode[2]) )
					{
						// Found kanji that matches SKIP
						found_list[num_found] = i;
						++num_found;
					}
				}
			}
			// Last code can be anything
			else if(userSkipCode[2] == 0)
			{
				int n;
				for(n=0; n < 23; ++n) // maximum of 3rd skip is 22
				{
					if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == n) )
					{
						// Found kanji that matches SKIP
						found_list[num_found] = i;
						++num_found;
					}
				}
			}
			
			// Search exact match given by user
			else if( (kanjidict[i].skipCode[0] == userSkipCode[0]) && (kanjidict[i].skipCode[1] == userSkipCode[1]) && (kanjidict[i].skipCode[2] == userSkipCode[2]) )
			{
				// Found kanji that matches given stroke count
				found_list[num_found] = i;
				++num_found;
			}
			}//Search only SKIP
		}
	}
	
	if (num_found > 0)
	{
		// This is the fast way if you want randomized kanji entries
		ukanjidict temp; // variable for used in swapping
		int k;
		
		// Swap the found kanjis to the beginning of the list
		// This is not perfect algorithm as kanjis switch places.
		for(k=0; k < num_found; k++)
		{
			temp = kanjidict[k];
			kanjidict[k] = kanjidict[found_list[k]];
			kanjidict[found_list[k]] = temp;
		}
		
		/* Seems not work correctly if SKIP is used
		ukanjidict temp[found_list[0]];
		int k;
		
		// Save the kanjis that need to move elsewhere.
		for(k = 0; k < found_list[0]; ++k)
		{
			temp[k] = kanjidict[k];
		}
		
		// Override indexes from beginning of kanjidict
		for(k=0; k < num_found; k++)
		{
			kanjidict[k] = kanjidict[found_list[k]];
		}
		
		int j =  num_found;
		// Re-order kanjidict so that places do not seem to change
		for(k=0; k < found_list[0]; ++k, ++j)
		{
			kanjidict[j] = temp[k];
		}*/
		
		kdict_index = 0;
		kdict_lines_read = num_found;
	}
	
	else if(strlen(search_str) < 4 && search_str[1] == '0')
	{
		// Nothing found and zero code, in that case list all kanjis
		kanjiListing = NORMAL;
		kdict_index = 0;
		kdict_lines_read = kdict_lines_read_backup;
		qsort( (void *)kanjidict, (size_t)kdict_lines_read + 1, sizeof( ukanjidict ), kanjidict_compare_stroke );
	}
	
	// Nothing found and no zero code
	else
	{
		kanjiListing = NORMAL;
		kdict_index = 0;
		kdict_lines_read = 0;
	}
	
	return num_found;
}

// #################################################################################################
// Search for the string. Note the search_str can
// exist anywhere in the checked string. If whole_string
// is TRUE, will try to isolate the whole word.
// For the purpose of this program, the search is NOT
// case sensitive.
// Allow that if last char of search_str is '*', then
// use this as a wildcard.
// TODO - Maybe allow wildcard at start of string?
// TODO - maybe allow '\?' to allow one char at start/end?
// #################################################################################################
int find_eng_string(char *search_str, int whole_string){
	kanjiListing = NONE;
	int i;
	int num_found = 0;
	char *pdest;
	char tmpch[2] = "\0";
	int my_strlen = 0;
	int tmp_num = 0;
	int search_str_len = 0;
	found_more = FALSE;
	int use_wildcard_end = FALSE; // Only used when searching whole word.
	int search_whole_string = 0;
	char wordbuff[512] = "\0";
	char search_str_lower[MAX_DESCRIP] = "\0";

	search_whole_string = whole_string;

	// Check for valid search string
	if (search_str == NULL)
		return 0;
	else if (search_str[0] == 0)
		return 0;
	// Search stroke count or SKIP if sentence begin with #
	else if (search_str[0] == '#' && strlen(search_str) > 1)
		return find_number_string(search_str);

	strcpy(search_str_lower, search_str);
	search_str_len = strlen(search_str); // Needed below also.
	if (search_str[search_str_len - 1] == '*'){
		use_wildcard_end = TRUE;
		search_str_lower[search_str_len - 1] = '\0'; // NEW
		search_str_len--; // NEW
	}

	// Check if user put a wildcard at the front of the search_str.
	// If so, ignore whole_string, treat as partial search.
	if (search_str_lower[0] == '*'){
		strcpy(search_str_lower, search_str_lower+1);
		search_str_len--;
		search_whole_string = FALSE;
	}

	// Convert to lower case. - Check for Memory leaks.
	_strlwr(search_str_lower);

	for (i=0; i < lines_read; i++){
		strcpy(wordbuff, kanji[i].e_descrip);
		_strlwr(wordbuff);
		// Get string length of wordbuff?
try_eng_sub_string:
		my_strlen = strlen(wordbuff);

		// Search lowercase string.
		pdest = strstr(wordbuff, search_str_lower);
		if (pdest != NULL){
			if (search_whole_string != TRUE){
				// OK to find anywhere - Add it!
				if (num_found < MAX_FOUND - 2){
					found_list[num_found] = i;
					num_found++;
				}else{
					found_more = TRUE;
					break;
				}
			}else{
				// Should check if this is the whole string.
				// Check before, and after to see if this is a whole string.
				// Checking if we are already at the beginning. If not, check prev char.
				if (pdest != wordbuff){
					// Check the character before
					tmpch[0] = wordbuff[pdest - wordbuff - 1];
					if (strcspn("\"\'., \\\?!&;:/$*%(){}[]-+=~", tmpch) == 25){
						// Not a valid character found, not whole word match.
						// Check for any other sub-string matches?
						tmp_num = (pdest - wordbuff + search_str_len);
						// Only try to search again if there is enough string to search, ie. at least strlen(search_str) chars.
						if (tmp_num + search_str_len < (my_strlen - 1)){ // Should be just '<' or '<=' ? 
							strcpy(wordbuff, wordbuff + tmp_num);
							goto try_eng_sub_string;
						}else{
							continue;
						}
					}
				}
				// Start OK, check end of the word if use_wildcard_end != TRUE.
				// If use_wildcard_end == TRUE, don't check the end. It doesn't have to match fully.
				// Did we find the end?
				if (!use_wildcard_end){
					//my_strlen = strlen(wordbuff); // - Moved up top
					tmp_num = (pdest - wordbuff + search_str_len);
					if (tmp_num != my_strlen){
						// Not at end, so need to check next char.
						tmpch[0] = wordbuff[pdest - wordbuff + search_str_len];
						if (strcspn("\"\'., \\\?!&;:/$*%(){}[]-+=~", tmpch) == 25){
							// Not a valid character found, not whole word match.
							// Check for any other sub-string matches?
							if (tmp_num + search_str_len < (my_strlen - 1)){ // Should be just '<' or '<=' ? 
								strcpy(wordbuff, wordbuff + tmp_num);
								goto try_eng_sub_string;
							}else{
								continue;
							}
						}
					}
				}
				// Found whole word, or used wildcard '*' and start is OK. Add to list.
				if (num_found < MAX_FOUND - 2){
					found_list[num_found] = i;
					num_found++;
				}else{
					found_more = TRUE;
					break;
				}				

			}// End if (search_whole_string != TRUE)
		}// End if (pdest != NULL)
	}// End for (i=0; i < lines_read; i++)

	return num_found;
}// End Func: int find_eng_string(char *search_str, int whole_string)

// #################################################################################################
// For the purposes of this program, all searches
// should be done using basic Kana ONLY.
// No Kanji should be included in the search.
// If whole_string == TRUE, this will search for an exact match only. Therefore, if the search string
// is 'hi' but the current string is 'hima', the search will fail. This obviously overrides s_type.
// If s_type != OPT_SEARCH_ANY, string must match from
// the start of the string, else can be a match
// for anywhere in the checked string.
// #################################################################################################
int find_SJIS_string(u8 *search_str, int s_type, int whole_string){
	kanjiListing = NONE;
	int i;
	int num_found = 0;
	found_more = FALSE;

	for (i=0; i < lines_read; i++){
		if (whole_string == TRUE){
			// The search string must be a whole match.
			if (strcmp((char *)search_str, (char *)kanji[i].k_kana) == 0){
				if (num_found < MAX_FOUND - 2){
					found_list[num_found] = i;
					num_found++;
				}else{
					found_more = TRUE;
					break;
				}
			}
		}else{
			if (s_type == OPT_SEARCH_ANY){
				// Find in any string position
				if (strstr(kanji[i].k_kana, search_str) != NULL){
					if (num_found < MAX_FOUND - 2){
						found_list[num_found] = i;
						num_found++;
					}else{
						found_more = TRUE;
						break;
					}
				}
			}else{
				// Search from the start of the string.
				// May need to check both strings are not NULL, else could get an error.
				if (strncmp(kanji[i].k_kana, search_str, strlen((char *)search_str)) == 0){
					// exact Match
					if (num_found < MAX_FOUND - 2){
						found_list[num_found] = i;
						num_found++;
					}else{
						found_more = TRUE;
						break;
					}
				}
			}// End if (s_type == OPT_SEARCH_ANY)
		}// End if (whole_string == TRUE)
	}// End for (i=0; i < lines_read; i++)

	return num_found;
}// End Func: int find_SJIS_string(u8 *search_str, int s_type);

// #################################################################################################
// This function is generally called from the Kanji Popup or from a selection of Kanjidic.
// If whole_string == TRUE, this will search for an exact match only. Therefore, if the search string
// is 'hi' but the current string is 'hima', the search will fail. This obviously overrides s_type.
// If s_type != OPT_SEARCH_ANY, string must match from
// the start of the string, else can be a match
// for anywhere in the checked string.
// #################################################################################################
int find_kanji_string(u8 *search_str, int s_type, int whole_string){
	kanjiListing = NONE;
	int i;
	int num_found = 0;
	found_more = FALSE;

	for (i=0; i < lines_read; i++){
		// No guarantee a Kanji actually exists. Need to check.
		if (kanji[i].k_kanji != NULL){
			if (whole_string == TRUE){
				// The search string must be a whole match.
				if (strcmp((char *)search_str, (char *)kanji[i].k_kanji) == 0){
					if (num_found < MAX_FOUND - 2){
						found_list[num_found] = i;
						num_found++;
					}else{
						found_more = TRUE;
						break;
					}
				}
			}else{
				if (s_type == OPT_SEARCH_ANY){
					// Find in any string position
					if (strstr(kanji[i].k_kanji, search_str) != NULL){
						if (num_found < MAX_FOUND - 2){
							found_list[num_found] = i;
							num_found++;
						}else{
							found_more = TRUE;
							break;
						}
					}
				}else{
					// Search from the start of the string.
					// May need to check both strings are not NULL, else could get error.
					if (strncmp(kanji[i].k_kanji, search_str, strlen((char *)search_str)) == 0){
						// exact Match
						if (num_found < MAX_FOUND - 2){
							found_list[num_found] = i;
							num_found++;
						}else{
							found_more = TRUE;
							break;
						}
					}
				}// End if (s_type == OPT_SEARCH_ANY)
			}// End if (whole_string == TRUE)
		}//End if (kanji[i].k_kanji != NULL)
	}// End for (i=0; i < lines_read; i++)

	return num_found;
}// int find_kanji_string(u8 *search_str, int s_type)


// #################################################################################################
// # This is a VERY basic UCS2 to SJIS conversion for Hiragana and Katakana only. No kanji is
// # accounted for.
// # TODO - No error checking for bounds or invalid chars have been included at this stage
// #################################################################################################
int basic_UCS2toSJIS(unsigned short *instr, char *outstr){
	int length = 0;
	int i = 0, j = 0;

	length = oslCccStrlenUCS2(instr);

	//printf("\n");

	for (i = 0; i < length; i++){
		switch((unsigned int)instr[i]){
			case 12354:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12356:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12358:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12360:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12362:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12353:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12355:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12357:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12359:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12361:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12363:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12365:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12367:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12369:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12371:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12364:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12366:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12368:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12370:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12372:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12373:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12375:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12377:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12379:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12381:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12374:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12376:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12378:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12380:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12382:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12383:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12385:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12388:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12390:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12392:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12384:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12386:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12389:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12391:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12393:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12394:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12395:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12396:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12397:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12398:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12399:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12402:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12405:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12408:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12411:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12400:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12403:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12406:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12409:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12412:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12401:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12404:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12407:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12410:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12413:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12414:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12415:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12416:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12417:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12418:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12425:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12426:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12427:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12428:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12429:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12420:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12422:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12424:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12431:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12435:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12288:
				outstr[j] = '';
				outstr[j+1] = '@';
				j=j+2;
				break;
			case 12434:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12419:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12421:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12423:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12430:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12387:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12450:
				outstr[j] = '';
				outstr[j+1] = 'A';
				j=j+2;
				break;
			case 12452:
				outstr[j] = '';
				outstr[j+1] = 'C';
				j=j+2;
				break;
			case 12454:
				outstr[j] = '';
				outstr[j+1] = 'E';
				j=j+2;
				break;
			case 12456:
				outstr[j] = '';
				outstr[j+1] = 'G';
				j=j+2;
				break;
			case 12458:
				outstr[j] = '';
				outstr[j+1] = 'I';
				j=j+2;
				break;
			case 12449:
				outstr[j] = '';
				outstr[j+1] = '@';
				j=j+2;
				break;
			case 12451:
				outstr[j] = '';
				outstr[j+1] = 'B';
				j=j+2;
				break;
			case 12453:
				outstr[j] = '';
				outstr[j+1] = 'D';
				j=j+2;
				break;
			case 12455:
				outstr[j] = '';
				outstr[j+1] = 'F';
				j=j+2;
				break;
			case 12457:
				outstr[j] = '';
				outstr[j+1] = 'H';
				j=j+2;
				break;
			case 12459:
				outstr[j] = '';
				outstr[j+1] = 'J';
				j=j+2;
				break;
			case 12461:
				outstr[j] = '';
				outstr[j+1] = 'L';
				j=j+2;
				break;
			case 12463:
				outstr[j] = '';
				outstr[j+1] = 'N';
				j=j+2;
				break;
			case 12465:
				outstr[j] = '';
				outstr[j+1] = 'P';
				j=j+2;
				break;
			case 12467:
				outstr[j] = '';
				outstr[j+1] = 'R';
				j=j+2;
				break;
			case 12460:
				outstr[j] = '';
				outstr[j+1] = 'K';
				j=j+2;
				break;
			case 12462:
				outstr[j] = '';
				outstr[j+1] = 'M';
				j=j+2;
				break;
			case 12464:
				outstr[j] = '';
				outstr[j+1] = 'O';
				j=j+2;
				break;
			case 12466:
				outstr[j] = '';
				outstr[j+1] = 'Q';
				j=j+2;
				break;
			case 12468:
				outstr[j] = '';
				outstr[j+1] = 'S';
				j=j+2;
				break;
			case 12469:
				outstr[j] = '';
				outstr[j+1] = 'T';
				j=j+2;
				break;
			case 12471:
				outstr[j] = '';
				outstr[j+1] = 'V';
				j=j+2;
				break;
			case 12473:
				outstr[j] = '';
				outstr[j+1] = 'X';
				j=j+2;
				break;
			case 12475:
				outstr[j] = '';
				outstr[j+1] = 'Z';
				j=j+2;
				break;
			case 12477:
				outstr[j] = '';
				outstr[j+1] = '\\';
					j=j+2;
				break;
			case 12470:
				outstr[j] = '';
				outstr[j+1] = 'U';
				j=j+2;
				break;
			case 12472:
				outstr[j] = '';
				outstr[j+1] = 'W';
				j=j+2;
				break;
			case 12474:
				outstr[j] = '';
				outstr[j+1] = 'Y';
				j=j+2;
				break;
			case 12476:
				outstr[j] = '';
				outstr[j+1] = '[';
				j=j+2;
				break;
			case 12478:
				outstr[j] = '';
				outstr[j+1] = ']';
				j=j+2;
				break;
			case 12479:
				outstr[j] = '';
				outstr[j+1] = '^';
				j=j+2;
				break;
			case 12481:
				outstr[j] = '';
				outstr[j+1] = '`';
				j=j+2;
				break;
			case 12484:
				outstr[j] = '';
				outstr[j+1] = 'c';
				j=j+2;
				break;
			case 12486:
				outstr[j] = '';
				outstr[j+1] = 'e';
				j=j+2;
				break;
			case 12488:
				outstr[j] = '';
				outstr[j+1] = 'g';
				j=j+2;
				break;
			case 12480:
				outstr[j] = '';
				outstr[j+1] = '_';
				j=j+2;
				break;
			case 12482:
				outstr[j] = '';
				outstr[j+1] = 'a';
				j=j+2;
				break;
			case 12485:
				outstr[j] = '';
				outstr[j+1] = 'd';
				j=j+2;
				break;
			case 12487:
				outstr[j] = '';
				outstr[j+1] = 'f';
				j=j+2;
				break;
			case 12489:
				outstr[j] = '';
				outstr[j+1] = 'h';
				j=j+2;
				break;
			case 12490:
				outstr[j] = '';
				outstr[j+1] = 'i';
				j=j+2;
				break;
			case 12491:
				outstr[j] = '';
				outstr[j+1] = 'j';
				j=j+2;
				break;
			case 12492:
				outstr[j] = '';
				outstr[j+1] = 'k';
				j=j+2;
				break;
			case 12493:
				outstr[j] = '';
				outstr[j+1] = 'l';
				j=j+2;
				break;
			case 12494:
				outstr[j] = '';
				outstr[j+1] = 'm';
				j=j+2;
				break;
			case 12495:
				outstr[j] = '';
				outstr[j+1] = 'n';
				j=j+2;
				break;
			case 12498:
				outstr[j] = '';
				outstr[j+1] = 'q';
				j=j+2;
				break;
			case 12501:
				outstr[j] = '';
				outstr[j+1] = 't';
				j=j+2;
				break;
			case 12504:
				outstr[j] = '';
				outstr[j+1] = 'w';
				j=j+2;
				break;
			case 12507:
				outstr[j] = '';
				outstr[j+1] = 'z';
				j=j+2;
				break;
			case 12496:
				outstr[j] = '';
				outstr[j+1] = 'o';
				j=j+2;
				break;
			case 12499:
				outstr[j] = '';
				outstr[j+1] = 'r';
				j=j+2;
				break;
			case 12502:
				outstr[j] = '';
				outstr[j+1] = 'u';
				j=j+2;
				break;
			case 12505:
				outstr[j] = '';
				outstr[j+1] = 'x';
				j=j+2;
				break;
			case 12508:
				outstr[j] = '';
				outstr[j+1] = '{';
				j=j+2;
				break;
			case 12497:
				outstr[j] = '';
				outstr[j+1] = 'p';
				j=j+2;
				break;
			case 12500:
				outstr[j] = '';
				outstr[j+1] = 's';
				j=j+2;
				break;
			case 12503:
				outstr[j] = '';
				outstr[j+1] = 'v';
				j=j+2;
				break;
			case 12506:
				outstr[j] = '';
				outstr[j+1] = 'y';
				j=j+2;
				break;
			case 12509:
				outstr[j] = '';
				outstr[j+1] = '|';
				j=j+2;
				break;
			case 12510:
				outstr[j] = '';
				outstr[j+1] = '}';
				j=j+2;
				break;
			case 12511:
				outstr[j] = '';
				outstr[j+1] = '~';
				j=j+2;
				break;
			case 12512:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12513:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12514:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12521:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12522:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12523:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12524:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12525:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case 12516:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12518:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12520:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12527:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12531:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12530:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12515:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12517:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12519:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12526:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12483:
				outstr[j] = '';
				outstr[j+1] = 'b';
				j=j+2;
				break;
			case  12532:
				outstr[j] = '';
				outstr[j+1] = '';
				j=j+2;
				break;
			case  12293:
				outstr[j] = '';
				outstr[j+1] = 'X';
				j=j+2;
				break;
			case  12540:
				outstr[j] = '';
				outstr[j+1] = '[';
				j=j+2;
				break;
			case  8213:
				outstr[j] = '';
				outstr[j+1] = '\\';
					j=j+2;
				break;

		}// End switch(instr[i])
		//printf(",%d", instr[i]);
	}// End for (i = 0; i < length; i++)
	outstr[j] = '\0';
	//printf("\n");

	return j;
}// int basic_UCS2toSJIS(unsigned short *instr, char *outstr)


// #################################################################################################
// # You must have set the font before calling this function!!!
// # When word_wrap != 0, the intention is not to cut off the text, but to attempt to align
// # the text for num_charsx and num_lines. This means that for example if num_charsx was set
// # to 5, but the first word on the line was 8 chars, it will print 8 chars and move to the
// # next line.  It will only truncate the string if it is more than num_lines.
// # If a set width must be maintained, pass the value 0 for word_wrap. This will simply wrap the
// # text around at the character level, not at the word level. Looks messy, but will fit perfectly.
// # Returns the number of lines printed.
// # e.g. IFPrintTextBox(ltn0, mybuttons[i].x + 5,mybuttons[i].y + 25, MAX_PRINT_CHARS_X + 5, MAX_PRINT_LINES, english[mybuttons[i].text_index], WORD_WRAP);
// #################################################################################################
	// Note - This code basically completely based off oslDrawTextBox. Sorry Sakya ;)
	// I just needed the func to return the number of lines drawn (zero based).
	// void oslDrawTextBox(int x0, int y0, int x1, int y1, const char *text, int format);
int IFPrintTextBox(int x0, int y0, int max_width, int max_height, const char *text){

#define BUFF_SIZE 512

	int x,y;// x2;
	unsigned char c;
	const char *text2;

	char buff[BUFF_SIZE] = "\0";
	char tmpbuff[BUFF_SIZE] = "\0";
	char tmpstr[2] = "\0";
	int line_count = 0;
	int index = 0;
	int strwidth = 0;

	x = x0;
	y = y0;

	memset(buff, 0, BUFF_SIZE);
	memset(tmpbuff, 0, BUFF_SIZE);

	while(*text){
		c = *text;
		if (c == ' ' || c == '/'){
			text2 = text;
			memset(tmpbuff, 0, BUFF_SIZE);
			strcpy(tmpbuff, buff);
			do{
				tmpstr[0] = *text2++;
				tmpstr[1] = '\0';
				strcat(tmpbuff, tmpstr);
				strwidth = oslGetStringWidth(tmpbuff);
				if (strwidth > (x0 + max_width)){
					text++;
					goto newline;
				}
			} while(*text2 != '\n' && *text2 != ' ' && *text2 != '/' && *text2);
		}// End if (c == ' ' || c == '/')

		memset(tmpbuff, 0, BUFF_SIZE);
		strcpy(tmpbuff, buff);
		tmpstr[0] = *text;
		tmpstr[1] = '\0';
		strcat(tmpbuff, tmpstr);
		strwidth = oslGetStringWidth(tmpbuff);
		if (strwidth > (x0 + max_width) || *text=='\n'){
newline:
			//Start a newline.
			index = 0;
			// Print the string...
			oslDrawString(x0,y, buff); // NEW
			memset(buff, 0, BUFF_SIZE);
			y += LINE_SPACE_IF_Y;
			line_count++;
			if (y + LINE_SPACE_IF_Y > (y0 + max_height))
				break;

			if (*text == '\n')
				text++;
			continue;
		}
		buff[index] = c;// NEW
		index++;
		text++;
	}// End while(*text)

	// Could still have a string to print here.
	if (buff[0] != 0){
		oslDrawString(x0,y, buff); // NEW
	}
	
	return line_count;//RTN_OK;
}// int IFPrintTextBox(int x0, int y0, int max_width, int max_height, const char *text);

// #################################################################################################
// # You must have set the font before calling this function!!!
// # When word_wrap != 0, the intention is not to cut off the text, but to attempt to align
// # the text for num_charsx and num_lines. This means that for example if num_charsx was set
// # to 5, but the first word on the line was 8 chars, it will print 8 chars and move to the
// # next line.  It will only truncate the string if it is more than num_lines.
// # If a set width must be maintained, pass the value 0 for word_wrap. This will simply wrap the
// # text around at the character level, not at the word level. Looks messy, but will fit perfectly.
// # Returns the number of lines printed (zero based).
// # e.g. IFPrintTextBox(ltn0, mybuttons[i].x + 5,mybuttons[i].y + 25, MAX_PRINT_CHARS_X + 5, MAX_PRINT_LINES, english[mybuttons[i].text_index], WORD_WRAP);
// #################################################################################################
	// Note - This code basically completely based off oslDrawTextBox. Sorry Sakya ;)
	// I just needed the func to return the number of lines drawn (zero based).
	// void oslDrawTextBox(int x0, int y0, int x1, int y1, const char *text, int format);
	// NOTE!!! - This is a SJIS version of the above. Each SJIS char has two bytes, unless it is an ASCII
	// char in which case it will have just one byte.
	// This should work for both ASCII and SJIS strings, but the correct font will need
    // to be set before calling this function.
int IFPrintTextBoxSJIS(int x0, int y0, int max_width, int max_height, const char *text){

#define BUFF_SIZE 512

	int x,y;// x2;
	unsigned char c;
	const char *text2;

	char buff[BUFF_SIZE] = "\0";
	char tmpbuff[BUFF_SIZE] = "\0";
	char tmpstr[3] = "\0";
	//char sjis_char[3] = "\0";
	int line_count = 0;
	int index = 0;
	int strwidth = 0;
	int is_ascii = FALSE;

	x = x0;
	y = y0;

	memset(buff, 0, BUFF_SIZE);
	memset(tmpbuff, 0, BUFF_SIZE);

	while(*text){
		is_ascii = FALSE;
		c = *text;
		// Check if it is an ASCII char
		if (c < 0x7F)
			is_ascii = TRUE;
		if (c == ' '){
			text2 = text;
			memset(tmpbuff, 0, BUFF_SIZE);
			strcpy(tmpbuff, buff);
			do{
				// Check if the next is an ascii or SJIS char.
				if (*(text2) < 0x7F){
					tmpstr[0] = *text2++;
					tmpstr[1] = '\0';
				}else{
					tmpstr[0] = *text2++;
					tmpstr[1] = *text2++;
					tmpstr[2] = '\0';
				}
				strcat(tmpbuff, tmpstr);
				// The correct font must be set before getting here.
				// Or the return value won't be correct.
				strwidth = oslGetStringWidth(tmpbuff);
				if (strwidth > (x0 + max_width)){
					text++;
					goto newline;
				}
			} while(*text2 != '\n' && *text2 != ' ' && *text2);
		}// End if (c == ' ')

		memset(tmpbuff, 0, BUFF_SIZE);
		strcpy(tmpbuff, buff);
		// Check if ascii or SJIS.
		if (is_ascii){
			tmpstr[0] = *text;
			tmpstr[1] = '\0';
		}else{
			tmpstr[0] = *text;
			tmpstr[1] = *(text+1);
			tmpstr[2] = '\0';
		}
		strcat(tmpbuff, tmpstr);
		strwidth = oslGetStringWidth(tmpbuff);
		if (strwidth > (x0 + max_width) || *text=='\n'){
newline:
			//Start a newline.
			index = 0;
			// Print the string...
			oslDrawString(x0,y, buff); // NEW
			memset(buff, 0, BUFF_SIZE);
			y += LINE_SPACE_IF_Y;
			line_count++;
			//Trop bas -> termin
			if (y + LINE_SPACE_IF_Y > (y0 + max_height))
				break;
			//Retour -> saute
			if (*text == '\n')
				text++;
			continue;
		}

		// Add ascii or SJIS
		if (is_ascii){
			buff[index] = c;// NEW
			index++;
			text++;
		}else{
			buff[index] = c;// NEW
			buff[index+1] = *(text+1);// NEW
			index+=2;
			text+=2;
		}
	}// End while(*text)

	// Could still have a string to print here.
	if (buff[0] != 0){
		oslDrawString(x0,y, buff); // NEW
	}
	
	return line_count;//RTN_OK;
}// int IFPrintTextBoxSJIS(int x0, int y0, int max_width, int max_height, const char *text);

// Return the width of the given line 0 zero based.
// Don't use this if the string has a newline char, as it will be unreliable.
// Meant only for straight SJIS text.
int GetIFPrintTextBoxSJISLineWidth(int x0, int y0, int max_width, int max_height, const char *text, int line_num){

#define BUFF_SIZE 512

	int x,y;// x2;
	unsigned char c;
	const char *text2;

	char buff[BUFF_SIZE] = "\0";
	char tmpbuff[BUFF_SIZE] = "\0";
	char tmpstr[3] = "\0";
	//char sjis_char[3] = "\0";
	int line_count = 0;
	int index = 0;
	int strwidth = 0;
	int is_ascii = FALSE;

	x = x0;
	y = y0;

	memset(buff, 0, BUFF_SIZE);
	memset(tmpbuff, 0, BUFF_SIZE);

	while(*text){
		is_ascii = FALSE;
		c = *text;
		// Check if it is an ASCII char
		if (c < 0x7F)
			is_ascii = TRUE;
		if (c == ' '){
			text2 = text;
			memset(tmpbuff, 0, BUFF_SIZE);
			strcpy(tmpbuff, buff);
			do{
				// Check if the next is an ascii or SJIS char.
				if (*(text2) < 0x7F){
					tmpstr[0] = *text2++;
					tmpstr[1] = '\0';
				}else{
					tmpstr[0] = *text2++;
					tmpstr[1] = *text2++;
					tmpstr[2] = '\0';
				}
				strcat(tmpbuff, tmpstr);
				// The correct font must be set before getting here.
				// Or the return value won't be correct.
				strwidth = oslGetStringWidth(tmpbuff);
				if (strwidth > (x0 + max_width)){
					text++;
					goto newline;
				}
			} while(*text2 != '\n' && *text2 != ' ' && *text2);
		}// End if (c == ' ')

		memset(tmpbuff, 0, BUFF_SIZE);
		strcpy(tmpbuff, buff);
		// Check if ascii or SJIS.
		if (is_ascii){
			tmpstr[0] = *text;
			tmpstr[1] = '\0';
		}else{
			tmpstr[0] = *text;
			tmpstr[1] = *(text+1);
			tmpstr[2] = '\0';
		}
		strcat(tmpbuff, tmpstr);
		strwidth = oslGetStringWidth(tmpbuff);
		if (strwidth > (x0 + max_width) || *text=='\n'){
newline:
			//Start a newline.
			index = 0;

			if (line_count == line_num){
				return oslGetStringWidth(buff);
			}
			
			memset(buff, 0, BUFF_SIZE);
			y += LINE_SPACE_IF_Y;
			line_count++;
			//Trop bas -> termin
			if (y + LINE_SPACE_IF_Y > (y0 + max_height))
				break;
			//Retour -> saute
			if (*text == '\n')
				text++;
			continue;
		}

		// Add ascii or SJIS
		if (is_ascii){
			buff[index] = c;// NEW
			index++;
			text++;
		}else{
			buff[index] = c;// NEW
			buff[index+1] = *(text+1);// NEW
			index+=2;
			text+=2;
		}
	}// End while(*text)

	// Could still have a string to print here.
	if (buff[0] != 0){
		if (line_count == line_num){
			return oslGetStringWidth(buff);
		}
	}
	
	return -1; //line_num not encountered.
}// int GetIFPrintTextBoxSJISLineWidth(int x0, int y0, int max_width, int max_height, const char *text, int line_num);



// #################################################################################################
// # When word_wrap != 0, the intention is not to cut off the text, but to attempt to align
// # the text for num_charsx and num_lines. This means that for example if num_charsx was set
// # to 5, but the first word on the line was 8 chars, it will print 8 chars and move to the
// # next line.  It will only truncate the string if it is more than num_lines.
// # If a set width must be maintained, pass the value 0 for word_wrap. This will simply wrap the
// # text around at the character level, not at the word level. Looks messy, but will fit perfectly.
// # Returns the number of lines printed.
// # st_index must have already been set.
// # screen_text array has been previously cleared before calling this function.
// # tmp_str will contain: x, y, offset_y, font_type (ENUM), color, shadowColor and options.
// # NOTE: Before returning, st_index should always be incrimented ready for the next input.
// #################################################################################################
//	Should be able to handle ASCII and SJIS strings.
int IFSetupScreenTextBox(uscreentextel tmp_st, int max_width, int max_height, const char *text){

	#define IFSSTB_BUFF_SIZE 1024

	int x,y;// x2;
	unsigned char c;
	const char *text2;

	char buff[IFSSTB_BUFF_SIZE] = "\0";
	char tmpbuff[IFSSTB_BUFF_SIZE] = "\0";
	char tmpstr[3] = "\0";
	int line_count = 0;
	int index = 0;
	int strwidth = 0;
	int is_ascii = FALSE;
	int length = 0;
	int i = 0;

	if (st_index >= MAX_SCREEN_TEXT)
		return -3;

	length = strlen(text);
	oslSetFont(tmp_st.font_type == FONT_JAPANESE? jpn0 : ltn0);
	oslIntraFontSetStyle(tmp_st.font_type == FONT_JAPANESE? jpn0 : ltn0, tmp_st.size, tmp_st.color, tmp_st.shadowColor, tmp_st.options);
	// remove \n to get the correct width.
	memset(buff, 0, IFSSTB_BUFF_SIZE);
	for (i = 0; i < length; i++){
		if (text[i] == '\n')
			buff[i] = ' ';
		else
			buff[i] = text[i];
	}
	strwidth = oslGetStringWidth(buff);

	//oslDebug("IFSetupScreenTextBox():\nlength = %d.\nstrwidth = %d.", length, strwidth);

	// Check if we need to wrap anything...
	if (strwidth < max_width){
		//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx.");
		screen_text[st_index].x = tmp_st.x;
		screen_text[st_index].y = tmp_st.y;
		screen_text[st_index].offset_y = tmp_st.offset_y;// Used to allow relative screen positions.
		screen_text[st_index].font_type = tmp_st.font_type;
		screen_text[st_index].size = tmp_st.size;
		screen_text[st_index].color = tmp_st.color;
		screen_text[st_index].shadowColor = tmp_st.shadowColor;
		screen_text[st_index].options = tmp_st.options;
		// Allocate memory and copy the string.
		//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Before memory alloc");
		screen_text[st_index].text = (char *) malloc((length + 1) * sizeof(char));
		if (screen_text[st_index].text == NULL){
			oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Memory alloc failed!");
			return -1;
		}
		//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Alloc OK. Memsetting");
		memset(screen_text[st_index].text, 0, length+1);
		//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Copying String");
		strcpy(screen_text[st_index].text, text);
		//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: returning...");
		st_index++;
		return line_count;//RTN_OK;
	}

	if (max_width < kchar_width){
		return -2;
	}

	// Code taken directly from IFPrintTextBox();

	x = tmp_st.x;// Shouldn't change.
	y = tmp_st.y;

	memset(buff, 0, IFSSTB_BUFF_SIZE);
	memset(tmpbuff, 0, IFSSTB_BUFF_SIZE);

	while(*text){
		is_ascii = FALSE;
		c = *text;
		// Check if it is an ASCII char
		if (c < 0x7F)
			is_ascii = TRUE;
		if (c == 0x20 || c == 0x2f){//' ' || c == '/'){
			text2 = text;
			memset(tmpbuff, 0, IFSSTB_BUFF_SIZE);
			strcpy(tmpbuff, buff);
			//oslDebug("Space or Slash found. String =\n%s", tmpbuff);
			do{
				if (*(text2) < 0x7F){
					tmpstr[0] = *text2++;
					tmpstr[1] = '\0';
				}else{
					tmpstr[0] = *text2++;
					tmpstr[1] = *text2++;
					tmpstr[2] = '\0';
				}
				strcat(tmpbuff, tmpstr);
				strwidth = oslGetStringWidth(tmpbuff);
				//oslDebug("In do. tmpstr = %s. *text2 = %c(Hex %x). c = %c(Hex %x)\nstrwidth = %d, max_width = %d.", tmpstr, *text2, *text2, c, c, strwidth, max_width);
				if (strwidth > (max_width)){
					//oslDebug("String \"%s\" (width %d)\nwider than %d.", tmpbuff, strwidth, max_width);
					text++;
					goto newline;
				}
			} while(*text2 != '\n' && *text2 != 0x20 && *text2 != 0x2f && *text2);//' ' && *text2 != '/' && *text2);
		}// End if (c == ' ' || c == '/')

		memset(tmpbuff, 0, IFSSTB_BUFF_SIZE);
		strcpy(tmpbuff, buff);
		// Check if ascii or SJIS.
		if (is_ascii){
			tmpstr[0] = *text;
			tmpstr[1] = '\0';
		}else{
			tmpstr[0] = *text;
			tmpstr[1] = *(text+1);
			tmpstr[2] = '\0';
		}
		strcat(tmpbuff, tmpstr);
		strwidth = oslGetStringWidth(tmpbuff);
		if (strwidth > (/*x0 +*/ max_width) || *text=='\n'){
newline:
			//Start a newline.
			index = 0;
			// Save the string to screen_text array...
//			oslDrawString(x,y, buff); // NEW
			// =========================================
			screen_text[st_index].x = tmp_st.x; // Not required anymore.
			screen_text[st_index].y = y;		// Use y pos or offset_y to tell where to print line.
			screen_text[st_index].offset_y = tmp_st.offset_y;	// Using index to tell the offset.
			screen_text[st_index].font_type = tmp_st.font_type;
			screen_text[st_index].size = tmp_st.size;
			screen_text[st_index].color = tmp_st.color;
			screen_text[st_index].shadowColor = tmp_st.shadowColor;
			screen_text[st_index].options = tmp_st.options;
			// Allocate memory and copy the string.
			length = strlen(buff);
			screen_text[st_index].text = (char *) malloc((length+1) * sizeof(char));
			if (screen_text[st_index].text == NULL){
				return -1;
			}
			memset(screen_text[st_index].text, 0, length + 1);
			strcpy(screen_text[st_index].text, buff);

			// =========================================
			memset(buff, 0, IFSSTB_BUFF_SIZE);
			y += LINE_SPACE_IF_Y;
			line_count++;

			// From old code
			if (st_index + 1 < MAX_SCREEN_TEXT){
				st_index++;
			}else{
				line_count--;// TODO - Check me.
				break;
			}

			// From IFPrintTextBox
			if (y + LINE_SPACE_IF_Y > (tmp_st.y + max_height))
				break;

			if (*text == '\n')
				text++;
			continue;
		}// End if (strwidth > (/*x0 +*/ max_width) || *text=='\n')
		// Add ascii or SJIS
		if (is_ascii){
			buff[index] = c;// NEW
			index++;
			text++;
		}else{
			buff[index] = c;// NEW
			buff[index+1] = *(text+1);// NEW
			index+=2;
			text+=2;
		}
	}// End while(*text)

	// Could still have a string to print here.
	if (buff[0] != 0){
		// Save the string to screen_text array...
//		oslDrawString(x0,y, buff); // NEW
		screen_text[st_index].x = tmp_st.x;
		screen_text[st_index].y = y;
		screen_text[st_index].font_type = tmp_st.font_type;
		screen_text[st_index].size = tmp_st.size;
		screen_text[st_index].color = tmp_st.color;
		screen_text[st_index].shadowColor = tmp_st.shadowColor;
		screen_text[st_index].options = tmp_st.options;
		// Allocate memory and copy the string.
		length = strlen(buff);
		screen_text[st_index].text = (char *) malloc((length+1) * sizeof(char));
		if (screen_text[st_index].text == NULL){
			return -1;
		}
		memset(screen_text[st_index].text, 0, length + 1);
		strcpy(screen_text[st_index].text, buff);
		
		if (st_index + 1 < MAX_SCREEN_TEXT){ // NEW 16/10/09. Didn't incriment before.
			st_index++;
		}else{
			line_count--;// TODO - Check me.
		}
	}// End if (buff[0] != 0)

	return line_count;//RTN_OK;
}// int IFSetupScreenTextBox(uscreentextel tmp_st, int max_width, int max_height, const char *text);


// #################################################################################################
// Note that SJIS strings will be appended with ascii '0' (ZERO)
// in order to tell language from the file name when loading files.
// Works for ASCII and SJIS strings.
// Return value: greater than 0 for an error, or RTN_OK for success.
// #################################################################################################
int SaveFoundSet(void){
	char buffer[256] = "\0";
	char tmpfname[256] = "\0";
	char *pFound_Char = NULL;
	char illegal_file_chars[] = "\\/:*\?\"<>|\0";
	int mystrlen = 0;
	int file_name_exists = FALSE;
	FILE *fSave = NULL;
	int i = 0;
	int rtn = 0;

	// First thing to do is check we actually have something to save.
	if (found_count < 1){
		//printf("SaveFoundSet()<<< No found set exists. Nothing to save. >>>\n");
		return SAV_NO_FOUNDSET;
	}

	//if (osk_language)
	memset(buffer, 0, 256); // clear any existing data
	strcpy(buffer, osk_text);

	mystrlen = strlen(buffer);

	// Need to check for any illegal filename characters, e.g. {\/:*?"<>|}
	// When replacing, don't use common chars like {$,.'}
	if (osk_language == OSK_ENGLISH){ // English (ASCII)
		// Check for illegal characters
		for (i = 0; i < mystrlen; i++){
			pFound_Char = strchr(illegal_file_chars, buffer[i]);
			if (pFound_Char != NULL){
				// Need to replace the illegal character
				switch(buffer[i]){
				// Found in SJIS
					/*bad*/case '\\': buffer[i] = '[';// Cant use - exists in SJIS string.
						break;
					/*OK*/case '/': buffer[i] = ']';// Cant use
						break;
					/*OK*/case ':': buffer[i] = '#';// OK
						break;
					/*OK*/case '*': buffer[i] = '+';// OK
						break;
					/*OK*/case '\?': buffer[i] = '~';// Cant use
						break;
					/*OK*/case '\"': buffer[i] = '^';// Cant use
						break;
					/*OK*/case '<': buffer[i] = '{';// Cant use
						break;
					/*OK*/case '>': buffer[i] = '}';// Cant use
						break;
					/*bad*/case '|': buffer[i] = '!';// OK
						break;
				}// End switch(buffer[i]);
			}
		}
	}// English ASCII string

	if (osk_language == OSK_JAPANESE){
		// Check for illegal characters
		for (i = 0; i < mystrlen; i++){
			pFound_Char = strchr(illegal_file_chars, buffer[i]);
			if (pFound_Char != NULL){
				// Need to replace the illegal character
				switch(buffer[i]){
				// Found in SJIS
					/*bad*/case '\\': buffer[i] = '#';// OK - doesn't exist in SJIS
						break;
					/*bad*/case '|': buffer[i] = '!';// OK - doesn't exist in SJIS
						break;
						//	default:
						//		printf("\n\n<<< Unexpected illegal file char in SJIS string. Aborting... >>>");
						//		c = _getch();
						//		return 7;
						//break;
				}// End switch(buffer[i]);
			}
		}
	}// Japanese SJIS string

	// Now, check if it already exists
	switch(cur_edict){
		case EDICT_ORIG:
			rtn = CreateFileList(saved_finds_file_path, AAID_PEL);
			break;
		case EDICT_COMP:
			rtn = CreateFileList(saved_finds_file_path, AAID_PE2);
			break;
		case EDICT_LEGAL:
			rtn = CreateFileList(saved_finds_file_path, AAID_PE3);
			break;
		case EDICT_LIFE:
			rtn = CreateFileList(saved_finds_file_path, AAID_PE4);
			break;
		case EDICT_CUSTOM:
			rtn = CreateFileList(saved_finds_file_path, AAID_PE5);
			break;
	}
//	rtn = CreateFileList(saved_finds_file_path, AAID_PEL);
	if (rtn == 0){// Error occured.
		//printf("SaveFoundSet()<<< Error in CreateFileList(). Aborting. >>>\n");
		return SAV_CREATEFILELIST_ERR;
	}

	if (osk_language == OSK_JAPANESE){
		//insert '0' to the start of the string if this is a SJIS string to mark it. Used when loading.
		memset(tmpfname, 0, 256);
		strcpy(tmpfname, buffer); // Temporarily store in tmpfname
		memset(buffer, 0, 256);
		strcpy(buffer, "0\0");
		strcat(buffer, tmpfname);
		memset(tmpfname, 0, 256); // clear before use again.
	}

	// Add correct extention
	switch(cur_edict){
		case EDICT_ORIG:
			strcat(buffer, ".pel\0");
			break;
		case EDICT_COMP:
			strcat(buffer, ".pe2\0");
			break;
		case EDICT_LEGAL:
			strcat(buffer, ".pe3\0");
			break;
		case EDICT_LIFE:
			strcat(buffer, ".pe4\0");
			break;
		case EDICT_CUSTOM:
			strcat(buffer, ".pe5\0");
			break;	
	}
//	strcat(buffer, ".pel\0");
	// Save the converted filename for use later. - Don't think I actually need this for operational use.
	strcpy(conv_search_fname, buffer);

	file_name_exists = FALSE;
	if (num_files > 0){
		for (i = 0; i < num_files; i++){
			if(osk_language == OSK_ENGLISH){
				if (stricmp(fname[i], buffer) == 0){// Case insensitive for english string
					file_name_exists = TRUE;
					break;
				}
			}else{
				if (strcmp(fname[i], buffer) == 0){
					file_name_exists = TRUE;
					break;
				}
			}
		}
	}// if (num_files > 0)


	// If it doesn't already exist, save the string.
	if (file_name_exists){
		// Let the user know it already exists.
		//printf("SaveFoundSet()<<< Found set already saved. Aborting. >>>\n");
		return SAV_FOUNDSET_EXISTS;
	}

	// Save the file...
	strcpy(tmpfname, saved_finds_file_path);
	strcat(tmpfname, buffer);
	//strcat(tmpfname, ".pel\0"); // - Done above already
	if( (fSave = fopen( tmpfname, "wb" )) == NULL ){
		//printf( "SaveFoundSet()<<< The file \"%s\"\n could not be created >>>\n", tmpfname );
		return SAV_FOPEN_ERR;
	}

	// Write number of elements
	if (fwrite(&found_count, sizeof(found_count), 1, fSave) != 1){
		//printf( "SaveFoundSet()<<< Error in fwrite - writing found_count >>>\n");
		fclose(fSave);
		return SAV_FWRITE_COUNT_ERR;
	}

	// Before saving elements, sort the data.
	if (sort_type != OPT_SORT_ORIG && found_count > 1){
		// Sort the data into original state.
		qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_orig );
	}

	// Write contents to the file...
	for (i = 0; i < found_count; i++){
		if (found_list[i] != -1){
			if (fwrite(&found_list[i], sizeof(found_list[i]), 1, fSave) != 1){
				//printf( "SaveFoundSet()<<< Error in fwrite - writing found_list[%d] >>>\n", i);
				fclose(fSave);
				// Re-sort the list if appropriate.
				switch (sort_type){
					// Case OPT_SORT_ORIG is already sorted, so don't need to do anything.
					case OPT_SORT_LENGTH:
						qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_length );
						break;
					case OPT_SORT_READING:
						qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_reading );
						break;
				}
				return SAV_FWRITE_LIST_ERR;
			}
		}else{
			break;
		}
	}

	fclose(fSave);

	// re-sort the list if appropriate.
	switch (sort_type){
		// Case OPT_SORT_ORIG is already sorted, so don't need to do anything.
		case OPT_SORT_LENGTH:
			qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_length );
			break;
		case OPT_SORT_READING:
			qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_reading );
			break;
	}
	return RTN_OK;

}// int SaveFoundSet(void);

// #################################################################################################
// List of files have been shown and
// user has selected one, now the index
// is stored in fname_index.
// Return value: RTN_OK for success, or value greater than 0 for error.
// #################################################################################################
int LoadFoundSet(void){
	int i = 0, str_len = 0;
	FILE *fLoad = NULL;
	int tmpint = 0;
	char fnamebuff[256] = "\0";
	char tmpbuff[256] = "\0";

	char *pFound_Char = NULL;
	char *pos = NULL;
	char placeholder_file_chars_ASCII[] = "[]#+~^{}!\0";
	char placeholder_file_chars_SJIS[] = "#!\0";

	if (fname_index < 0){
		return LOAD_ILLEGAL_INDEX;
	}
	
	strcpy(fnamebuff, saved_finds_file_path);
	strcat(fnamebuff, fname[fname_index]);
	
	if( (fLoad = fopen( fnamebuff, "rb" )) == NULL ){
		return LOAD_FOPEN_ERR;
	}


	// Read in count
	if (fread(&tmpint, sizeof(tmpint), 1, fLoad) != 1){
		fclose(fLoad);
		return LOAD_FREAD_COUNT_ERR;
	}

	// Clear list - Do this last thing in case we exit early.
	for (i = 0; i < MAX_FOUND; i++)
		found_list[i] = -1;

	found_count = tmpint;
	
	if (found_count > MAX_FOUND - 2)
		found_count = MAX_FOUND - 2;
	found_index = 0;
	found_more = FALSE;
	// Read contents of the file...
	for (i = 0; i < found_count; i++){
		if (fread(&tmpint, sizeof(found_list[i]), 1, fLoad) == 1){
			found_list[i] = tmpint;
		}else{
			fclose(fLoad);
			return LOAD_FREAD_LIST_ERR;
		}
	}
	fclose(fLoad);

	// sort the list if appropriate.
	switch (sort_type){
		// Case OPT_SORT_ORIG is already sorted, so don't need to do anything.
		case OPT_SORT_LENGTH:
			qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_length );
			break;
		case OPT_SORT_READING:
			qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_reading );
			break;
	}

	 // Now set osk_text to the correct string. - Not really needed now, but may be useful later.

//-----------------------------------------------------------------------------------------------------------
	memset(tmpbuff, 0, sizeof(tmpbuff));
	strcpy(tmpbuff, fname[fname_index]);
	str_len = strlen(tmpbuff);

	// Remove the file extention
	pos = strrchr(tmpbuff, '.');
	if (pos == NULL){ // Not found.
		//printf("ShowFiles()<<< Error in strrchr(). Aborting. >>>\n");
		//oslMessageBox("GS_SHOW_FILES - Error in strrchr().", "Display Error", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, 0, 0, 0, 0));
		return LOAD_STRRCHR_ERR;
	}

	pos--;
	strncpy(tmpbuff, tmpbuff, pos - tmpbuff);
	tmpbuff[str_len-4] = '\0';
	str_len = strlen(tmpbuff);
	
	if (tmpbuff[0] == '0'){
		tmpint = 0; // Used to decide what type of string to print later.
		// Japanese SJIS string
		// Take out the '0' char from index 0.
		strcpy(tmpbuff, tmpbuff+1);
		str_len--;
		// Convert placeholder characters back to illegal characters
		for (i = 0; i < str_len; i++){
			pFound_Char = strchr(placeholder_file_chars_SJIS, tmpbuff[i]);
			if (pFound_Char != NULL){
				// Need to replace the illegal character
				switch(tmpbuff[i]){
					// Found in SJIS
					/*bad*/case '#': tmpbuff[i] = '\\';// OK - doesn't exist in SJIS
						break;
					/*bad*/case '!': tmpbuff[i] = '|';// OK - doesn't exist in SJIS
						break;
				}// End switch(tmpbuff[i]);
			}
		}
	}else{
		// English ASCII string
		tmpint = 1; // Used to decide what type of string to print later.
		// Convert placeholder characters back to illegal characters
		for (i = 0; i < str_len; i++){
			pFound_Char = strchr(placeholder_file_chars_ASCII, tmpbuff[i]);
			if (pFound_Char != NULL){
				// Need to replace the illegal character
				switch(tmpbuff[i]){
					// Found in SJIS
					/*bad*/case '[': tmpbuff[i] = '\\';// Cant use - exists in SJIS string.
						break;
					/*OK*/case ']': tmpbuff[i] = '/';// Cant use
						break;
					/*OK*/case '#': tmpbuff[i] = ':';// OK
						break;
					/*OK*/case '+': tmpbuff[i] = '*';// OK
						break;
					/*OK*/case '~': tmpbuff[i] = '\?';// Cant use
						break;
					/*OK*/case '^': tmpbuff[i] = '\"';// Cant use
						break;
					/*OK*/case '{': tmpbuff[i] = '<';// Cant use
						break;
					/*OK*/case '}': tmpbuff[i] = '>';// Cant use
						break;
					/*bad*/case '!': tmpbuff[i] = '|';// OK
						break;
				}// End switch(tmpbuff[i]);
			}
		}
	}// End if (fname[i][0] == '0')

	strcpy(osk_text, tmpbuff);
	osk_language = prev_osk_language = tmpint;
	//oslDebug("LoadFoundSet. osk_text = %s, osk_language = %d", osk_text, osk_language);

	
//-----------------------------------------------------------------------------------------------------------

	return RTN_OK;

}// int LoadFoundSet(void);

// #################################################################################################
// List of files have been shown.
// The index is stored in fname_index.
// Return value: RTN_OK for success, or value greater than 0 for error.
// #################################################################################################
int DeleteFoundSet(void){
	int rtn = 0, prev_fname_index = 0;
	char fnamebuff[256] = "\0";


	if (fname_index < 0 || fname_index >= MAX_SAVED_FILE_INDEX || num_files < 1){
		return DEL_ILLEGAL_INDEX;
	}
	
	strcpy(fnamebuff, saved_finds_file_path);
	strcat(fnamebuff, fname[fname_index]);

	//if (!CheckFileExists(fnamebuff))
	//	return DEL_FILE_NOT_FOUND;

	// Found, now delete the file.
	//rtn = sceIoRemove(fnamebuff);
	//oslDebug("File to delete is %s.", fnamebuff);
	if (MEDEBUG) oslDebug("Before Calling remove(%s);", fnamebuff);
	//rtn = _unlink(fnamebuff);
	rtn = remove(fnamebuff);
	if (MEDEBUG) oslDebug("remove returned %d.", rtn);
	if (rtn != 0){
		return DEL_DID_NOT_DEL; // Let calling function handle this.
	}else{
		num_files--;
		//sceIoSync(saved_finds_file_path, 1); // Do here?
		//sceIoSync("ms0://", 0); // Do here?
		// Reset file list.
		if (fname_index > 0)
			prev_fname_index = fname_index - 1;

		if (MEDEBUG) oslDebug("Before calling CreateFileList(%s,%d);", saved_finds_file_path, AAID_PEL);
		rtn = CreateFileList(saved_finds_file_path, AAID_PEL);
		if (MEDEBUG) oslDebug("CreateFileList returned %d.", rtn);
		if (rtn == 0){
			return DEL_CREATEFILELIST_ERR;
		}

		fname_index = prev_fname_index;

		if (MEDEBUG) oslDebug("DeleteFoundSet returning RTN_OK");
		return RTN_OK;
	}

}// int DeleteFoundSet(void)

// #################################################################################################
// Returns: 0 on error; sets create_file_list_errno to the appropriate error value.
//         -1 for no files in dir
//          or, number of valid files found
// #################################################################################################
int CreateFileList(char *dirstr, int FileExtFilter){
	int i = 0;
	int dfd = 0;
	int ext_id = 0;
	int count = 0;
	int keep_going = TRUE;
	int rtn = 0;

	if (MEDEBUG) oslDebug("Clearing the Array...");
	// Clear file name array first.
	for (i = 0; i < MAX_SAVED_FILE_INDEX; i++){
		
		if (fname[i] != NULL){
			free(fname[i]);
			fname[i] = NULL; // forgot this and caused major errors when deleting files.
		}
	}
	if (MEDEBUG) oslDebug("After clearing the Array...");
	num_files = fname_index = 0;

	if (MEDEBUG) oslDebug("Before calling sceIoDopen.");
	dfd = sceIoDopen(dirstr);
	if (MEDEBUG) oslDebug("After sceIoDopen, dfd = %d.", dfd);
	if(dfd >= 0){
		num_files = 0; // Global Var

		count = 0;
		keep_going = TRUE;
		while (num_files < MAX_SAVED_FILE_INDEX){// && keep_going == TRUE){
			count++;
			if (MEDEBUG) oslDebug("In while, before calling sceIoDread");
			if ((rtn = sceIoDread(dfd, &mydir)) <= 0){
				if (MEDEBUG) oslDebug("sceIoDread returned %d.", rtn);
				break;
			}


			if (mydir.d_name[0] == '.'){
				if (MEDEBUG) oslDebug("Dir found, continuing...");
				continue;
			}

			// Check if using a file type filter
			if (FileExtFilter == AAID_ANY){
				ext_id = AAID_ANY;
			}else{
				if (MEDEBUG) oslDebug("Checking file filter...");
				ext_id = getExtId(mydir.d_name);
			}

			if (ext_id == FileExtFilter){
				if (MEDEBUG) oslDebug("Filter OK. Allocating Memory.");
				fname[num_files] = (char *) malloc(strlen(mydir.d_name) + 1);
				if (fname[num_files] == NULL){
					if (MEDEBUG) oslDebug("Allocating Memory failed");
					//oslDebug("Error. malloc returned NULL");
					create_file_list_errno = CREATE_FL_MALLOC_ERR;
					sceIoDclose(dfd);
					return 0;
				}
				//oslDebug("malloc for fname returned OK");
				if (MEDEBUG) oslDebug("Memsetting and copying name");
				memset(fname[num_files], 0, strlen(mydir.d_name) + 1);
				strcpy(fname[num_files], mydir.d_name);
				num_files++;
				if (MEDEBUG) oslDebug("Copying name complete. num_files = %d", num_files);
				//oslDebug("added file %s.", mydir.d_name);
			}
		}// End while (num_files < MAX_SAVED_FILE_INDEX)
		sceIoDclose(dfd);
		if (MEDEBUG) oslDebug("after sceIoDclose");
	}
	else{
		//oslDebug("Error. sceIoDopen returned %i.", dfd);
		if (MEDEBUG) oslDebug("Error. create_file_list_errno = CREATE_FL_DIR_ERR");
		create_file_list_errno = CREATE_FL_DIR_ERR;
		return 0;
	}

	// Got here so did we get any files?
	if (num_files == 0){
		//oslDebug("Error. No files have been read.");
		if (MEDEBUG) oslDebug("num_files == 0");
		return -1;
	}

	create_file_list_errno = RTN_OK;
	//oslDebug("About to leave func CreateFileList, num_files = %d", num_files);

	if (MEDEBUG) oslDebug("Returning. num_files = %d", num_files);
	return num_files;

}// int CreateFileList(char *dirstr, int FileExtFilter);

int getExtId(char *szFilePath){
	char *pszExt;

	if((pszExt = strrchr(szFilePath, '.'))){
		pszExt++;
		if (stricmp("txt",pszExt) == 0){
			return AAID_TXT;
		}else{
			if (stricmp("mp3",pszExt) == 0){
				return AAID_MP3;
			}else{
				if (stricmp("pel",pszExt) == 0){
					return AAID_PEL;
				}else{
					if (stricmp("pe2",pszExt) == 0){
						return AAID_PE2;
					}else{
						if (stricmp("pe3",pszExt) == 0){
							return AAID_PE3;
						}else{
							if (stricmp("pe4",pszExt) == 0){
								return AAID_PE4;
							}
							else{
								if (stricmp("pe5",pszExt) == 0){
									return AAID_PE5;
								}
							}
						}
					}
				}
			}
		}
	}
	return AAID_UNKNOWN;
}// int getExtId(char *szFilePath);

void ResetFoundSet(void){
	int i = 0;

	// Clear list - Do this last thing in case we exit early.
	for (i = 0; i < MAX_FOUND; i++)
		found_list[i] = -1;

	found_index = found_count = 0;
}

// #################################################################################################
// Done at startup - Checks all four study lists exist,
// and if no, creates them.
// #################################################################################################
int CheckStudyLists(void){
	char tmpbuff[256] = "\0";
	int i = 0, j = 0, rtn = 0;
	int x = 0;

	// Clear study list array and set defaults
	for (j = 0; j < MAX_NUM_STUDY_LIST; j++){
		for (i = 0; i < MAX_STUDY_LIST; i++){
			studylist[j][i].index = -1;
			studylist[j][i].marked = FALSE;
		}
		studylist_index[j] = studylist_count[j] = 0;
	}

	studylist_enabled = FALSE;
	cur_studylist = 0;

	// Check if the four study lists exist, if not, create them
	for (x = 0; x < MAX_EDICT; x++){
		for (j = 0; j < MAX_NUM_STUDY_LIST; j++){
			strcpy(tmpbuff, study_list_dirname);
			strcat(tmpbuff, studylist_fname[x][j]);
			if (CheckFileExists(tmpbuff) != TRUE){
				// Create a default file
				rtn = CreateDefaultStudyList(x, j);
				if (rtn != RTN_OK){
					//oslDebug("CreateDefaultStudyList(%d) returned = %d.\nfilename = %s.", j, rtn, tmpbuff);
					// We have a problem, and can't continue to use study lists.
					studylist_enabled = FALSE;
					return STUDY_LIST_CHK_CREATE_FILE_ERR; // Error Creating file.
				}
			}
			//// At this stage, the file should at least exist.
			//// Do we want to load the file to check contents here?
			//if (x == EDICT_ORIG){
			//	rtn = LoadStudyList(EDICT_ORIG, j);
			//	if ( rtn != RTN_OK ){
			//		studylist_enabled = FALSE;
			//		return STUDY_LIST_CHK_LOAD_FILE_ERR; // Error Loading file.
			//	}
			//}
		}
	}// Checking file exists, and loading contents (for orig edict).


	studylist_enabled = TRUE;
	cur_studylist = 0;
	return RTN_OK;

}// int CheckStudyLists(void)

// Returns TRUE if exists, or FALSE.
int CheckFileExists(char *file_fullpathname){
	FILE *fTest = NULL;

	if( (fTest = fopen( file_fullpathname, "rb" )) == NULL ){
		return FALSE; // Error, couldn't create file.
	}else{
		fclose(fTest);
		return TRUE;
	}

}// int CheckFileExists(char *file_fullpathname)

// Returns RTN_OK on success, or positive interger for error number.
int CreateDefaultStudyList(int edict_type, int listnum){
	char tmpbuff[256] = "\0";
	int i = 0;
	FILE *fTest = NULL;

	if (listnum < 0 || listnum >= MAX_NUM_STUDY_LIST)
		return 1; // Index error

	// Create filename String
	strcpy(tmpbuff, study_list_dirname);
	strcat(tmpbuff, studylist_fname[edict_type][listnum]);

	if( (fTest = fopen( tmpbuff, "wb" )) == NULL ){
		return 2; // Error, couldn't create file.
	}

	i = 0;
	// Write only a zero index to the file.
	if (fwrite(&i, sizeof(i), 1, fTest) != 1){
		//printf( "SaveFoundSet()<<< Error in fwrite - writing found_count >>>\n");
		fclose(fTest);
		return 3;
	}

	fclose(fTest);

	return RTN_OK;

}// int CreateDefaultStudyList(int listnum);

// At this stage, the file should at least exist.
int LoadAllStudyLists(int edict_type){
	int j = 0, rtn = 0;

	for (j = 0; j < MAX_NUM_STUDY_LIST; j++){		
		rtn = LoadStudyList(edict_type, j);
		if ( rtn != RTN_OK ){
			studylist_enabled = FALSE;
			return STUDY_LIST_CHK_LOAD_FILE_ERR; // Error Loading file.
		}
	}

	// All lists, counts and index have been set.
	cur_studylist = 0;
	
	return RTN_OK;
}

// Returns RTN_OK for success or non negative int for error.
int LoadStudyList(int edict_type, int listnum){
	int i = 0;
	FILE *fLoad = NULL;
	int tmpint = 0;
	char tmpbuff[256] = "\0";

	if (listnum < 0 || listnum >= MAX_NUM_STUDY_LIST){
		return 1; // Illegal Index
	}
	
	// Create filename String
	strcpy(tmpbuff, study_list_dirname);
	strcat(tmpbuff, studylist_fname[edict_type][listnum]);
	
	if( (fLoad = fopen( tmpbuff, "rb" )) == NULL ){
		return 2; // File Error
	}


	// Read in count
	if (fread(&tmpint, sizeof(tmpint), 1, fLoad) != 1){
		fclose(fLoad);
		return 3; // Read Error - Index
	}

	// Clear list - Do this last thing in case we exit early.
	for (i = 0; i < MAX_STUDY_LIST; i++){
		studylist[listnum][i].index = -1;
		studylist[listnum][i].marked = FALSE;
	}

	studylist_count[listnum] = tmpint;

	// Do sanity checks
	if (studylist_count[listnum] > MAX_STUDY_LIST)
		studylist_count[listnum] = MAX_STUDY_LIST;

	if (studylist_count[listnum] < 0)
		studylist_count[listnum] = 0;

	if (studylist_count[listnum] > 0){
		// Read contents of the file...
		for (i = 0; i < studylist_count[listnum]; i++){
			if (fread(&tmpint, sizeof(tmpint), 1, fLoad) == 1){
				studylist[listnum][i].index = tmpint;
			}else{
				fclose(fLoad);
				return 4; // Read Error - List
			}
		}
	}

	studylist_index[listnum] = 0; // New 26/10/09

	fclose(fLoad);

	return RTN_OK;

}// int LoadStudyList(int listnum)

int SaveStudyList(int edict_type, int listnum){
	char tmpfname[256] = "\0";
	FILE *fSave = NULL;
	int i = 0;

	// Check index if it is within bounds
	if (listnum < 0 || listnum >= MAX_NUM_STUDY_LIST){
		return 1; // Illegal Index
	}

	// Create filename String
	strcpy(tmpfname, study_list_dirname);
	strcat(tmpfname, studylist_fname[edict_type][listnum]);

	if( (fSave = fopen( tmpfname, "wb" )) == NULL ){// OK to overwrite
		return 2; // File Error.
	}

	if (studylist_count[listnum] < 0)
		studylist_count[listnum] = 0;

	if (studylist_count[listnum] > MAX_STUDY_LIST)
		studylist_count[listnum] = MAX_STUDY_LIST;

	// Write number of elements
	if (fwrite(&studylist_count[listnum], sizeof(int), 1, fSave) != 1){
		fclose(fSave);
		return 3; // Write Error - Index
	}

	if (studylist_count[listnum] > 0){
		// Write contents to the file...
		for (i = 0; i < studylist_count[listnum]; i++){
			if (studylist[listnum][i].index != -1){
				if (fwrite(&studylist[listnum][i].index, sizeof(int), 1, fSave) != 1){
					fclose(fSave);
					return 4; // Write Error - List
				}
			}else{
				break;
			}
		}
	}

	fclose(fSave);

	return RTN_OK;

}// int SaveStudyList(int listnum);

// #################################################################################################
// If returns error (not incl list full) should consider
// setting studylist_enabled = FALSE;
// Should be called during GS_SHOW_SEL (only where prev_game_status eq GS_PLAYING or GS_SHOW_RESULTS).
// Return: RTN_OK for success, or non-negative int for error.
//         returns STUDY_LIST_FULL_ERR if the list is full. Should show a message.
// #################################################################################################
int AddToStudyList(int listnum, int edict_index){
	int i = 0, rtn = 0;

	// Check if listnum is valid
	if (listnum < 0 || listnum >= MAX_NUM_STUDY_LIST){
		return 5; // Illegal Index
	}

	// Makesure edict_index is within bounds also
	if (edict_index < 0 || edict_index >= /*MAX_LINES*/edict_max[cur_edict]){
		return 6; // Illegal Index
	}

	if (studylist_count[listnum] >=  (MAX_STUDY_LIST - 1)){
		return STUDY_LIST_FULL_ERR; // List is already full!
	}

	// Does it already exist in the list
	for (i = 0; i < studylist_count[listnum]; i++){
		if (studylist[listnum][i].index == edict_index)
			return STUDY_LIST_ENTRY_EXISTS_ERR;
	}

	// Add the entry
	studylist[listnum][studylist_count[listnum]].index = edict_index;
	studylist[listnum][studylist_count[listnum]].marked = FALSE;
	studylist_count[listnum]++;

	// Resave study list.
	rtn = SaveStudyList(cur_edict, listnum);
	if (rtn != RTN_OK)
		return rtn;

	return RTN_OK;

}// int AddToStudyList(int listnum);

// #################################################################################################
// Negative value for list_index will delete all.
// Should only be called when in the Study List view.
// #################################################################################################
int DelFromStudyList(int listnum, int list_index){ // Use a negative list_index to delete all.
	int i = 0, rtn = 0;

	// Check if listnum is valid
	if (listnum < 0 || listnum >= MAX_NUM_STUDY_LIST){
		return 5; // Illegal Index
	}

	// Makesure list_index is within bounds also
	if (list_index >= MAX_STUDY_LIST){
		return 6; // Illegal Index
	}

	// Generally should be checked before getting here.
	if ( studylist_count[listnum] < 1 ){
		return STUDY_LIST_EMPTY_ERR; // List already empty
	}

	// negative index means delete all in list.
	// User has already been asked if they wish
	// to proceed.

	if (list_index < 0){
		// Delete all
		for (i = (studylist_count[listnum] - 1); i >= 0; i--){
			studylist[listnum][i].index = -1;
			studylist[listnum][i].marked = FALSE;
		}

		studylist_count[listnum] = 0;
		studylist_index[listnum] = 0;

	}else{
		// - If at the last index, just delete and -- list_count. (need to fix index too!).
		// - If not at last, need to move everything else down one,
		//   then set the last index to default value.
		// Remove selected entry
		if (list_index == studylist_count[listnum] - 1){// Could only be 1 entry
			studylist[listnum][list_index].index = -1;
			studylist[listnum][list_index].marked = FALSE;
			studylist_count[listnum]--;
			if (list_index > 0)
				studylist_index[listnum]--;
		}else{
			// Need to move everything above list_index down one.
			// Should work if the first element is selected (i.e. index == 0;)
			// Shouldn't need to update the index here.
			for (i = list_index; i < (studylist_count[listnum] - 1); i++){
				studylist[listnum][i].index = studylist[listnum][i + 1].index;
				studylist[listnum][i].marked = studylist[listnum][i + 1].marked;
			}
			// Set the last one to default values.
			studylist[listnum][studylist_count[listnum] - 1].index = -1;
			studylist[listnum][studylist_count[listnum] - 1].marked = FALSE;
			studylist_count[listnum]--;
		}
	}

	// Resave study list.
	rtn = SaveStudyList(cur_edict, listnum);
	if (rtn != RTN_OK)
		return rtn;

	return RTN_OK;

}// int DelFromStudyList(int listnum, int list_index);

// When the Kanjidic Popup is shown, this function handles processing the input events.
// The function CheckKanjiPopup checks to see if we can enter popup mode.
int CheckKanjiPopupInput(OSL_CONTROLLER *keys){
	if (show_kdict){
		if (keys->pressed.circle){
			if (cursor.x == show_kdict_x && cursor.y == show_kdict_y){
				// Don't show this one again.
				kdict_shown = TRUE;
			}else
				kdict_shown = FALSE;
			show_kdict = FALSE;
			//break;
			return 0;
		}
		if (keys->pressed.cross){
			if (cursor.x == show_kdict_x && cursor.y == show_kdict_y){
				// Don't show this one again.
				kdict_shown = TRUE;
			}else
				kdict_shown = FALSE;
			show_kdict = FALSE;
			// Search for the Kanji
			found_count = find_kanji_string(kanjidict[kdict_search_index].kanji, search_type, match_whole);
			// Sort the found set.
			if (found_count > 1){ // Added 18/10/09
				// Sort the data
				switch (sort_type){
					// Case OPT_SORT_ORIG is already sorted, so don't need to do anything.
					case OPT_SORT_LENGTH:
						qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_length );
						break;
					case OPT_SORT_READING:
						qsort( (void *)found_list, (size_t)found_count, sizeof( int * ), found_compare_reading );
						break;
				}
			}

			is_finding = FALSE;
			found_index = 0;
			strcpy(osk_text, kanjidict[kdict_search_index].kanji);
			osk_language = OSK_JAPANESE;
			//game_status = GS_SHOW_RESULTS;
			//prev_game_status = GS_PLAYING;
			SetGS(GS_LEVEL_02, GS_SHOW_RESULTS, TRUE); // New 24/10/09
			//break;
			return 0;
		}// End if (keys->pressed.cross)

		//// =======================  Debugging only - REMOVE FOR RELEASE =====================
		//if (keys->pressed.L){ // Move to prev file
		//	if ((kdict_search_index - 1) >= 0 ){
		//		kdict_search_index--;
		//		kdict_index = kdict_search_index; // NEW 14/10/09
		//		SetupScreenTextPopup();// NEW 12/10/09
		//		kanji_popup_cur_y = 0;// NEW 15/10/09
		//	}
		//	//break;
		//	return 0;
		//}
		//if (keys->pressed.R){ // Move to next file
		//	if ((kdict_search_index + 1) < kdict_lines_read ){
		//		kdict_search_index++;
		//		kdict_index = kdict_search_index; // NEW 14/10/09
		//		SetupScreenTextPopup();// NEW 12/10/09
		//		kanji_popup_cur_y = 0;// NEW 15/10/09
		//	}
		//	//break;
		//	return 0;
		//}
		//// ================================  End Debugging ================================

		// Check scrolling Kanji Popup info
		if (kanji_popup_end_y > LINE_SPACE_IF_Y * SHOW_KDICT_MAX_LINES){
			if (keys->pressed.up){ // Scroll Back
				if (kanji_popup_cur_y - LINE_SPACE_IF_Y > 0)
					kanji_popup_cur_y -= LINE_SPACE_IF_Y;
				else
					kanji_popup_cur_y = 0;
				//break;
				return 0;
			}
			if (keys->pressed.down){ // Scroll Forward
				if (kanji_popup_cur_y + LINE_SPACE_IF_Y <= (kanji_popup_end_y - (LINE_SPACE_IF_Y * SHOW_KDICT_MAX_LINES)))
					kanji_popup_cur_y += LINE_SPACE_IF_Y;
				else
					kanji_popup_cur_y = kanji_popup_end_y - (LINE_SPACE_IF_Y * SHOW_KDICT_MAX_LINES);
				//break;
				return 0;
			}
		}
	}// End if (show_kdict)
	return RTN_OK;

}// int CheckKanjiPopupInput(void);

// index is zero based, and total is the actual count.
// Works fairly OK with x and y positions (not just index positions), so won't change it for now.
void DrawScrollbars(int x, int y, int length, int size_type, int index, int total){
	int x1, y1, length1;

	x1 = x;
	y1 = y;
	length1 = length;

	if (size_type != SCROLLBAR_BIG && size_type != SCROLLBAR_SML){
		return;
	}

	if (length1 < (size_type==SCROLLBAR_BIG? SCROLLBAR_DOT_HEIGHT : SCROLLBAR_SML_DOT_HEIGHT))
		return;
	if (length1 > (size_type==SCROLLBAR_BIG? SCROLLBAR_HEIGHT : SCROLLBAR_SML_HEIGHT))
		length1 = size_type==SCROLLBAR_BIG? SCROLLBAR_HEIGHT : SCROLLBAR_SML_HEIGHT;
	if (x1 < 0)
		x1 = 0;
	if (y1 < 0)
		y1 = 0;
	if (x1 > (AA_SCREEN_WIDTH - (size_type==SCROLLBAR_BIG?SCROLLBAR_WIDTH:SCROLLBAR_SML_WIDTH)-8))
		x1 = (AA_SCREEN_WIDTH - (size_type==SCROLLBAR_BIG?SCROLLBAR_WIDTH:SCROLLBAR_SML_WIDTH)-8);
	if (y1 > AA_SCREEN_HEIGHT - length)
		y1 = AA_SCREEN_HEIGHT - length;

	if (index < 0)
		return;
	if (total < 1)
		return;

	// Don't allow divide by zero to occur
	if (index == 0){
		// Dot goes to the start of the Bar.
	}
	
	oslSetTransparentColor(RGB(255, 0, 255));
	// Make semi transparent? - TODO
	// Draw the body, then the DOT
	spr_cur->x = x1;
	spr_cur->y = y1;
	oslSetImageTileSize(spr_cur, (size_type==SCROLLBAR_BIG? SCROLLBAR_SRC_X : SCROLLBAR_SML_SRC_X), (size_type==SCROLLBAR_BIG? SCROLLBAR_SRC_Y : SCROLLBAR_SML_SRC_Y), (size_type==SCROLLBAR_BIG? SCROLLBAR_WIDTH : SCROLLBAR_SML_WIDTH), length1);
	oslDrawImage(spr_cur);

	// Work out placement of the DOT
	spr_cur->x = x1 + (size_type==SCROLLBAR_BIG? SCROLLBAR_DOT_OFFSET_X : SCROLLBAR_SML_DOT_OFFSET_X);
	// Basic algoritm is as follows:
	// DOT.Y = BAR.Y + ((BAR.HEIGHT-DOT.HEIGHT)/(total-1))*(index)
	if (index == 0){
		// Dot goes to the start of the Bar.
		scrollbar[1].y = y1;
	}else{
		scrollbar[1].y = y1 + ((length1-(size_type==SCROLLBAR_BIG? SCROLLBAR_DOT_HEIGHT : SCROLLBAR_SML_DOT_HEIGHT)) / ((float)total-1) * index);
	}
	spr_cur->y = scrollbar[1].y;
	oslSetImageTileSize(spr_cur, (size_type==SCROLLBAR_BIG? SCROLLBAR_DOT_SRC_X : SCROLLBAR_SML_DOT_SRC_X), (size_type==SCROLLBAR_BIG? SCROLLBAR_DOT_SRC_Y : SCROLLBAR_SML_DOT_SRC_Y), (size_type==SCROLLBAR_BIG? SCROLLBAR_DOT_WIDTH : SCROLLBAR_SML_DOT_WIDTH), (size_type==SCROLLBAR_BIG? SCROLLBAR_DOT_HEIGHT : SCROLLBAR_SML_DOT_HEIGHT));
	oslDrawImage(spr_cur);
	oslDisableTransparentColor();

}// void DrawScrollbars(int x, int y, int length, int size_type, int index, int total);

void DrawCursor(void){
	if (cursor.enabled == TRUE){
		oslSetTransparentColor(RGB(255, 0, 255));
		spr_cur->x = cursor.x;
		spr_cur->y = cursor.y;
		oslSetImageTileSize(spr_cur, cursor.sourcex,cursor.sourcey,cursor.width, cursor.height);
		oslDrawImage(spr_cur);
		oslDisableTransparentColor();
	}
}

void DrawTitle(int x, int y){
	// Draw Title
	oslSetTransparentColor(RGB(255, 0, 255));
	spr_cur->x = x;//TITLE_SCREENPOS_X;
	spr_cur->y = y;//TITLE_SCREENPOS_Y;
	oslSetImageTileSize(spr_cur, TITLE_X,TITLE_Y,TITLE_WIDTH,TITLE_HEIGHT);
	oslDrawImage(spr_cur);
	oslDisableTransparentColor();
	// Draw version number
	oslSetFont(ltn0);
	oslIntraFontSetStyle(ltn0, 0.5f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	oslDrawString(x /*TITLE_SCREENPOS_X*/ + TITLE_WIDTH + 6, y /*TITLE_SCREENPOS_Y*/ + LINE_SPACE_IF_Y, VER_NUM_STR);
	oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
}

void DrawMenu(void){
	int i = 0, j = 0;
	static int menu_tmp_y0 = 0;

	oslSetTransparentColor(RGB(255, 0, 255));

	if (menu_status != M_CLOSED){ // Must be open or closing
		if (menu_status == M_OPENING){
			if (menu_counter == 0){
				menu_tmp_y0 = 2;
			}
			if (menu_counter + 1 > MENU_MAX_TICK){
				menu_status = M_OPEN;
				menu_tmp_y0 = 2;
				oslDrawFillRect(MENU_X, MENU_Y, MENU_WIDTH, (AA_SCREEN_HEIGHT - 1), RGBA(15, 15, 15, 208));
				// Snap cursor to Options button
				//if (cursor.y <= MENU_Y){// If not in range of menu
				cursor.x = mnu_icon[MENU_OPT].x + MENU_ICON_WIDTH;
				cursor.y = mnu_icon[MENU_OPT].y + MENU_ICON_HEIGHT;
				//}
			}else{
				oslDrawFillRect(MENU_X, (AA_SCREEN_HEIGHT - 1) - menu_tmp_y0, MENU_WIDTH, (AA_SCREEN_HEIGHT - 1), RGBA(15, 15, 15, 208));
				menu_counter += 1;
				menu_tmp_y0 += 2;
			}
		}else{
			if (menu_status == M_CLOSING){
				if (menu_counter - 1 < 0 || menu_tmp_y0 > MENU_HEIGHT){
					menu_status = M_CLOSED;
					menu_counter = 0;					
				}else{
					oslDrawFillRect(MENU_X, MENU_Y + menu_tmp_y0, MENU_WIDTH, (AA_SCREEN_HEIGHT - 1), RGBA(15, 15, 15, 208));
					menu_counter -= 1;
					menu_tmp_y0 += 2;
					if (menu_tmp_y0 < 0)
						menu_tmp_y0 = 0;
				}
			}else{            // ========== M E N U   I S   O P E N ===========
				// Draw full Menu
				oslDrawFillRect(MENU_X, MENU_Y, MENU_WIDTH, (AA_SCREEN_HEIGHT - 1), RGBA(15, 15, 15, 208));

//				strcpy(tmpbuff, "\0");
				//MENU_FIND
				if (game_status == GS_SHOW_RESULTS)
					mnu_icon[MENU_FIND].enabled = FALSE;
				else
					mnu_icon[MENU_FIND].enabled = TRUE;

				if (game_status == GS_SHOW_EDICT_FILES)
					mnu_icon[MENU_EDICT_LOAD].enabled = FALSE;
				else
					mnu_icon[MENU_EDICT_LOAD].enabled = TRUE;

				// Is the base Edict or Kanjidic
				if (gs_level[GS_LEVEL_01] == GS_PLAYING){
					mnu_icon[MENU_EDICT].enabled = FALSE;
					mnu_icon[MENU_KDICT].enabled = TRUE;
				}else{
					mnu_icon[MENU_EDICT].enabled = TRUE;
					mnu_icon[MENU_KDICT].enabled = FALSE;
				}

				if (game_status == GS_SHOW_STUDYLIST){
					mnu_icon[MENU_LOAD].enabled = FALSE;
					mnu_icon[MENU_SAVE].enabled = FALSE;
					mnu_icon[MENU_STUDY].enabled = FALSE;
					mnu_icon[MENU_STUDY_ADD].enabled = FALSE;
					mnu_icon[MENU_STUDY_MINUS].enabled = TRUE;
					mnu_icon[MENU_STUDY_MINUS_ALL].enabled = TRUE;

				}else{
					mnu_icon[MENU_LOAD].enabled = TRUE;
					mnu_icon[MENU_SAVE].enabled = TRUE;
					mnu_icon[MENU_STUDY].enabled = TRUE; // Default
					mnu_icon[MENU_STUDY_ADD].enabled = FALSE; // Default.
					mnu_icon[MENU_STUDY_MINUS].enabled = FALSE;
					mnu_icon[MENU_STUDY_MINUS_ALL].enabled = FALSE;

					if (game_status == GS_PLAYING || game_status == GS_SHOW_RESULTS)
						mnu_icon[MENU_STUDY_ADD].enabled = TRUE;

					if (game_status == GS_SHOW_SEL){
						if (prev_game_status == GS_PLAYING || prev_game_status == GS_SHOW_RESULTS){
							mnu_icon[MENU_STUDY_ADD].enabled = TRUE;
						}

						if (prev_game_status == GS_SHOW_KANJIDICT){
							mnu_icon[MENU_KDICT].enabled = FALSE;
						}//else{
						//	mnu_icon[MENU_KDICT].enabled = TRUE;
						//}

						if (prev_game_status == GS_SHOW_STUDYLIST){
							mnu_icon[MENU_STUDY].enabled = FALSE;
						}
					}

				}

				// If there was an error with the Study Lists, disable related menu options.
				if (studylist_enabled == FALSE){
					mnu_icon[MENU_STUDY].enabled = FALSE;
					mnu_icon[MENU_STUDY_ADD].enabled = FALSE;
					mnu_icon[MENU_STUDY_MINUS].enabled = FALSE;
					mnu_icon[MENU_STUDY_MINUS_ALL].enabled = FALSE;
				}

				for (i = 0; i < MAX_MENU_ICON; i++){
					if (mnu_icon[i].mouse_over && mnu_icon[i].enabled){
						oslDrawFillRect(mnu_icon[i].x - MENU_ICON_OFFSET, mnu_icon[i].y - MENU_ICON_OFFSET, mnu_icon[i].x + MENU_ICON_WIDTH + MENU_ICON_OFFSET, mnu_icon[i].y + MENU_ICON_HEIGHT + MENU_ICON_OFFSET, RGBA(115, 115, 115, 186));//RGBA(198, 198, 198, 186));
						oslSetFont(ltn0);
						oslIntraFontSetStyle(ltn0, 1.0f,WHITE,BLACK,INTRAFONT_ALIGN_LEFT);
						switch (i){
							case MENU_EXIT:
								j = oslGetStringWidth("Exit from PSPedict.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Exit from PSPedict.");
								break;
							case MENU_SAVE:
								j = oslGetStringWidth("Save current found set.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Save current found set.");
								break;
							case MENU_SEARCH:
								j = oslGetStringWidth("Return to the start of current list.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Return to the start of current list.");
								break;
							case MENU_LOAD:
								j = oslGetStringWidth("Load a found set.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Load a found set.");
								break;
							case MENU_EDICT_LOAD:
								j = oslGetStringWidth("Load an Edict file.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Load an Edict file.");
								break;
							case MENU_OPT:
								j = oslGetStringWidth("Show Options.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Show Options.");
								break;
							case MENU_FIND:
								j = oslGetStringWidth("Return to last found set.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Return to last found set.");
								break;
							case MENU_KDICT:
								j = oslGetStringWidth("Show Kanjidic.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Show Kanjidic.");
								break;
							case MENU_EDICT:
								j = oslGetStringWidth("Show Edict.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Show Edict.");
								break;
							case MENU_STUDY:
								j = oslGetStringWidth("Show Study Lists.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Show Study Lists.");
								break;
							case MENU_STUDY_ADD:
								j = oslGetStringWidth("Add entry to current Study List.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Add entry to current Study List.");
								break;
							case MENU_STUDY_MINUS:
								j = oslGetStringWidth("Delete entry from current Study List.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Delete entry from current Study List.");
								break;
							case MENU_STUDY_MINUS_ALL:
								j = oslGetStringWidth("Delete ALL entries from current Study List.");
								oslDrawString((AA_SCREEN_WIDTH - j)/2, AA_SCREEN_HEIGHT - 20,"Delete ALL entries from current Study List.");
								break;
						}
					}// End if (mnu_icon[i].mouse_over)
					if (mnu_icon[i].enabled){
						spr_cur->x = mnu_icon[i].x;
						spr_cur->y = mnu_icon[i].y;
						oslSetImageTileSize(spr_cur, mnu_icon[i].sourcex, mnu_icon[i].sourcey, mnu_icon[i].width, mnu_icon[i].height);
						oslDrawImage(spr_cur);
					}
				}// End for (i = 0; i < MAX_MENU_ICON; i++)
			}// End if (menu_status == M_CLOSING)
		}// End if (menu_status == M_OPENING)
	}// End if (menu_status != M_CLOSED)

	oslDisableTransparentColor();

}// void DrawMenu(void)

// This func actually uses text drawn to a drawbuffer, not 
// the screen_text array as some of the other functions use.
void DrawKanjiPopup(void){
	int y = 0; //,x = 0;
	static int kdict_tmp_x0 = 0, kdict_tmp_y0 = 0, kdict_tmp_x1 = 0, kdict_tmp_y1 = 0;
	//int rtn = 0;

	if (show_kdict){
		if (show_kdict_tick == 0){
			kdict_tmp_x0 = kdict_tmp_x1 = 239;
			kdict_tmp_y0 = kdict_tmp_y1 = 135;
		}
		if (show_kdict_tick <= 25){
			show_kdict_tick++;
			kdict_tmp_x0 -= 8;
			kdict_tmp_y0 -= 4;
			kdict_tmp_x1 += 8;
			kdict_tmp_y1 += 4;
			oslDrawFillRect(kdict_tmp_x0, kdict_tmp_y0, kdict_tmp_x1, kdict_tmp_y1, RGB(255, 230, 172));
			oslDrawRect(kdict_tmp_x0, kdict_tmp_y0, kdict_tmp_x1, kdict_tmp_y1, RGB(5, 5, 5));
		}
		else{
			if (kdict_search_index >= 0 && kdict_search_index < MAX_KANJIDICT){

				SetupScreenTextPopup();

				oslDisableTransparentColor();

				oslDrawFillRect(SHOW_KDICT_RECT_X, SHOW_KDICT_RECT_Y, SHOW_KDICT_RECT_X + SHOW_KDICT_RECT_WIDTH, SHOW_KDICT_RECT_Y + SHOW_KDICT_RECT_HEIGHT, RGB(255, 230, 172));
				oslDrawRect(SHOW_KDICT_RECT_X, SHOW_KDICT_RECT_Y, SHOW_KDICT_RECT_X + SHOW_KDICT_RECT_WIDTH, SHOW_KDICT_RECT_Y + SHOW_KDICT_RECT_HEIGHT, RGB(5, 5, 5));
				
				// Draw other kanji info here.

				//// ### Debugging ###
				//oslSetFont(ltn0);
				//oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_RIGHT);
				//oslDrawStringf(AA_SCREEN_WIDTH - 100, HELP_Y *2 ,"%d", kdict_search_index + 1);
				////oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				////oslDrawStringf(50, AA_SCREEN_HEIGHT - 20, "(%d/%d)", kanji_popup_cur_y, kanji_popup_end_y);

				// Draw Kanji
				oslSetFont(jpn0);
				oslIntraFontSetStyle(jpn0, 1.4f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				y = SHOW_KDICT_TEXT_Y;
				oslDrawString(SHOW_KDICT_TEXT_X, y+3,(char *)kanjidict[kdict_search_index].kanji);
				
				// Draw Stroke Count
				oslSetFont(ltn0);
				oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				oslDrawStringf(SHOW_STROKE_TEXT_X, y, "Stroke Count: %d", kanjidict[kdict_search_index].num_strokes);

				// Draw SKIP
				oslSetFont(ltn0);
				oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
				oslDrawStringf(SHOW_SKIP_TEXT_X, y, "/ SKIP: %d %d %d", kanjidict[kdict_search_index].skipCode[0], kanjidict[kdict_search_index].skipCode[1], kanjidict[kdict_search_index].skipCode[2]);
				
				y += LINE_SPACE_IF_Y;

				// Now draw the text prev drawn to spr_cur texture.
				drawBuf->x = SHOW_KDICT_TEXT_X;
				drawBuf->y = y;
				oslSetImageTileSize(drawBuf, 0, kanji_popup_cur_y, SHOW_KDICT_RECT_WIDTH - SHOW_KDICT_TEXT_INDENT_X - 1, (LINE_SPACE_IF_Y * SHOW_KDICT_MAX_LINES));
				oslDrawImage(drawBuf);

				// Draw Scrollbar if necessarry.
				if (kanji_popup_end_y > LINE_SPACE_IF_Y * SHOW_KDICT_MAX_LINES)
					DrawScrollbars(SCROLLBAR_SML_X, SCROLLBAR_SML_Y, SCROLLBAR_SML_HEIGHT, SCROLLBAR_SML, kanji_popup_cur_y, kanji_popup_end_y - (LINE_SPACE_IF_Y * SHOW_KDICT_MAX_LINES));

				// Start of actual printing = (SHOW_KDICT_TEXT_Y + LINE_SPACE_IF_Y)
				// Bottom of allowable printing area = ((SHOW_KDICT_RECT_Y + SHOW_KDICT_RECT_HEIGHT) - LINE_SPACE_IF_Y)
				// e.g. don't need to draw more than kanji_popup_end_y - ((SHOW_KDICT_RECT_HEIGHT - LINE_SPACE_IF_Y) - (SHOW_KDICT_TEXT_Y + LINE_SPACE_IF_Y))

				// Draw instructions
				oslIntraFontSetStyle(ltn0, 0.8f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
				oslDrawString(SHOW_KDICT_TEXT_X + 3, (SHOW_KDICT_RECT_Y + SHOW_KDICT_RECT_HEIGHT + LINE_IF_OFFSET_Y) - LINE_SPACE_IF_Y, "X - Search kanji");
				y = oslGetStringWidth("( ) - Return");
				oslDrawString(SHOW_KDICT_RECT_X + SHOW_KDICT_RECT_WIDTH - SHOW_KDICT_TEXT_INDENT_X - 3 - y, (SHOW_KDICT_RECT_Y + SHOW_KDICT_RECT_HEIGHT + LINE_IF_OFFSET_Y) - LINE_SPACE_IF_Y, "( ) - Return");
				oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

				oslDisableTransparentColor();

			}// End if (kdict_search_index >= 0 && kdict_search_index < MAX_KANJIDICT)
		}// End if (show_kdict_tick <= 25/*50*/)
	}// End if (show_kdict)

}// void DrawKanjiPopup(OSL_IMAGE **sprite, OSL_FONT **font_ltn0, OSL_FONT **font_jpn0);

// ######################################################################################
// # Called from Drawing loop for GS_SHOW_SEL.
// # Can be used for game_status == GS_SHOW_SEL where prev_game_stats == GS_SHOW_KANJIDICT.
// # Note that SetupScreenTextKdic() must have already been called.
// # See function information for SetupScreenTextKdic for more info.
// ######################################################################################
int DrawScreenTextKdic(void){
	int i = 0;
	int y = 0, y2 = 0;
	int tmp_y = 0, max_y = 0;

	////Debugging
	//oslSetFont(ltn0);
	//oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_RIGHT);
	//oslDrawStringf(AA_SCREEN_WIDTH - 100, HELP_Y,"%d", kdict_index + 1);
	// Reset setting in case I forget!
	//oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

	// Draw Kanji
	y = HELP_Y;
	oslSetFont(jpn0);
	oslIntraFontSetStyle(jpn0, 1.4f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	oslDrawString(HELP_X, y-2,(char *)kanjidict[kdict_index].kanji);
	// Draw Stroke Count
	oslSetFont(ltn0);
	oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	oslDrawStringf(HELP_X + KDICT_STROKE_OFFSET_X, y, "Stroke Count: %d", kanjidict[kdict_index].num_strokes);
	// Draw SKIP
	oslSetFont(ltn0);
	oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	oslDrawStringf(HELP_X + KDICT_SKIP_OFFSET_X, y, "/ SKIP: %d %d %d", kanjidict[kdict_index].skipCode[0], kanjidict[kdict_index].skipCode[1], kanjidict[kdict_index].skipCode[2]);
	
	y += LINE_SPACE_IF_Y;
	y2 = y;

	// Now we know y after printing Kanji and Kana,
	// Check if we need to draw scrollbars.

	// May need to move this code into Setup func, and
	// store as global vars.
	//int tmp_y = 0, max_y = 0;
	max_y = y2;
	tmp_y = y2;
	max_y += LINE_SPACE_IF_Y;
	//tmp_y += LINE_SPACE_IF_Y;
	for (i = 1; i < MAX_SCREEN_TEXT; i++){
		if (screen_text[i].text == NULL){
			break;
		}
		max_y += screen_text[i-1].offset_y;
		//if (i <= st_index + 1)
		if (i - 1 < st_index)
			tmp_y += screen_text[i-1].offset_y;
	}


	if (max_y > (MAX_ST_HEIGHT - LINE_SPACE_IF_Y))
		DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, tmp_y - y2, max_y - (MAX_ST_HEIGHT - LINE_SPACE_IF_Y));

	// Now print vertically scrollable screen_text.
	// Note, this differs from the loop in DrawScreenTextEdict() as we're using offset_y, not y pos
	// to move through the list.
	st_can_scroll = TRUE;
	for (i = st_index; i < MAX_SCREEN_TEXT; i++){
		// Check we got something to draw
		if (screen_text[i].text == NULL){
			// No more to draw
			st_can_scroll = FALSE;
			break;
		}


		if ((y + screen_text[i].offset_y) >= MAX_ST_HEIGHT){
			break;
		}

		oslSetFont(screen_text[i].font_type == FONT_JAPANESE? jpn0 : ltn0);
		oslIntraFontSetStyle(screen_text[i].font_type == FONT_JAPANESE? jpn0 : ltn0, screen_text[i].size, screen_text[i].color, screen_text[i].shadowColor, screen_text[i].options);
		//oslDrawString(HELP_X + screen_text[i].x, HELP_Y + screen_text[i].y, screen_text[i].text);
		oslDrawString(HELP_X + screen_text[i].x, y, screen_text[i].text);
		y += screen_text[i].offset_y;
	}// End for (i = 0; i < MAX_SCREEN_TEXT; i++)

	// Draw Search String
	oslSetFont(ltn0);
	oslIntraFontSetStyle(ltn0, 0.8f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
	oslDrawStringf(HELP_X, AA_SCREEN_HEIGHT - 20, "X - Search kanji");
	oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

	return RTN_OK;
}// int DrawScreenTextKdic(void);

// ######################################################################################
// # Called from Drawing loop for GS_SHOW_SEL.
// # Can be used for game_status == GS_SHOW_SEL where prev_game_stats ==
// # GS_PLAYING || GS_SHOW_RESULTS || GS_SHOW_STUDYLIST.
// # Note that SetupScreenTextEdict(int index) must have already been called.
// # See function information for SetupScreenTextEdict for more info.
// ######################################################################################
int DrawScreenTextEdict(int index){
	int i = 0, rtn = 0;
	int y = 0, y2 = 0;
	int tmp_width = 0;
	int tmpsk_index = 0;
	//int tmpint = 0;

	// Draw Kanji/Kana separate. 
	// They generally won't be long enough to need to scroll them.
	// Only scrolling English Descrip

	y = HELP_Y;
	rtn = 0;
	ClearScreenKanjiList(); // Moved 7/11/09
	if (kanji[index].k_kanji != NULL){

		oslSetFont(jpn0);
		oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
		rtn = IFPrintTextBoxSJIS(HELP_X, y, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), MAX_PRINT_LINES * LINE_SPACE_IF_Y, (char *)kanji[index].k_kanji);

		// Now account for kanji on multiple lines.
		// Uses a function to return width for a particular line.
		// Possibly more process intensive, but seems to work OK.
		// Other choices, were to just cheat and use x -> end of the screen (could cause possible issues though).
		// Or save the actual string to search later. Went with first option though, and it seems to work.

		// Debug
		//whole_width = oslGetStringWidth((char *)kanji[index].k_kanji);

		//ClearScreenKanjiList();
		for (i = 0; i <= rtn; i++){
			tmp_width = GetIFPrintTextBoxSJISLineWidth(HELP_X, y, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), MAX_PRINT_LINES * LINE_SPACE_IF_Y, (char *)kanji[index].k_kanji, i);
			screenkanji[i].index = index;
			screenkanji[i].start_x = HELP_X;
			screenkanji[i].start_y = (y + (i*LINE_SPACE_IF_Y)) - SK_OFFSET_Y;//HELP_Y + y - SK_OFFSET_Y;
			screenkanji[i].num_chars = (tmp_width / kchar_width);
			screenkanji[i].kanji_type = EDICT_KANJI;
		}
		tmpsk_index = i;// Will be the next available index.
		//oslDebug("tmpsk_index = %d", tmpsk_index);
	}
	// If no kanji, just have a blank line. Originally moved the kana up a line, but this looked weird.
	y += (rtn+1)*LINE_SPACE_IF_Y;

	oslSetFont(jpn0);
	oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);

	rtn = IFPrintTextBoxSJIS(HELP_X, y, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), MAX_PRINT_LINES * LINE_SPACE_IF_Y, (char *)kanji[index].k_kana);

	y += (rtn+1)*LINE_SPACE_IF_Y;
	y2 = y;

	// Now we know y after printing Kanji and Kana,
	// Check if we need to draw scrollbars.
	//tmpint = (MAX_ST_HEIGHT_EDICT - y); // For Testing
	if (((st_count) * LINE_SPACE_IF_Y) >= (MAX_ST_HEIGHT_EDICT - y)-LINE_SPACE_IF_Y)
		DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, st_index, st_count - (((MAX_ST_HEIGHT_EDICT - y - (LINE_SPACE_IF_Y*2))/LINE_SPACE_IF_Y)-1) );

	// Now print vertically scrollable english descrip.
	st_can_scroll = TRUE;
	for (i = st_index; i < MAX_SCREEN_TEXT; i++){
		// Check we got something to draw
		if (screen_text[i].text == NULL){
			// No more to draw
			st_can_scroll = FALSE;
			break;
		}

		if (y > (MAX_ST_HEIGHT_EDICT - y2)){
			break;
		}

		// See if we need to update screenkanji list?? - Will this be too slow? No, seems OK.
		if (screen_text[i].font_type == FONT_JAPANESE){
			if (HasKanji((u8 *)screen_text[i].text)){
				// Add to screenkanji array
				oslSetFont(jpn0);
				oslIntraFontSetStyle(jpn0, screen_text[i].size, screen_text[i].color, screen_text[i].shadowColor, screen_text[i].options);
				tmp_width = GetIFPrintTextBoxSJISLineWidth(HELP_X/*HELP_X + screen_text[i].x*/, y, AA_SCREEN_WIDTH - (HELP_X/*HELP_X + screen_text[i].x*/) - (kchar_width*3), MAX_PRINT_LINES * LINE_SPACE_IF_Y, screen_text[i].text, 0);
				screenkanji[tmpsk_index].index = i;//index;
				screenkanji[tmpsk_index].start_x = HELP_X + screen_text[i].x;
				screenkanji[tmpsk_index].start_y = y - SK_OFFSET_Y; // Added offset 14/11/09.
				screenkanji[tmpsk_index].num_chars = (tmp_width / kchar_width);
				screenkanji[tmpsk_index].kanji_type = TC_KANJI;
				tmpsk_index++;
			}
		}


		// Draw the text
		oslSetFont(screen_text[i].font_type == FONT_JAPANESE? jpn0 : ltn0);
		oslIntraFontSetStyle(screen_text[i].font_type == FONT_JAPANESE? jpn0 : ltn0, screen_text[i].size, screen_text[i].color, screen_text[i].shadowColor, screen_text[i].options);
		oslDrawString(HELP_X + screen_text[i].x, y, screen_text[i].text);
		//oslDrawString(HELP_X + screen_text[i].x, y+screen_text[i].y, screen_text[i].text);
		y += LINE_SPACE_IF_Y;
	}// End for (i = 0; i < MAX_SCREEN_TEXT; i++)

	//y = AA_SCREEN_HEIGHT - (LINE_SPACE_IF_Y * 6);
	//// Debugging
	//if (whole_width != 0){
	//	oslDrawFillRect(200, y, AA_SCREEN_WIDTH, AA_SCREEN_HEIGHT, RGB(255, 255, 255));
	//	oslDrawStringf(200, y, "whole width = %d", whole_width);
	//	y += LINE_SPACE_IF_Y;
	//	for (i=0; i < 5; i++){
	//		if (line_width[i] != -1){
	//			oslDrawStringf(200, y+(i*LINE_SPACE_IF_Y), "line%d width = %d", i, line_width[i]);
	//		}else
	//			break;
	//	}
	//}
	//oslDrawStringf(HELP_X, AA_SCREEN_HEIGHT - LINE_SPACE_IF_Y, "ix=%d,1.nc=%d,x=%d,y=%d,i=%d", tmpsk_index, screenkanji[1].num_chars, screenkanji[1].start_x, screenkanji[1].start_y, screenkanji[1].index);

	//oslSetFont(ltn0);
	//oslIntraFontSetStyle(ltn0, 0.7f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	//oslDrawStringf(AA_SCREEN_WIDTH - 120, LINE_SPACE_IF_Y*2, "(%d/%d)", cursor.x, cursor.y);
	//oslDrawStringf(HELP_X, AA_SCREEN_HEIGHT - LINE_SPACE_IF_Y, "st_cnt=%d,st_ix=%d,M_S_HGT-y=%d", st_count, st_index, tmpint);

	//oslDebug("DrawScreenTextEdict():\nLeaving Function.");

	return RTN_OK;
}// int DrawScreenTextEdict(OSL_FONT *font_jpn, OSL_FONT *font_ltn, int index);


// ######################################################################################
// # SetupScreenTextEdict is run when the user click the X button on a list
// # meaning they want to select that line to view. Failing to call SetupScreenTextEdict
// # means the english description that shows up will belong to some other entry. Not good.
// # This function must also be called whenever we are already on GS_SHOW_SEL and the user
// # moves forwards or backwards between the prev/next selection. Failure to call this func
// # will result in the last english description showing.
// # It may have been simpler to call SetupScreenTextEdict from the drawing function
// # DrawScreenTextEdict, but I wanted to remove the overhead of having to setup the
// # screen_text array everytime the screen is drawn. It is only called when the selection
// # changes.
// # SetupScreenTextEdict fills the screen_text array with the english
// # description lines to print. They can be scrolled vertically if the
// # text is longer than the screen height.
// # Used when we are on GS_SHOW_SEL and prev_game_status ==	GS_PLAYING || GS_SHOW_RESULTS || 
// # GS_SHOW_STUDYLIST.
// # Called from CheckButtons().
// # tmp_str will contain: x, y, OSL_FONT *, color, shadowColor and options.
// # Changed Behaviour: Now only used for the english description from edict.
// #                    Kanji/Kana hardly ever go over. They are drawn using a IF_PrintTextBox from
// #                    the drawing function.
// # 7/11/09 - Added support for Tanaka Corpus example sentences.
// ######################################################################################
int SetupScreenTextEdict(int index){
	int i = 0, rtn = 0, x = 0, y = 0;
	//int tmpnum = 0;

	//oslDebug("SetupScreenTextEdict():\nClearing screen_text array.\nkchar_width = %d.\nindex = %d.", kchar_width, index);
	// First, clear the screen_text array
	for (i = 0; i < MAX_SCREEN_TEXT; i++){
		screen_text[i].x = 0;
		screen_text[i].y = 0;
		screen_text[i].offset_y = LINE_SPACE_IF_Y;
		screen_text[i].font_type = FONT_LATIN;
		if (screen_text[i].text != NULL){
			free(screen_text[i].text);
			screen_text[i].text = NULL;
		}
		screen_text[i].size = 1.0f;
		screen_text[i].color = BLACK;
		screen_text[i].shadowColor = WHITE;
		screen_text[i].options = INTRAFONT_ALIGN_LEFT;
	}

	st_index = st_count = 0;
	st_can_scroll = TRUE;

	y = x = 0;

	// Draw English Description
	tmp_screen_text.x = x;
	tmp_screen_text.y = y;
	tmp_screen_text.font_type = FONT_LATIN;
	tmp_screen_text.size = 1.0f;
	tmp_screen_text.color = BLACK;
	tmp_screen_text.shadowColor = WHITE;
	tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

	//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
	rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, (char *)kanji[index].e_descrip);

	if (rtn == -1){
		// Error allocating memory. Exit.
		oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
		endOSLib();
		return 0;
	}

	y += (rtn+1)*LINE_SPACE_IF_Y;

	// NOTE - Would need to rethink this if we end up using many examples. It would probably not
	//        be appropriate to show 10 TC sentences for example. Would have to select them at random or something.
	if (show_examples){
		for (i=0; i < MAX_TC_FILEPOS_COUNT; i++){
			if (kanji[index].TCfilepos[i] == -1)
				break;

			rtn = LoadTCLine(fTCName, kanji[index].TCfilepos[i], &tmpTC);

			if (rtn == RTN_OK){
				//y += LINE_SPACE_IF_Y;
				// Add a spacer
				tmp_screen_text.x = x;
				tmp_screen_text.y = y;
				tmp_screen_text.font_type = FONT_LATIN;
				tmp_screen_text.size = 1.0f;
				tmp_screen_text.color = BLACK;
				tmp_screen_text.shadowColor = WHITE;
				tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

				//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
				rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, " ");

				if (rtn == -1){
					// Error allocating memory. Exit.
					oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
					endOSLib();
					return 0;
				}

				y += (rtn+1)*LINE_SPACE_IF_Y;

				// === Add the example. ===
				tmp_screen_text.x = x;
				tmp_screen_text.y = y;
				tmp_screen_text.font_type = FONT_LATIN;
				tmp_screen_text.size = 1.0f;
				tmp_screen_text.color = RED;
				tmp_screen_text.shadowColor = WHITE;
				tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

				//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
				rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, "Ex:\0");

				if (rtn == -1){
					// Error allocating memory. Exit.
					oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
					endOSLib();
					return 0;
				}

				y += (rtn+1)*LINE_SPACE_IF_Y;

				// First Japanese
				tmp_screen_text.x = x;
				tmp_screen_text.y = y;
				tmp_screen_text.font_type = FONT_JAPANESE;//FONT_LATIN;
				tmp_screen_text.size = 1.0f;
				tmp_screen_text.color = BLUE;
				tmp_screen_text.shadowColor = WHITE;
				tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

				//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
				rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, (char *)tmpTC.j_scentence);

				if (rtn == -1){
					// Error allocating memory. Exit.
					oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
					endOSLib();
					return 0;
				}

				y += (rtn+1)*LINE_SPACE_IF_Y;

				// Second English
				tmp_screen_text.x = x;
				tmp_screen_text.y = y;
				tmp_screen_text.font_type = FONT_LATIN;
				tmp_screen_text.size = 1.0f;
				tmp_screen_text.color = BLACK;
				tmp_screen_text.shadowColor = WHITE;
				tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

				//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
				rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, tmpTC.e_scentence);

				if (rtn == -1){
					// Error allocating memory. Exit.
					oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
					endOSLib();
					return 0;
				}
			}// End if (rtn == RTN_OK)
		}// End for (i=0; i < MAX_TC_FILEPOS_COUNT; i++)
	}// End if (show_examples)

	// Need to save the end?
	st_count = st_index; // st_index should have already been increased ready for the next input.// + 1;
	st_index = 0;

	//oslDebug("SetupScreenTextEdict():\nLeaving Function.\nst_count = %d", st_count);

	return RTN_OK;
}// int SetupScreenTextEdict(int index);



// Used when we are on GS_SHOW_SEL and prev_game_status ==	GS_SHOW_KANJIDICT
int SetupScreenTextKdic(void){
	int i = 0, rtn = 0, x = 0, y = 0;

	// First, clear the screen_text array
	for (i = 0; i < MAX_SCREEN_TEXT; i++){
		screen_text[i].x = 0;
		screen_text[i].y = 0;
		screen_text[i].offset_y = LINE_SPACE_IF_Y;
		screen_text[i].font_type = FONT_LATIN;
		if (screen_text[i].text != NULL){
			free(screen_text[i].text);
			screen_text[i].text = NULL;
		}
		screen_text[i].size = 1.0f;
		screen_text[i].color = BLACK;
		screen_text[i].shadowColor = WHITE;
		screen_text[i].options = INTRAFONT_ALIGN_LEFT;
	}

	st_index = st_count = 0;
	st_can_scroll = TRUE;

	//// Debugging Only
	//if (kdict_index == 6){
	//	oslSetFont(jpn0);
	//	oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	//	oslDebug("String Width to end = %d\nFirst char = 0x%x.\nSecond char = 0x%x.",oslGetStringWidth(kanjidict[6].KunYomi), kanjidict[6].KunYomi[0], kanjidict[6].KunYomi[1]);
	//	//oslDebug("String Width to end = %d\nFirst char = 0x%x.\nSecond char = 0x%x.",cccStrlenSJIS(kanjidict[6].KunYomi), kanjidict[6].KunYomi[0], kanjidict[6].KunYomi[1]);
	//}

	y = x = 0;

	// ############
	// Draw On Yomi
	// ############
	// Set String "On:  " first.
	screen_text[st_index].x = x;
	screen_text[st_index].y = y;
	// If On Yomi exists, then set offset_y to 0, meaning, draw on the same line.
	if (kanjidict[kdict_index].OnYomi != NULL){
		screen_text[st_index].offset_y = 0;// Next string on the same line.
		screen_text[st_index].color = BLACK;
	}else{
		screen_text[st_index].offset_y = LINE_SPACE_IF_Y;
		y += LINE_SPACE_IF_Y;
		screen_text[st_index].color = GRAY;
	}
	screen_text[st_index].font_type = FONT_LATIN;
	// Save the string.
	screen_text[st_index].text = (char *) malloc((strlen("On:  ") + 1) * sizeof(char));
	if (screen_text[st_index].text == NULL){
		oslDebug("SetupScreenTextKdic():\nOn Yomi(%d): Memory alloc failed! Aborting.", kdict_index);
		endOSLib();
		return 0;
	}
	//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Alloc OK. Memsetting");
	memset(screen_text[st_index].text, 0, strlen("On:  ")+1);
	//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Copying String");
	strcpy(screen_text[st_index].text, "On:  \0");

	screen_text[st_index].size = 1.0f;
	screen_text[st_index].shadowColor = WHITE;
	screen_text[st_index].options = INTRAFONT_ALIGN_LEFT;

	st_index++;

	// Now check if we have an On Yomi to print.
	if (kanjidict[kdict_index].OnYomi != NULL){
		oslSetFont(ltn0);
		oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
		x += oslGetStringWidth("On:  ");
		//oslSetFont(jpn0);
		//oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
		tmp_screen_text.x = x;
		tmp_screen_text.y = y;
		tmp_screen_text.offset_y = LINE_SPACE_IF_Y;
		tmp_screen_text.font_type = FONT_JAPANESE;
		tmp_screen_text.size = 1.0f;
		tmp_screen_text.color = MAGENTA;
		tmp_screen_text.shadowColor = WHITE;
		tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

		//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
		//rtn = IFSetupScreenTextBox(tmp_screen_text, MAX_PRINT_CHARS_X, 100, (char *)kanji[index].e_descrip, WORD_WRAP);
		rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - x - (kchar_width*5), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, (char *)kanjidict[kdict_index].OnYomi);

		if (rtn == -1){
			// Error allocating memory. Exit.
			oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
			endOSLib();
			return 0;
		}
		y += (rtn+1)*LINE_SPACE_IF_Y;
	}// End if (kanjidict[kdict_index].OnYomi != NULL)


	// TODO - Make sure st_index is set correct, and we won't over write something.
	// #############
	// Draw Kun Yomi
	// #############
	x = 0;
	// Set String "Kun: " first.
	screen_text[st_index].x = x;
	screen_text[st_index].y = y;
	// If On Yomi exists, then set offset_y to 0, meaning, draw on the same line.
	if (kanjidict[kdict_index].KunYomi != NULL){
		screen_text[st_index].offset_y = 0;// Next string on the same line.
		screen_text[st_index].color = BLACK;
	}else{
		screen_text[st_index].offset_y = LINE_SPACE_IF_Y;
		y += LINE_SPACE_IF_Y;
		screen_text[st_index].color = GRAY;
	}
	screen_text[st_index].font_type = FONT_LATIN;
	// Save the string.
	screen_text[st_index].text = (char *) malloc((strlen("Kun: ") + 1) * sizeof(char));
	if (screen_text[st_index].text == NULL){
		oslDebug("SetupScreenTextKdic():\nKun Yomi(%d): Memory alloc failed! Aborting.", kdict_index);
		endOSLib();
		return 0;
	}
	//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Alloc OK. Memsetting");
	memset(screen_text[st_index].text, 0, strlen("Kun: ")+1);
	//oslDebug("IFSetupScreenTextBox():\nlength < num_charsx: Copying String");
	strcpy(screen_text[st_index].text, "Kun: \0");

	screen_text[st_index].size = 1.0f;
	screen_text[st_index].shadowColor = WHITE;
	screen_text[st_index].options = INTRAFONT_ALIGN_LEFT;

	st_index++;

	// Now check if we have an On Yomi to print.
	if (kanjidict[kdict_index].KunYomi != NULL){
		oslSetFont(ltn0);
		oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
		x += oslGetStringWidth("Kun: ");
		//oslSetFont(jpn0);
		//oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
		tmp_screen_text.x = x;
		tmp_screen_text.y = y;
		tmp_screen_text.offset_y = LINE_SPACE_IF_Y;
		tmp_screen_text.font_type = FONT_JAPANESE;
		tmp_screen_text.size = 1.0f;
		tmp_screen_text.color = BLACK;
		tmp_screen_text.shadowColor = WHITE;
		tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

		//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
		//rtn = IFSetupScreenTextBox(tmp_screen_text, MAX_PRINT_CHARS_X, 100, (char *)kanji[index].e_descrip, WORD_WRAP);
		rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - x - (kchar_width*5), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, (char *)kanjidict[kdict_index].KunYomi);

		if (rtn == -1){
			// Error allocating memory. Exit.
			oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
			endOSLib();
			return 0;
		}
		y += (rtn+1)*LINE_SPACE_IF_Y;
	}// End if (kanjidict[kdict_index].KunYomi != NULL)


	// ####################
	// Draw English Descrip
	// ####################

	// Now check if we have an On Yomi to print.
	if (kanjidict[kdict_index].e_descrip != NULL){
		x = 0;
		tmp_screen_text.x = x;
		tmp_screen_text.y = y;
		tmp_screen_text.offset_y = LINE_SPACE_IF_Y;
		tmp_screen_text.font_type = FONT_LATIN;
		tmp_screen_text.size = 1.0f;
		tmp_screen_text.color = BLACK;
		tmp_screen_text.shadowColor = WHITE;
		tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

		rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - (kchar_width*3), (MAX_PRINT_LINES * 2) * LINE_SPACE_IF_Y, kanjidict[kdict_index].e_descrip);

		if (rtn == -1){
			// Error allocating memory. Exit.
			oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
			endOSLib();
			return 0;
		}
		y += (rtn+1)*LINE_SPACE_IF_Y;
	}// End if (kanjidict[kdict_index].KunYomi != NULL)

	// Need to save the end?
	st_count = st_index + 1;
	st_index = 0;

	return RTN_OK;
}


// ############################################
// # Used when showing kanji info via popup.
// # Only scroll what comes after the kanji.
// # Unlike other SetupScreenText functions, this
// # one uses a drawbuffer to draw the text to.
// # That is because the background is on colour.
// ############################################
int SetupScreenTextPopup(void){
	int rtn = 0; //, i = 0;
	int x = 0, y = 0;


	// Draw other kanji info here.
	if (kdict_search_index >= 0 && kdict_search_index < MAX_KANJIDICT){

		y = 6;//SHOW_KDICT_TEXT_Y; //Offset for IntraFont
		x = 0;

		oslSetDrawBuffer(drawBuf);
		oslDisableTransparentColor();
		oslClearImage(drawBuf, RGB(255, 230, 172));
		//oslClearImage(drawBuf, RGB(215, 230, 186));// Only for debugging

		// Draw On Yomi (If it exists)
		if (kanjidict[kdict_search_index].OnYomi != NULL){
			x = 0;//SHOW_KDICT_TEXT_X;
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(x, y, "On:  ");
			x += oslGetStringWidth("On:  ");
			oslSetFont(jpn0);
			oslIntraFontSetStyle(jpn0, 1.0f,MAGENTA,WHITE,INTRAFONT_ALIGN_LEFT);
			rtn = IFPrintTextBoxSJIS(x, y, SHOW_KDICT_RECT_WIDTH - SHOW_KDICT_TEXT_INDENT_X - x - (kchar_width*3), MAX_IFTB_HEIGHT, (char *)kanjidict[kdict_search_index].OnYomi);
			y += (rtn+1)*LINE_SPACE_IF_Y;
		}else{
			x = 0;
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,GRAY,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(x, y, "On:  ");
			y += LINE_SPACE_IF_Y;
		}
		// Draw Kun Yomi (If it exists)
		if (kanjidict[kdict_search_index].KunYomi != NULL){
			x = 0;//SHOW_KDICT_TEXT_X;
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(x, y, "Kun: ");
			x += oslGetStringWidth("Kun: ");
			oslSetFont(jpn0);
			oslIntraFontSetStyle(jpn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			rtn = IFPrintTextBoxSJIS(x, y, SHOW_KDICT_RECT_WIDTH - SHOW_KDICT_TEXT_INDENT_X - x - (kchar_width*3), MAX_IFTB_HEIGHT, (char *)kanjidict[kdict_search_index].KunYomi);
			y += (rtn+1)*LINE_SPACE_IF_Y;
		}else{
			x = 0;
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,GRAY,WHITE,INTRAFONT_ALIGN_LEFT);
			oslDrawString(x, y, "Kun: ");
			y += LINE_SPACE_IF_Y;
		}
		
		// Draw English Description
		if (kanjidict[kdict_search_index].e_descrip != NULL){
			oslSetFont(ltn0);
			oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
			rtn = IFPrintTextBox(0/*SHOW_KDICT_TEXT_X*/, y, SHOW_KDICT_RECT_WIDTH - SHOW_KDICT_TEXT_INDENT_X - kchar_width, MAX_IFTB_HEIGHT, kanjidict[kdict_search_index].e_descrip);
			// Need to know where this ends... TODO Store Y somewhere.
			y += (rtn+1)*LINE_SPACE_IF_Y;
			kanji_popup_end_y = y - 6;
		}

		oslSetDrawBuffer(OSL_DEFAULT_BUFFER);

		oslDisableTransparentColor();

		oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);
	}

	return RTN_OK;
}

void ClearScreenKanjiList(void){
	int i = 0;

	for (i = 0; i < MAX_SHOWN_F; i++){
		screenkanji[i].index = -1;
		screenkanji[i].start_x = screenkanji[i].start_y = screenkanji[i].num_chars = 0;
	}
}

int found_compare_length(const void *arg1, const void *arg2){
	int len1 = 0, len2 = 0;

	// First compare purely on string length.
	// Get length of string 1.
	if (kanji[*(int *)arg1].k_kanji != NULL){
		len1 = strlen((char *)kanji[*(int *)arg1].k_kanji);
	}else{
		len1 = strlen((char *)kanji[*(int *)arg1].k_kana);
	}

	// Get length of string 2.
	if (kanji[*(int *)arg2].k_kanji != NULL){
		len2 = strlen((char *)kanji[*(int *)arg2].k_kanji);
	}else{
		len2 = strlen((char *)kanji[*(int *)arg2].k_kana);
	}

	if ((len1 - len2) == 0){
		// If length is identical, do secondary sort on english string length
		len1 = strlen(kanji[*(int *)arg1].e_descrip);
		len2 = strlen(kanji[*(int *)arg2].e_descrip);
		return len1 - len2;
	}else
		return len1 - len2;
}// int found_compare(const void *arg1, const void *arg2);

int found_compare_reading(const void *arg1, const void *arg2){
	return PD_wcscmp((u8 *)kanji[*(int *)arg1].k_kana, (u8 *)kanji[*(int *)arg2].k_kana);//, *(int *)arg1, *(int *)arg2);
}

int found_compare_orig(const void *arg1, const void *arg2){
	// As the find adds entries as they occur from the start
	// of the array, to sort on original order, simply sort
	// on the index value.

	return *(int *)arg1 - *(int *)arg2;
}// int found_compare(const void *arg1, const void *arg2);

int kanjidict_compare_stroke(const void *arg1, const void *arg2){
	ukanjidict *k1, *k2;

	k1 = (ukanjidict *)arg1;
	k2 = (ukanjidict *)arg2;

	if (k1->num_strokes != k2->num_strokes){
		return k1->num_strokes - k2->num_strokes;
	}else{
		// Secondary sort by SJIS kanji order.
		if (k1->kanji[0] != k2->kanji[0])
			return k1->kanji[0] - k2->kanji[0];
		else
			return k1->kanji[1] - k2->kanji[1];
	}
}// int kanjidict_compare_stroke(const void *arg1, const void *arg2);

int kanjidict_compare_orig(const void *arg1, const void *arg2){
	ukanjidict *k1, *k2;

	k1 = (ukanjidict *)arg1;
	k2 = (ukanjidict *)arg2;

	// Sort it out by kanji order.
	if (k1->kanji[0] != k2->kanji[0])
		return k1->kanji[0] - k2->kanji[0];
	else
		return k1->kanji[1] - k2->kanji[1];
}// int kanjidict_compare_orig(const void *arg1, const void *arg2);

// #######################################################
// I know this is bogus, but I couldn't get the vanilla
// wcscmp to work the way I needed, and an internet search
// turned up nothing, so here's what I got. Note that this
// should sort Katakana before Hiragana.
// #######################################################
int PD_wcscmp(const u8* cs, u8* ct){
	int is_ascii = FALSE, is_ascii2 = FALSE;
	u8 c, c2;

	while(*cs && *ct){
		is_ascii = is_ascii2 = FALSE;
		c = *cs;
		c2 = *ct;
		// Check if it is an ASCII char
		if (c < 0x7F)
			is_ascii = TRUE;
		if (c2 < 0x7F)
			is_ascii2 = TRUE;
		//oslDebug("num1 = %d, num2 = %d\nis_ascii = %d, is_ascii2 = %d\nc = %c(0x%x), c2 = %c(0x%x)", num1, num2, is_ascii, is_ascii2, c, c, c2, c2);
		// Can we return already?
		if (is_ascii == TRUE && is_ascii2 == FALSE)
			return -1;
		if (is_ascii == FALSE && is_ascii2 == TRUE)
			return 1;
		// Both are ascii or both SJIS
		if (is_ascii){
			if (c == c2){
				cs++;
				ct++;
			}else
				return (c - c2);
		}else{
			// both non-ascii chars
			if (c == c2){
				c = *(cs+1);
				c2 = *(ct+1);
				if (c == c2){
					cs+=2;
					ct+=2;
				}else
					return (c - c2);
			}else{
				// As Katakana comes after Hiragana in the SJIS table.
				// Need to account for this to sort Katakana before Hiragana.
				//                      Katakana                                      Hiragana
				if ((c == 0x83 && (*(cs+1) >= 0x40 && *(cs+1) <= 0x96)) && (c2 == 0x82 && (*(ct+1) >= 0x9F && *(ct+1) <= 0xF1)))
					return -1;
				else{
					//                      Hiragana                                      Katakana
					if ((c == 0x82 && (*(cs+1) >= 0x9F && *(cs+1) <= 0xF1)) && (c2 == 0x83 && (*(ct+1) >= 0x40 && *(ct+1) <= 0x96)))
						return 1;
					else
						return (c - c2);
				}
			}
		}
	}
	// Checked all the way to the end of one or both strings.
	if (*cs == *ct)
		return 0;
	if (!*cs)
		return -1;
	else
		return 1;
	
}// int PD_wcscmp(const u8* cs, u8* ct);

int SetupScreenGSHELP(void){
	int i = 0, rtn = 0, x = 0, y = 0;
	//int tmpnum = 0;

	//oslDebug("SetupScreenTextEdict():\nClearing screen_text array.\nkchar_width = %d.\nindex = %d.", kchar_width, index);
	// First, clear the screen_text array
	for (i = 0; i < MAX_SCREEN_TEXT; i++){
		screen_text[i].x = 0;
		screen_text[i].y = 0;
		screen_text[i].offset_y = LINE_SPACE_IF_Y;
		screen_text[i].font_type = FONT_LATIN;
		if (screen_text[i].text != NULL){
			free(screen_text[i].text);
			screen_text[i].text = NULL;
		}
		screen_text[i].size = 1.0f;
		screen_text[i].color = BLACK;
		screen_text[i].shadowColor = WHITE;
		screen_text[i].options = INTRAFONT_ALIGN_LEFT;
	}

	st_index = st_count = 0;
	st_can_scroll = TRUE;

	y = x = 0;

	// Draw English Description
	tmp_screen_text.x = x;
	tmp_screen_text.y = y;
	tmp_screen_text.font_type = FONT_LATIN;
	tmp_screen_text.size = 0.7f;
	tmp_screen_text.color = BLACK;
	tmp_screen_text.shadowColor = WHITE;
	tmp_screen_text.options = INTRAFONT_ALIGN_LEFT;

	//char help_text[] = "Controls\n- START - Access the Menu\n- CROSS - Make a selection (Menu or List)\n- CIRCLE - Cancel or Return\n- TRIANGLE - Enter English text with On Screen Keyboard\n- SQUARE - Enter Japanese text with OSK (kana only)\n- L - Page Up or Prev Entry if in selection mode\n- R - Page Down or Next Entry if in selection mode\n- Up - Prev List Item/Scroll Up\n- Down - Next List Item/Scroll Down\n- Left - Prev Study List\n- Right - Next Study List\n \nAbout Searches/Sorts\nYou can search in English or Japanese.\nJapanese OSK can be used for Hiragana/Katakana only.\nKanji can be searched from the\nKanji Popup (hover the cursor over\na Kanji to bring up extra info)\nor from a selection of Kanjidic.\nWhen searching by English, you can\nplace an \'*\'at the end of the string\nto serve as a wildcard.\0";
	char help_text[] = "Controls\n- START - Access the Menu\n- CROSS - Make a selection (Menu or List)\n- CIRCLE - Cancel or Return\n- TRIANGLE - Enter English text with On Screen Keyboard\n- SQUARE - Enter Japanese text with OSK (kana only)\n- L - Page Up or Prev Entry if in selection mode\n- R - Page Down or Next Entry if in selection mode\n- Up - Prev List Item/Scroll Up\n- Down - Next List Item/Scroll Down\n- Left - Prev Study List\n- Right - Next Study List\n- SELECT Randomize Study List\n \nAbout Searches/Sorts\nYou can search in English or Japanese.\nJapanese OSK can be used for Hiragana/Katakana only.\nKanji can be searched from the Kanji Popup (hover the cursor over a Kanji to bring up extra info) or from a selection of Kanjidic.\nWhen searching in English, you can place an \'*\' at the end of the string to serve as a wildcard.\nSKIP code and stroke count search is possible through English searches. Begin the search with # and then write the number of strokes or SKIP code. E.g. #1 or #1 1 1. You may also combine these both by writing #1 0 0 5. This searches all kanjis that has a pattern of 1 and a stroke count of 5. In this mode 0 means anything.\n\nSee the readme.txt for further information.\nThank you.\0";

	//oslDebug("SetupScreenTextEdict():\nBefore calling IFSetupScreenTextBox for e_descrip.");
	rtn = IFSetupScreenTextBox(tmp_screen_text, AA_SCREEN_WIDTH - HELP_X - (kchar_width*2), (MAX_PRINT_LINES * 4) * LINE_SPACE_IF_Y, help_text);

	if (rtn == -1){
		// Error allocating memory. Exit.
		oslDebug("SetupScreenTextEdict Error: Memory Allocation Failed. Aborting");
		endOSLib();
		return 0;
	}

	// Need to save the end?
	st_count = st_index; // st_index should have already been increased ready for the next input.// + 1;
	st_index = 0;

	//oslDebug("SetupScreenGSHELP():\nLeaving Function.\nst_count = %d", st_count);

	return RTN_OK;

}// int SetupScreenGSHELP(void);

int DrawScreenTextHelp(void){
	int i = 0, rtn = 0;
	int y = 0, y2 = 0;

	oslClearScreen(RGB(250, 250, 250));
	DrawTitle(TITLE_SCREENPOS_X, 0);

	y = HELP_Y + (LINE_SPACE_IF_Y);
	rtn = 0;

	// Got the bottom of the drawing stuff.
	y2 = y;

	// Now we know y after printing Kanji and Kana,
	// Check if we need to draw scrollbars.
	if (((st_count) * LINE_SPACE_IF_Y) > (MAX_ST_HEIGHT - y))
		DrawScrollbars(SCROLLBAR_X, SCROLLBAR_Y, SCROLLBAR_HEIGHT, SCROLLBAR_BIG, st_index, st_count - ((MAX_ST_HEIGHT - y)/LINE_SPACE_IF_Y)+(y/LINE_SPACE_IF_Y));

	// Now print vertically scrollable english text.
	st_can_scroll = TRUE;
	for (i = st_index; i < MAX_SCREEN_TEXT; i++){
		// Check we got something to draw
		if (screen_text[i].text == NULL){
			// No more to draw
			st_can_scroll = FALSE;
			break;
		}

		if (y > (MAX_ST_HEIGHT - y2)){
			break;
		}

		oslSetFont(screen_text[i].font_type == FONT_JAPANESE? jpn0 : ltn0);
		oslIntraFontSetStyle(screen_text[i].font_type == FONT_JAPANESE? jpn0 : ltn0, screen_text[i].size, screen_text[i].color, screen_text[i].shadowColor, screen_text[i].options);
		oslDrawString(5 + screen_text[i].x, y, screen_text[i].text);
		y += LINE_SPACE_IF_Y;
	}// End for (i = 0; i < MAX_SCREEN_TEXT; i++)

	// Draw Search String
	oslSetFont(ltn0);
	oslIntraFontSetStyle(ltn0, 0.8f,BLUE,WHITE,INTRAFONT_ALIGN_LEFT);
	oslDrawStringf(HELP_X, AA_SCREEN_HEIGHT - 20, "( ) - Return");
	oslIntraFontSetStyle(ltn0, 1.0f,BLACK,WHITE,INTRAFONT_ALIGN_LEFT);

	return RTN_OK;

}// int DrawScreenTextHelp(void);

// Ini File stuff
// Done at startup
int INI_Check(void){
	int rtn = 0;

	if (CheckFileExists(fIniName) == FALSE){
		// Create it
		rtn = INI_Create();
		if (rtn != RTN_OK)
			return rtn;
		else// Don't need to 'load' the file, as we have already set the defaults.
			return RTN_OK;
	}

	// We have a file to load.
	rtn = INI_Load();
	if (rtn != RTN_OK)
		return rtn;

	return RTN_OK;
}

void INI_SetDefaults(void){
	search_type = OPT_SEARCH_ANY;
	match_whole = FALSE;
	sort_type = OPT_SORT_ORIG;
}

int INI_Load(void){
	FILE *fIni = NULL;
	char buff[512] = "\0";
	char tmpopt[64] = "\0";
	int opt_val = 0;
	int rtn = 0;

	if( (fIni = fopen( fIniName, "r" )) == NULL ){
		oslDebug( "\n\nThe file \"%s\"\n could not be read. Unable to load preferences.\n", fIniName );
		return 0;
	}

	INI_SetDefaults();// Incase of missing options or invalid/old ini file.

	while (!feof(fIni)){
		memset(buff, 0, sizeof(buff));
		memset(tmpopt, 0, sizeof(tmpopt));
		if (fgets(buff, 512, fIni) == NULL){
			if (feof(fIni)){
				fclose(fIni);
				return RTN_OK;
			}else{// Error
				fclose(fIni);
				return 0;
			}
		}
		// Check for comment statement.
		if (buff[0] == '#')
			continue;

		rtn = sscanf(buff, "%64s %d\n", tmpopt, &opt_val);

		if (rtn == 2){
			// Save the new options.
			if (stricmp(tmpopt, "search_type") == 0){
				if (opt_val > OPT_SEARCH_ANY || opt_val < OPT_SEARCH_START)
					search_type = OPT_SEARCH_ANY;
				else
					search_type = opt_val;
				continue;
			}
			if (stricmp(tmpopt, "match_whole") == 0){
				if (opt_val == 0)
					match_whole = FALSE;
				else
					match_whole = TRUE;
				continue;
			}
			if (stricmp(tmpopt, "sort_type") == 0){
				if (opt_val > OPT_SORT_READING || opt_val < OPT_SORT_ORIG)
					sort_type = OPT_SORT_ORIG;
				else
					sort_type = opt_val;
				continue;
			}
			if (stricmp(tmpopt, "show_examples") == 0){
				if (opt_val == 0)
					show_examples = FALSE;
				else
					show_examples = TRUE;
				continue;
			}
			
			// Added by Batanen
			if (stricmp(tmpopt, "kanjiHiding") == 0){
				if (opt_val == 0)
					kanjiHiding = FALSE;
				else
					kanjiHiding = TRUE;
				continue;
			}
			
			if (stricmp(tmpopt, "kanaHiding") == 0){
				if (opt_val == 0)
					kanaHiding = FALSE;
				else
					kanaHiding = TRUE;
				continue;
			}
			
			if (stricmp(tmpopt, "descHiding") == 0){
				if (opt_val == 0)
					descHiding = FALSE;
				else
					descHiding = TRUE;
				continue;
			}
			
			// Just ingnore if we got here.
		}
	}// End while (!feof(fIni))
	fclose(fIni);
	return RTN_OK;
}// int LoadINI(void);

int INI_Create(void){
	int rtn = 0;

	INI_SetDefaults();

	rtn = INI_Save();
	if (rtn != RTN_OK){
		return rtn;
	}

	return RTN_OK;
}

int INI_Save(void){
	FILE *fIni = NULL;

	if( (fIni = fopen( fIniName, "w" )) == NULL ){
		oslDebug( "\n\nThe file \"%s\"\n could not be created. Unable to save preferences.\n", fIniName );
		return 0;
	}

	fprintf(fIni, "#This is the ini file for PSPedict. Please do not\n#edit this file manually.\n");
	fprintf(fIni, "search_type %d\nmatch_whole %d\nsort_type %d\nshow_examples %d\nkanjiHiding %d\nkanaHiding %d\ndescHiding %d\n", search_type, match_whole, sort_type, show_examples, kanjiHiding, kanaHiding, descHiding);
	fclose(fIni);

	return RTN_OK;
}

// Game Status depth functions.
void SetBaseGS(int tmp_game_status){
	int i = 0;

	if (tmp_game_status == GS_SHOW_KANJIDICT || tmp_game_status == GS_PLAYING){
		gs_level[GS_LEVEL_01] = tmp_game_status;
		game_status = prev_game_status = gs_level[GS_LEVEL_01];
		for (i = GS_LEVEL_02; i < MAX_LEVEL; i++)
			gs_level[i] = -1;
	}
	cur_gs_level = GS_LEVEL_01;
}

void SetGS(int level, int tmp_game_status, int clear_above){
	int i = 0;

	if (level < 0 || level >= MAX_LEVEL){
		return;
	}

	if (level == GS_LEVEL_01)
		SetBaseGS(tmp_game_status);


	gs_level[level] = tmp_game_status;
	game_status = tmp_game_status;
	prev_game_status = GetPrevGS(level);

	if(clear_above != 0 && level < (MAX_LEVEL - 1)){
		for (i = level+1; i < MAX_LEVEL; i++)
			gs_level[i] = -1;
	}

	cur_gs_level = GetCurGSLevel();

}

void ReturnToPrevGS(void){
	int i = 0;

	// Handle special case first.
	if (cur_gs_level == GS_LEVEL_01){
		SetBaseGS(gs_level[GS_LEVEL_01]);
		//game_status = prev_game_status = gs_level[GS_LEVEL_01];
		//for (i = GS_LEVEL_02; i < MAX_LEVEL; i++)
		//	gs_level[i] = -1;
		return;
	}

	game_status = GetPrevGS(cur_gs_level);
	cur_gs_level = GetPrevGSLevel(cur_gs_level);
	prev_game_status = GetPrevGS(cur_gs_level);
	//cur_gs_level = GetCurGSLevel();

	// Reset all above
	for (i = cur_gs_level + 1; i < MAX_LEVEL; i++)
		gs_level[i] = -1;
	return;
}

// Returns the actual game status
int GetPrevGS(int start_gs_level){
	int i = 0;

	if (start_gs_level < 0 || start_gs_level >= MAX_LEVEL || start_gs_level == GS_LEVEL_01)
		return gs_level[GS_LEVEL_01];

	for (i = start_gs_level-1; i >= 0; i--){
		if (gs_level[i] != -1)
			return gs_level[i];
	}

	return gs_level[GS_LEVEL_01]; // Use as a fallback?
}

// Returns the level of the current game status
int GetCurGSLevel(void){
	int i = 0;

	for (i = MAX_LEVEL-1; i >= 0; i--){
		if (gs_level[i] != -1)
			return i;
	}

	return 0; // Fallback
}

// Returns the level of the prev game status
int GetPrevGSLevel(int start_gs_level){
	int i = 0;

	if (start_gs_level < 0 || start_gs_level >= MAX_LEVEL || start_gs_level == GS_LEVEL_01)
		return GS_LEVEL_01;

	for (i = start_gs_level-1; i >= 0; i--){
		if (gs_level[i] != -1)
			return i;
	}

	return 0; // Fallback
}

int HasKanji(u8 *string){
	int length;
	int i = 0;

	length = strlen((char *)string);

	for (i = 0; i < length;){
		if ((u8)(string[i]) <= 0x7E){
			i++;
			continue;// ASCII or halfwidth Katakana
		}

		if ((u8)(string[i]) >= 0x88){
			// Is a kanji
			return TRUE;
		}

		i += 2;
	}

	return FALSE;
}// int HasKanji(u8 *string);

